package com.bocloud.ams.booter.controller.audit;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.bocloud.ams.entity.audit.OperationalAudit;
import com.bocloud.ams.entity.audit.OperationalAuditBak;
import com.bocloud.ams.service.audit.OperationalAuditService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.utils.Common;

import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;

@RestController
@RequestMapping("/v1/operational/audit")
@Tag(name = "操作审计管理")
public class OperationalAuditController {

    @Autowired
    private OperationalAuditService operationalAuditService;

    /**
     * 资源变更记录
     *
     * @param pager
     * @return
     */
    @Operation(tags = {"CMC"}, summary = "操作审计列表")
    @GetMapping
    public GeneralResult<GridBean<OperationalAudit>> operationalAudit(Pager pager,
                                                                      @RequestParam(value = "positionKey", required = false) String positionKey) {
        return operationalAuditService.list(pager, positionKey);
    }

    /**
     * 资源变更记录详情
     *
     * @param id
     * @return
     */
    @Operation(tags = {"CMC"}, summary = "操作审计详情")
    @GetMapping(value = "/{id}")
    public GeneralResult<OperationalAudit> operationalAudit(@PathVariable(value = Common.ID) Long id,
                                                            @RequestParam(value = "positionKey", required = false) String positionKey) {
        return operationalAuditService.detail(id, positionKey);
    }

    @Operation(tags = {"CMC"}, summary = "操作审计备份列表")
    @GetMapping(value = "/bak")
    public GeneralResult<GridBean<OperationalAuditBak>> operationalAuditBak(Pager pager,
                                                                            @RequestParam(value = "positionKey", required = false) String positionKey) {
        return operationalAuditService.bakList(pager, positionKey);
    }
}
