package com.bocloud.ams.booter.controller.collection;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.bocloud.ams.entity.collection.CollectionTaskInstance;
import com.bocloud.ams.service.collection.CollectionTaskInstanceService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.utils.Common;

import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;

@RestController
@RequestMapping("/v1/collection/tasks/instances")
@Tag(name = "采集任务实例管理")
public class CollectionTaskInstanceController {

    @Autowired
    private CollectionTaskInstanceService collectionTaskInstanceService;

    /**
     * 采集任务实例列表
     *
     * @param pager
     * @return
     */
    @Operation(tags = {"CMC"}, summary = "采集任务实例列表")
    @GetMapping
    public GeneralResult<GridBean<CollectionTaskInstance>> list(Pager pager,
                                                                @RequestParam(value = "positionKey", required = false) String positionKey) {
        return collectionTaskInstanceService.list(pager, positionKey);
    }

    @GetMapping(value = "/{id}")
    @Operation(tags = {"CMC", "CSC"}, summary = "采集任务实例详情")
    public GeneralResult<CollectionTaskInstance> detail(@PathVariable(value = Common.ID) Long id,
                                                        @RequestParam(value = "positionKey", required = false) String positionKey) {
        return collectionTaskInstanceService.detail(id, positionKey);
    }

    @Operation(tags = {"CMC"}, summary = "采集任务实例备份列表")
    @GetMapping("/backup")
    public GeneralResult<GridBean<CollectionTaskInstance>> bakList(Pager pager,
                                                                   @RequestParam(value = "positionKey", required = false) String positionKey) {
        return collectionTaskInstanceService.bakList(pager, positionKey);
    }

    @GetMapping(value = "/{name}/{id}")
    @Operation(tags = {"CMC", "CSC"}, summary = "采集任务实例备份详情")
    public GeneralResult<CollectionTaskInstance> bakDetail(@PathVariable(value = Common.ID) Long id,
                                                           @PathVariable(value = Common.NAME) String name,
                                                           @RequestParam(value = "positionKey", required = false) String positionKey) {
        return collectionTaskInstanceService.bakDetail(id, name, positionKey);
    }
}
