package com.bocloud.ams.booter.controller.instance;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.bocloud.ams.entity.instance.InstanceAuthority;
import com.bocloud.ams.service.instance.InstanceAuthorityService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;

import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;

@RestController
@RequestMapping("/v1/instances/authority")
@Tag(name = "实例权限管理")
public class InstanceAuthoryController {

    @Autowired
    private InstanceAuthorityService instanceAuthorityService;

    /**
     * 实例权限列表
     *
     * @return
     */
    @Operation(tags = {"CMC"}, summary = "实例权限列表")
    @GetMapping
    public GeneralResult<GridBean<InstanceAuthority>> list(Pager pager,
                                                           @RequestParam(value = "positionKey", required = false) String positionKey) {
        return instanceAuthorityService.list(pager, positionKey);
    }

    /***
     * 新增实例权限
     *
     * @param authority
     * @param context
     * @return
     */
    @Operation(tags = {"CMC"}, summary = "实例权限新增")
    @PostMapping
    public GeneralResult<Void> save(@RequestBody InstanceAuthority authority,
        @Value(Common.REQ_CONTEXT) RequestContext context) {
        return instanceAuthorityService.create(authority, context.getTarget());
    }

    @Operation(tags = {"CMC"}, summary = "实例权限修改")
    @PutMapping(value = "/{id}")
    public GeneralResult<Void> modify(@RequestBody InstanceAuthority authority,
        @Value(Common.REQ_CONTEXT) RequestContext context) {
        return instanceAuthorityService.modify(authority, context.getTarget());
    }

    /**
     * 删除实例权限
     *
     * @return
     */
    @Operation(tags = {"CMC"}, summary = "实例权限删除")
    @DeleteMapping(value = "/{id}")
    public GeneralResult<Void> remove(@PathVariable(value = Common.ID) Long id,
        @Value(Common.REQ_CONTEXT) RequestContext context) {
        return instanceAuthorityService.remove(id, context.getTarget());
    }

    /**
     * 批量删除实例权限
     *
     * @return
     */
    @Operation(tags = {"CMC"}, summary = "实例权限批量删除")
    @DeleteMapping(value = "/byrole")
    public GeneralResult<Void> removeByRole(@RequestBody InstanceAuthority authority,
        @Value(Common.REQ_CONTEXT) RequestContext context) {
        return instanceAuthorityService.removeByRole(authority, context.getTarget());
    }

}
