package com.bocloud.ams.booter.controller.instance;

import com.bocloud.ams.entity.instance.InstanceRelationHistoryRecord;
import com.bocloud.ams.entity.model.InstanceRelationHistoryPreflightModel;
import com.bocloud.ams.service.instance.InstanceRelationHistoryRecordService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

@RestController
@RequestMapping("/v1/instances/relations/history")
@Tag(name = "关联历史管理")
public class InstanceRelationHistoryRecordController {

    @Autowired
    private InstanceRelationHistoryRecordService instanceRelationHistoryRecordService;

    /**
     * 关联历史列表
     *
     * @param pager
     * @param context
     * @return
     */
    @GetMapping
    @Operation(tags = {"CMC", "CSC"}, summary = "关联历史列表")
    public GeneralResult<GridBean<InstanceRelationHistoryRecord>> list(Pager pager,
                                                                       @Value(Common.REQ_CONTEXT) RequestContext context,
                                                                       @RequestParam(value = "positionKey", required = false) String positionKey) {
        return instanceRelationHistoryRecordService.list(pager, positionKey);
    }

    @PutMapping("/{id}/preflight")
    @Operation(tags = {"CMC", "CSC"}, summary = "关联历史预检")
    public GeneralResult<InstanceRelationHistoryPreflightModel> preflight(@PathVariable(value = Common.ID) Long id,
        @Value(Common.REQ_CONTEXT) RequestContext context) {
        return instanceRelationHistoryRecordService.preflight(id, context);

    }

    @PutMapping("/{id}/reconcile")
    @Operation(tags = {"CMC", "CSC"}, summary = "关联历史调和")
    public GeneralResult reconcile(@PathVariable(value = Common.ID) Long id,
        @Value(Common.REQ_CONTEXT) RequestContext context) {
        return instanceRelationHistoryRecordService.reconcile(id, context);

    }

    @PutMapping("/{id}/cancel")
    @Operation(tags = {"CMC", "CSC"}, summary = "关联历史取消调和")
    public GeneralResult cancel(@PathVariable(value = Common.ID) Long id,
        @Value(Common.REQ_CONTEXT) RequestContext context) {
        return instanceRelationHistoryRecordService.cancel(id, context);

    }

    @Operation(tags = {"CMC"}, summary = "关联历史批量取消调和")
    @PutMapping(value = "/batch/cancel")
    public GeneralResult<Void> batchCancel(@RequestBody List<Long> ids,
        @Value(Common.REQ_CONTEXT) RequestContext context) {
        return instanceRelationHistoryRecordService.batchCancel(ids, context);

    }

}
