package com.bocloud.ams.booter.controller.module;

import com.bocloud.ams.entity.module.ModuleGroup;
import com.bocloud.ams.service.module.ModuleGroupService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

@Tag(name = "模型分组管理")
@RestController
@RequestMapping("/v1/module/group")
public class ModuleGroupController {
    @Autowired
    private ModuleGroupService moduleGroupService;

    /**
     * 模型分组采集结构查询
     *
     * @param category
     * @return
     */
    @GetMapping(value = "/collections")
    @Operation(tags = {"CMC"}, summary = "模型分组采集结构查询")
    public GeneralResult<List<ModuleGroup>> collections(@RequestParam(value = "category") String category,
                                                        @RequestParam(value = "moduleName", required = false) String moduleName,
                                                        @RequestParam(value = "positionKey", required = false) String positionKey) {
        return moduleGroupService.collections(category, moduleName, positionKey);
    }

    /**
     * 模型分组根节点结构查询
     *
     * @return
     */
    @GetMapping(value = "/roots")
    @Operation(tags = {"CMC"}, summary = "模型分组根节点结构查询")
    public GeneralResult<List<ModuleGroup>> queryRoot(@RequestParam(value = "positionKey", required = false) String positionKey) {
        return moduleGroupService.queryRoot(positionKey);
    }

    /**
     * 当前根节点的子分组查询
     *
     * @return
     */
    @GetMapping
    @Operation(tags = {"CMC"}, summary = "模型分组所有分组节点查询")
    public GeneralResult<List<ModuleGroup>> queryRootChildren(@RequestParam(value = "positionKey", required = false) String positionKey) {
        return moduleGroupService.queryRootChildren(positionKey);
    }

    /**
     * 模型分组子级数据(下级节点和模型数据)查询
     *
     * @param groupId
     * @return
     */
    @GetMapping(value = "/{groupId}/children")
    @Operation(tags = {"CMC"}, summary = "模型分组子级数据(下级节点和模型数据)查询")
    public GeneralResult<ModuleGroup> queryChildren(@PathVariable(value = "groupId") Long groupId,
                                                    @RequestParam(value = "menu", required = false) String menu,
                                                    @Value(Common.REQ_CONTEXT) RequestContext context,
                                                    @RequestParam(value = "positionKey", required = false) String positionKey) {
        return moduleGroupService.queryChildren(groupId, menu, context, positionKey);
    }

    /**
     * 模型分组查询明细
     *
     * @param id
     * @return
     */
    @GetMapping(value = "/{id}")
    @Operation(tags = {"CMC"}, summary = "模型分组详情")
    public GeneralResult<ModuleGroup> detail(@PathVariable(value = Common.ID) Long id,
                                             @RequestParam(value = "positionKey", required = false) String positionKey) {
        return moduleGroupService.detail(id, positionKey);
    }

    /**
     * 新增模型分组
     *
     * @param group
     * @param context
     * @return
     */
    @PostMapping
    @Operation(tags = {"CMC"}, summary = "模型分组新增")
    public GeneralResult<Void> save(@RequestBody ModuleGroup group, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return moduleGroupService.save(group, context.getTarget());
    }

    /**
     * 更新模型分组
     *
     * @param group
     * @param context
     * @return
     */
    @PutMapping
    @Operation(tags = {"CMC"}, summary = "模型分组修改")
    public GeneralResult<Void> modify(@RequestBody ModuleGroup group,
        @Value(Common.REQ_CONTEXT) RequestContext context) {
        return moduleGroupService.modify(group, context.getTarget());
    }

    /**
     * 删除模型分组
     *
     * @param id
     * @return
     */
    @DeleteMapping(value = "/{id}")
    @Operation(tags = {"CMC"}, summary = "模型分组删除")
    public GeneralResult<Void> remove(@PathVariable(value = Common.ID) Long id,
        @Value(Common.REQ_CONTEXT) RequestContext context) {
        return moduleGroupService.remove(id, context.getTarget());
    }

    /**
     * 新查询模型分组
     *
     * @return
     */
    @GetMapping(value = "/list")
    @Operation(tags = {"CMC"}, summary = "模型分组查询")
    public GeneralResult<List<ModuleGroup>> querylist(@RequestParam(value = "positionKey", required = false) String positionKey) {
        return moduleGroupService.querylist(positionKey);
    }

    /**
     * 模型分组子级及父级数据(下级节点和模型数据)查询
     *
     * @param groupId
     * @return
     */
    @GetMapping(value = "/{groupId}/root/children")
    @Operation(tags = {"CMC"}, summary = "模型分组子级及父级数据(下级节点和模型数据)查询")
    public GeneralResult<List<ModuleGroup>> queryRootChildren(@PathVariable(value = "groupId") Long groupId,
                                                              @RequestParam(value = "menu", required = false) String menu,
                                                              @Value(Common.REQ_CONTEXT) RequestContext context,
                                                              @RequestParam(value = "positionKey", required = false) String positionKey) {
        return moduleGroupService.queryRootChildren(groupId, menu, context, positionKey);
    }

}
