package com.bocloud.ams.booter.controller.module;

import com.bocloud.ams.entity.module.ModuleRelation;
import com.bocloud.ams.entity.module.RelationCategory;
import com.bocloud.ams.service.module.RelationCategoryService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

@Tag(name =  "关联类型分组管理")
@RestController
@RequestMapping("/v1/relation/category")
public class RelationCategoryController {

    @Autowired
    private RelationCategoryService relationCategoryService;

    /**
     * 查询关联类型分组记录
     *
     * @param pager
     * @return
     */
    @GetMapping
    @Operation(tags = {"CMC", "CSC"}, summary ="关联类型分组列表")
    public GeneralResult<GridBean<RelationCategory>> list(Pager pager,
                                                          @RequestParam(value = "positionKey", required = false) String positionKey) {
        return relationCategoryService.list(pager, positionKey);
    }

    @GetMapping("/all")
    @Operation(tags = {"CMC", "CSC"}, summary ="关联类型分组列表（所有）")
    public GeneralResult<List<RelationCategory>> listAll(Pager pager,
                                                         @RequestParam(value = "positionKey", required = false) String positionKey) {
        return relationCategoryService.listAll(pager.getParams(), pager.getSorter(), positionKey);
    }

    /**
     * 新增关联类型分组
     *
     * @param record
     * @param context
     * @return
     */
    @PostMapping
    @Operation(tags = {"CMC", "CSC"}, summary ="关联类型分组新增")
    public GeneralResult<Void> save(@RequestBody RelationCategory record,
        @Value(Common.REQ_CONTEXT) RequestContext context) {

        if (!paramsCheck(record)) {
            return new GeneralResult<>(false, "参数异常");
        }
        try {
            return relationCategoryService.create(record, context.getTarget());
        } catch (Exception e) {
            return new GeneralResult<>(false, "关联类型分组新增失败");
        }
    }

    /**
     * 修改关联类型分组
     *
     * @param record
     * @param context
     * @return
     */
    @PutMapping(value = "/{id}")
    @Operation(tags = {"CMC", "CSC"}, summary ="关联类型分组修改")
    public GeneralResult modify(@RequestBody RelationCategory record, @PathVariable(value = Common.ID) Long id,
        @Value(Common.REQ_CONTEXT) RequestContext context) {

        if (!paramsCheck(record)) {
            return new GeneralResult<>(false, "参数异常");
        }
        record.setId(id);

        return relationCategoryService.modify(record, context.getTarget());
    }

    /**
     * 移除关联类型分组
     *
     * @param id
     * @param context
     * @return
     */
    @DeleteMapping("/{id}")
    @Operation(tags = {"CMC", "CSC"}, summary ="关联类型分组删除")
    public GeneralResult remove(@PathVariable(value = Common.ID) Long id,
        @Value(Common.REQ_CONTEXT) RequestContext context) {
        try {
            return relationCategoryService.remove(id, context.getTarget());
        } catch (Exception e) {
            return new GeneralResult<>(false, "关联类型分组删除失败");
        }
    }

    private boolean paramsCheck(RelationCategory record) {
        // 参数校验
        if (null == record || StringUtils.isEmpty(record.getName()) || StringUtils.isEmpty(record.getCode())) {
            return false;
        }
        return true;
    }

    @GetMapping(value = "/{id}/module/relations")
    @Operation(tags = {"CMC"}, summary ="关联类型分组模型关系查询")
    public GeneralResult<List<ModuleRelation>> moduleRelationList(@PathVariable(value = Common.ID) Long id,
                                                                  @RequestParam(value = "moduleId") Long moduleId,
                                                                  @RequestParam(value = "positionKey", required = false) String positionKey) {
        return relationCategoryService.moduleRelationList(id, moduleId, positionKey);
    }

}
