package com.bocloud.ams.booter.controller.module;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.bocloud.ams.entity.module.Relation;
import com.bocloud.ams.service.module.RelationService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;

import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;

@Tag(name =  "关联类型管理")
@RestController
@RequestMapping("/v1/relation")
public class RelationController {

    @Autowired
    private RelationService relationService;

    /**
     * 查询关联类型记录
     *
     * @param pager
     * @param context
     * @return
     */
    @GetMapping
    @Operation(tags = {"CMC", "CSC"}, summary ="关联类型列表")
    public GeneralResult<GridBean<Relation>> list(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context,
                                                  @RequestParam(value = "positionKey", required = false) String positionKey) {
        return relationService.list(pager, positionKey);
    }

    /**
     * 新增关联类型
     *
     * @param record
     * @param context
     * @return
     */
    @PostMapping
    @Operation(tags = {"CMC", "CSC"}, summary ="关联类型新增")
    public GeneralResult<Void> save(@RequestBody Relation record, @Value(Common.REQ_CONTEXT) RequestContext context) {

        if (!paramsCheck(record)) {
            return new GeneralResult<>(false, "参数异常");
        }
        return relationService.create(record, context.getTarget());
    }

    /**
     * 修改关联类型
     *
     * @param record
     * @param context
     * @return
     */
    @PutMapping(value = "/{id}")
    @Operation(tags = {"CMC", "CSC"}, summary ="关联类型修改")
    public GeneralResult modify(@RequestBody Relation record, @PathVariable(value = Common.ID) Long id,
        @Value(Common.REQ_CONTEXT) RequestContext context) {

        if (!paramsCheck(record)) {
            return new GeneralResult<>(false, "参数异常");
        }
        record.setId(id);
        return relationService.modify(record, context.getTarget());
    }

    /**
     * 移除关联类型
     *
     * @param id
     * @param context
     * @return
     */
    @DeleteMapping("/{id}")
    @Operation(tags = {"CMC", "CSC"}, summary ="关联类型删除")
    public GeneralResult remove(@PathVariable(value = Common.ID) Long id,
        @Value(Common.REQ_CONTEXT) RequestContext context) {
        return relationService.remove(id, context.getTarget());
    }

    private boolean paramsCheck(Relation record) {
        // 参数校验
        if (null == record || StringUtils.isEmpty(record.getName()) || StringUtils.isEmpty(record.getCode())
            || null == record.getCategoryId() || 0L == record.getCategoryId()) {
            return false;
        }
        return true;
    }
}
