package com.bocloud.ams.booter.listener;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.ams.entity.collection.CollectionTaskInstance;
import com.bocloud.ams.entity.enums.CollectionEnum;
import com.bocloud.ams.entity.enums.Status;
import com.bocloud.ams.entity.model.TaskInstanceBean;
import com.bocloud.ams.service.collection.CollectionTaskInstanceService;
import com.bocloud.ams.service.utils.CollectionHistoryHelper;
import com.bocloud.ams.task.model.JobResult;
import com.megatron.common.model.GeneralResult;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.core.MessageListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.nio.charset.Charset;
import java.util.Date;

/**
 * @Author liuyuanyuan
 * @Date 2022/6/14 0014 18:31
 * @Description:采集任务结果监听
 */
@Slf4j
@Component("taskResultListener")
public class TaskResultListener implements MessageListener {
    @Autowired
    private CollectionTaskInstanceService cmdbCollectionTaskInstanceService;
    @Autowired
    private CollectionHistoryHelper historyHelper;

    private void processDebugLog(String format, Object... objects) {
        if (log.isDebugEnabled()) {
            log.debug(format, objects);
        }
    }

    @Override
    public void onMessage(Message message) {
        try {
            String content = new String(message.getBody(), Charset.forName("UTF-8"));
            processDebugLog("get bocloud message:{}", content);
            if (StringUtils.isBlank(content)) {
                // 消息没有内容，直接确认收到 淹没处理
                return;
            }
            JobResult jobResult = JSONObject.parseObject(content, JobResult.class);
            if (null == jobResult) {
                // 消息参数格式错误，直接确认收到淹没处理
                return;
            }
            log.info("收到任务执行消息{}", content);
            //查询任务数据
            GeneralResult<CollectionTaskInstance> instanceResult = cmdbCollectionTaskInstanceService.queryByTaskId(jobResult.getJob().getTaskId());
            if (instanceResult.isFailed()) {
                log.info("任务数据不存在，taskId{}", jobResult.getJob().getTaskId());
                return;
            }
            CollectionTaskInstance instance = instanceResult.getData();
            TaskInstanceBean instanceBean = new TaskInstanceBean();
            instanceBean.setOutput(jobResult.getMsg());
            instanceBean.setId(instance.getId());
            instanceBean.setGmtModify(new Date());
            instanceBean.setCollectionFinish(new Date());
            BigDecimal start = new BigDecimal(instance.getGmtStart().getTime());
            BigDecimal end = new BigDecimal(instanceBean.getCollectionFinish().getTime());
            instanceBean.setCollectionTime(Math.abs(end.subtract(start).divide(new BigDecimal(1000), 3, RoundingMode.HALF_UP).doubleValue()));
            if (jobResult.isSuccess()) {
                instanceBean.setStatus(Status.Inst.RUNNING.name());
                instanceBean.setCollectionStatus(Status.Inst.SUCCESS.name());
            } else {
                instanceBean.setCollectionStatus(Status.Inst.FAILED.name());
                instanceBean.setStatus(CollectionEnum.Status.ERROR.name());
                instanceBean.setGmtFinish(new Date());
            }
            cmdbCollectionTaskInstanceService.modify(instanceBean, instance.getCreatorId());
            log.info("采集任务:{}已完成！开始时间:{},结束时间:{},总耗时:{}", jobResult.getJob().getTaskId(), start, end, instanceBean.getCollectionTime());
            if (jobResult.isSuccess()) {
                GeneralResult result = historyHelper.history(instance.getId(), instance.getCode(), jobResult.getMsg(), instance.getCreatorId());
                if (result.isFailed()) {
                    cmdbCollectionTaskInstanceService.updateStatus(instance.getId(), CollectionEnum.Status.ERROR.name());
                }
            }
        } catch (Exception e) {
            log.error("Get error message:{}", e.getMessage());
        }
    }
}
