package com.bocloud.ams.booter.scheduler;


import com.alibaba.excel.util.DateUtils;
import com.bocloud.ams.entity.config.CmdbConfig;
import com.bocloud.ams.entity.TableLog;
import com.bocloud.ams.entity.audit.OperationalAudit;
import com.bocloud.ams.entity.audit.OperationalAuditBak;
import com.bocloud.ams.entity.enums.Status;
import com.bocloud.ams.repository.config.CmdbConfigRepository;
import com.bocloud.ams.repository.TableLogRepository;
import com.bocloud.ams.repository.audit.OperationalAuditBakRepository;
import com.bocloud.ams.repository.audit.OperationalAuditRepository;
import com.bocloud.ams.repository.utils.CmdbUtils;
import com.google.common.collect.Lists;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.Param;
import com.megatron.common.model.Sign;
import com.megatron.common.utils.ListTool;
import com.megatron.common.utils.MapTools;
import com.megatron.framework.core.CurrentService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

@Slf4j
@Component
public class OperationalAuditBakScheduler {

    @Autowired
    private CurrentService service;
    @Autowired
    private OperationalAuditRepository operationalAuditRepository;


    @Autowired
    private OperationalAuditBakRepository operationalAuditBakRepository;
    @Autowired
    private TableLogRepository tableLogRepository;

    @Autowired
    private CmdbConfigRepository cmdbConfigRepository;

    //每月第一天的0时1分开始创建当月的备份表
    @Scheduled(cron = "${operational.audit.create-table:0 1 0 1 * ? }")
    private void createTable() {
        try {
            if (!service.isLeader()) {
                return;
            }
            CmdbConfig switchConfig = cmdbConfigRepository.queryByCode("CmdbOperationalAuditDataBakSwitch");
            if (null == switchConfig || StringUtils.isEmpty(switchConfig.getValue()) || !switchConfig.getValue().equalsIgnoreCase("true")) {
                log.info("备份操作审计未开启");
                return;
            }
            String time = DateUtils.format(new Date(), "yyyyMM");
            //ci_operational_audit_bak_202304
            String tableNameTaskNodeBak = CmdbUtils.CMDB_OPERATIONAL_AUDIT_BAK + time;
            createOperationalAuditBakTable(tableNameTaskNodeBak);
        } catch (Exception e) {
            log.error("定时创建库表失败", e);
        }
    }

    private GeneralResult<Void> createOperationalAuditBakTable(String tableName) {
        String status = Status.Inst.SUCCESS.name();
        try {
            if (operationalAuditBakRepository.countTable(tableName) > 0) {
                log.warn("创建表空间{}失败，表空间已经存在", tableName);
                return new GeneralResult<>(false, "创建失败");
            }
            operationalAuditBakRepository.createTable(tableName, new OperationalAuditBak());
            return new GeneralResult<>(true, "创建成功");
        } catch (Exception e) {
            status = Status.Inst.FAILED.name();
            log.error("create OperationalAuditBak table error:", e);
            return new GeneralResult<>(false, "创建失败");
        } finally {
            TableLog tableLog = new TableLog();
            tableLog.setCode("BACKUP");
            tableLog.setStatus(status);
            tableLog.setName(tableName);
            tableLog.setTemplateTable(CmdbUtils.CMDB_OPERATIONAL_AUDIT_BAK);
            tableLog.setGmtCreate(new Date());
            tableLog.setDeleted(false);
            tableLogRepository.save(tableLog);
        }
    }

    @Scheduled(cron = "${operational.audit.bakup:0 0 4 * * ?}")
    public void operationalAuditDataBak() {
        if (!service.isLeader()) {
            return;
        }
        log.info("开始备份操作审计数据");
        List<OperationalAudit> instances = null;
        List<Param> params = Lists.newArrayList();
        try {
            CmdbConfig switchConfig = cmdbConfigRepository.queryByCode("CmdbOperationalAuditDataBakSwitch");
            if (null == switchConfig || StringUtils.isEmpty(switchConfig.getValue()) || !switchConfig.getValue().equalsIgnoreCase("true")) {
                log.info("备份操作审计未开启");
                return;
            }
            Integer days = 90;
            CmdbConfig config = cmdbConfigRepository.queryByCode("CmdbOperationalAuditDataBakDaysAgo");
            if (null != config && StringUtils.isNotEmpty(config.getValue())) {
                days = Integer.valueOf(config.getValue());
            }
            days = null == days ? 90 : days;
            Calendar calendar = Calendar.getInstance();
            calendar.add(Calendar.DAY_OF_MONTH, -days);
            calendar.set(Calendar.HOUR_OF_DAY, 0);
            calendar.set(Calendar.MINUTE, 0);
            calendar.set(Calendar.SECOND, 0);
            calendar.set(Calendar.MILLISECOND, 0);
            SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            String time = formatter.format(calendar.getTime());
            log.info("备份{}之前的数据", time);
            params.add(new Param(MapTools.simpleMap("gmtCreate", time), Sign.LT));
            //查询备份数据的备份表后缀
            instances = this.operationalAuditRepository.listTime(params);

        } catch (Exception e) {
            log.error("查询操作审计失败", e);
            return;
        }
        if (ListTool.isEmpty(instances)) {
            log.info("结束备份操作审计数据，没有查询到需要备份的数据");
            return;
        }
        //按照时间分组
        Map<String, List<OperationalAudit>> map = new HashMap<>();
        for (OperationalAudit audit : instances) {
            String key = audit.getDateStr().replace("-", "");
            String tableName = CmdbUtils.CMDB_OPERATIONAL_AUDIT_BAK + key;
            GeneralResult result = chargeTableExist(tableName);
            if (result.isSuccess()) {
                batchesBak(audit.getDateStr(), tableName,params);
            }
        }

        log.info("结束备份操作审计数据");
    }

    private GeneralResult<Void> chargeTableExist(String tableName) {
        String status = Status.Inst.SUCCESS.name();
        try {
            if (operationalAuditBakRepository.countTable(tableName) > 0) {
                return new GeneralResult<>(true, "表已经存在");
            }
            operationalAuditBakRepository.createTable(tableName, new OperationalAuditBak());
            return new GeneralResult<>(true, "创建成功");
        } catch (Exception e) {
            status = Status.Inst.FAILED.name();
            log.error("create OperationalAuditBak table error:", e);
            return new GeneralResult<>(false, "创建失败");
        } finally {
            TableLog tableLog = new TableLog();
            tableLog.setCode("BACKUP");
            tableLog.setName(tableName);
            tableLog.setStatus(status);
            tableLog.setTemplateTable("ci_operational_audit");
            tableLog.setGmtCreate(new Date());
            tableLog.setDeleted(false);
            tableLogRepository.save(tableLog);
        }
    }

    private void batchesBak(String dateStr, String tableName,List<Param> params) {
        try {
            List<Param> newParams = Lists.newArrayList();
            newParams.addAll(params);
            newParams.add(new Param(MapTools.simpleMap("dateStr", dateStr), Sign.LK));
            while (true) {
                List<OperationalAudit> instances = operationalAuditRepository.list(newParams, 1000);
                if (ListTool.isEmpty(instances)) {
                    log.info("结束备份任务数据，没有查询到需要备份的数据");
                    break;
                }
                int stepSize = 100;
                int size = instances.size();
                log.info("备份操作审计数{}，每批操作审计数{}", size, stepSize);
                if (size > stepSize) {
                    int num = size % stepSize == 0 ? size / stepSize : size / stepSize + 1;
                    for (int i = 0; i < num; i++) {
                        log.info("备份第{}批数据", i + 1);
                        if (i + 1 == num) {
                            batchBak(instances.subList(i * stepSize, size), tableName);
                        } else {
                            batchBak(instances.subList(i * stepSize, (i + 1) * stepSize), tableName);
                        }
                    }

                } else {
                    batchBak(instances, tableName);
                }
            }

        } catch (Exception e) {
            log.error("备份操作审计异常", e);
        }
    }

    private void batchBak(List<OperationalAudit> instances, String tableName) {
        try {
            List<Long> ids = instances.stream().map(OperationalAudit::getId).collect(Collectors.toList());
            List<OperationalAuditBak> instanceBaks = Lists.newArrayList();
            for (OperationalAudit instance : instances) {
                OperationalAuditBak bak = new OperationalAuditBak();
                BeanUtils.copyProperties(instance, bak);
                instanceBaks.add(bak);
            }
            if (!instanceBaks.isEmpty()) {
                operationalAuditBakRepository.batchSaveOperationalAuditBak(instanceBaks, tableName, instanceBaks.size());
            }
            //删除原表数据
            operationalAuditRepository.delete(ids);

        } catch (Exception e) {
            log.error("操作审计备份异常", e);
        }
    }

}
