package com.bocloud.ams.entity.collection;

import com.bocloud.ams.annotation.ColumnType;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.megatron.common.utils.DateDeserializer;
import com.megatron.common.utils.DateSerializer;
import com.megatron.entity.annotations.*;
import com.megatron.entity.bean.Generic;
import com.megatron.entity.meta.PKStrategy;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;
import org.springframework.format.annotation.DateTimeFormat;
import java.util.Date;

/**
 * @author huliwei
 * @date 2023/4/24
 */
@Data
@NoArgsConstructor
@AllArgsConstructor
@Table("ci_collection_task_instance_bak")
public class CollectionTaskInstanceBak extends Generic {
    @PK(value = PKStrategy.SELF)
    private Long id;
    @Column(value = "uuid",length = 64)
    private String uuid;
    /**
     * 任务类型
     */
    @Column(value = "type",length = 64)
    private String type;
    @Column(value = "code",length = 64)
    private String code;
    @ColumnType(value = "longtext")
    @Column("graph")
    private String graph;
    @Column(value = "cron",length = 64)
    private String cron;
    /**
     * 开始时间
     */
    @Column("gmt_start")
    @JsonSerialize(using = DateSerializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss:SSS")
    private Date gmtStart;
    /**
     * 结束时间
     */
    @Column("gmt_finish")
    @JsonSerialize(using = DateSerializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss:SSS")
    private Date gmtFinish;

    /**
     * 总耗时
     */
    @Column("finish_time")
    private Long finishTime;

    /**
     * 任务ID
     */
    @Column("task_id")
    private Long taskId;
    /**
     * 超时时间
     */
    @Column("timeout")
    private Long timeout;

    /**
     * 耗时
     */
    @ColumnType(value = "double")
    @Column("cost")
    private Double cost;

    /**
     * 输入参数
     */
    @ColumnType(value = "text")
    @Column("input")
    private String input;

    /**
     * 输出参数
     */
    @ColumnType(value = "longtext")
    @Column("output")
    private String output;

    /**
     * 采集结束时间
     */
    @Column("collection_finish")
    @JsonSerialize(using = DateSerializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date collectionFinish;

    /**
     * 采集总耗时
     */
    @ColumnType(value = "double")
    @Column("collection_time")
    private Double collectionTime;

    /**
     * 采集状态
     */
    @Column(value = "collection_status",length = 64)
    private String collectionStatus;

    @IgnoreUpdate
    @Column(
            value = "gmt_create",
            nullable = false
    )
    @JsonSerialize(
            using = DateSerializer.class
    )
    @JsonDeserialize(
            using = DateDeserializer.class
    )
    @DateTimeFormat(
            pattern = "yyyy-MM-dd HH:mm:ss"
    )
    private Date gmtCreate;
    @Column("gmt_modify")
    @JsonSerialize(
            using = DateSerializer.class
    )
    @JsonDeserialize(
            using = DateDeserializer.class
    )
    @DateTimeFormat(
            pattern = "yyyy-MM-dd HH:mm:ss"
    )
    private Date gmtModify;
    @IgnoreUpdate
    @Column(value = "creator_id",nullable = false)
    private Long creatorId;
    @Column("owner_id")
    private Long ownerId;
    @Column("mender_id")
    private Long menderId;
    @Column(value = "name" , length = 256)
    private String name;
    @Column(value = "status" ,length = 16)
    private String status;
    @Column(value = "is_deleted",length = 1,nullable = false)
    private Boolean deleted;
    @Column(value = "props",length = 1024)
    private String props;
    @Column(value = "remark",length = 512)
    private String remark;

}
