package com.bocloud.ams.repository.association;

import com.bocloud.ams.entity.association.AutoRelationRule;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.model.SimpleBean;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.BasicGenericDao;
import com.megatron.framework.core.CurrentService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @Author liuyuanyuan
 * @Date 2022/5/9 0009 16:49
 * @Description:
 */
@Repository
public class AutoRelationRuleRepository extends BasicGenericDao<AutoRelationRule, Long> {
    public AutoRelationRuleRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    public List<AutoRelationRule> queryByModuleRelationIds(List<Long> ids) throws Exception {
        String sql = "select * from ci_auto_relation_rule where is_deleted=0 and module_relation_id in(:ids) ";
        return this.list(AutoRelationRule.class, sql, MapTools.simpleMap("ids", ids));
    }

    public List<AutoRelationRule> list(Pager pager) throws Exception {
        String sql = "select b.name source_module_name,c.name target_module_name,d.name cmdb_relation_category_name, e.name cmdb_relation_name, a.* from ci_auto_relation_rule a LEFT JOIN ci_module b ON a.source_module_id = b.id LEFT JOIN ci_module c ON a.target_module_id = c.id LEFT JOIN ci_relation_category d ON a.relation_category_id = d.id LEFT JOIN ci_module_relation e ON a.module_relation_id = e.id where a.is_deleted = 0 ";
        sql = getQueryBuilder().buildRaw(sql, pager, "a");
        return this.list(AutoRelationRule.class, sql, getQueryBuilder().getParam(pager.getParams()));
    }

    public List<AutoRelationRule> list(Pager pager, String positionKey) throws Exception {
        String sql;
        if (StringUtils.isNotBlank(positionKey)) {
            sql = "select b.name source_module_name,c.name target_module_name,d.name cmdb_relation_category_name, e.name cmdb_relation_name, a.* from " + positionKey + "." +
                    "ci_auto_relation_rule a LEFT JOIN " + positionKey + "." +
                    "ci_module b ON a.source_module_id = b.id LEFT JOIN " + positionKey + "." +
                    "ci_module c ON a.target_module_id = c.id LEFT JOIN " + positionKey + "." +
                    "ci_relation_category d ON a.relation_category_id = d.id LEFT JOIN " + positionKey + "." +
                    "ci_module_relation e ON a.module_relation_id = e.id where a.is_deleted = 0 ";
        } else {
            sql = "select b.name source_module_name,c.name target_module_name,d.name cmdb_relation_category_name, e.name cmdb_relation_name, a.* from ci_auto_relation_rule a LEFT JOIN ci_module b ON a.source_module_id = b.id LEFT JOIN ci_module c ON a.target_module_id = c.id LEFT JOIN ci_relation_category d ON a.relation_category_id = d.id LEFT JOIN ci_module_relation e ON a.module_relation_id = e.id where a.is_deleted = 0 ";
        }
        sql = getQueryBuilder().buildRaw(sql, pager, "a");
        return this.list(AutoRelationRule.class, sql, getQueryBuilder().getParam(pager.getParams()));
    }

    public List<SimpleBean> list(List<Param> params, Map<String, String> sorter) throws Exception {
        String sql = "select a.id,a.name from ci_auto_relation_rule a where 1=1";
        sql = getQueryBuilder().buildRaw(sql, new Pager(1, Integer.MAX_VALUE, params, sorter), "a");
        List<AutoRelationRule> list = this.list(AutoRelationRule.class, sql, getQueryBuilder().getParam(params));
        List<SimpleBean> beans = new ArrayList<>();
        for (AutoRelationRule rule : list) {
            beans.add(new SimpleBean(rule.getId(), rule.getName()));
        }
        return beans;
    }

    public List<SimpleBean> list(List<Param> params, Map<String, String> sorter, String positionKey) throws Exception {
        String sql;
        if (StringUtils.isNotBlank(positionKey)) {
            sql = "select a.id,a.name from " + positionKey + ".ci_auto_relation_rule a where 1=1";
        } else {
            sql = "select a.id,a.name from ci_auto_relation_rule a where 1=1";
        }
        sql = getQueryBuilder().buildRaw(sql, new Pager(1, Integer.MAX_VALUE, params, sorter), "a");
        List<AutoRelationRule> list = this.list(AutoRelationRule.class, sql, getQueryBuilder().getParam(params));
        List<SimpleBean> beans = new ArrayList<>();
        for (AutoRelationRule rule : list) {
            beans.add(new SimpleBean(rule.getId(), rule.getName()));
        }
        return beans;
    }

    public AutoRelationRule query(Long id) throws Exception {
        String sql = "select * from ci_auto_relation_rule where is_deleted=0 and id = :id";
        List<AutoRelationRule> list = this.list(AutoRelationRule.class, sql, MapTools.simpleMap(Common.ID, id));
        return list.isEmpty() ? null : list.get(0);
    }

    public AutoRelationRule query(Long id, String positionKey) throws Exception {
        String sql;
        if (StringUtils.isNotBlank(positionKey)) {
            sql = "select * from " + positionKey + ".ci_auto_relation_rule where is_deleted=0 and id = :id";
        } else {
            sql = "select * from ci_auto_relation_rule where is_deleted=0 and id = :id";
        }
        List<AutoRelationRule> list = this.list(AutoRelationRule.class, sql, MapTools.simpleMap(Common.ID, id));
        return list.isEmpty() ? null : list.get(0);
    }

    public int count(List<Param> params) throws Exception {
        String sql = "select count(1) from ci_auto_relation_rule where is_deleted=0 ";
        sql = getQueryBuilder().buildRaw(sql, params, null, "");
        return this.countQuery(sql, getQueryBuilder().getParam(params)).intValue();
    }

    public int count(List<Param> params, String positionKey) throws Exception {
        String sql;
        if (StringUtils.isEmpty(positionKey)) {
            sql = "select count(1) from ci_auto_relation_rule where is_deleted=0 ";
        } else {
            sql = "select count(1) from " + positionKey + ".ci_auto_relation_rule where is_deleted=0 ";
        }
        sql = getQueryBuilder().buildRaw(sql, params, null, "");
        return this.countQuery(sql, getQueryBuilder().getParam(params)).intValue();
    }

    public int countSameName(String name) throws Exception {
        String sql = "select count(1) from ci_auto_relation_rule where is_deleted=0 and name = :name ";
        return this.countQuery(sql, MapTools.simpleMap("name", name)).intValue();
    }

    public int isOnly(Long relationCategoryId, Long sourceModuleId, Long targetModuleId) throws Exception {
        String sql = "select count(1) from ci_auto_relation_rule where is_deleted=0 and relation_category_id = :relationCategoryId and source_module_id =:sourceModuleId and target_module_id = :targetModuleId ";
        Map<String, Object> params = MapTools.simpleMap("relationCategoryId", relationCategoryId);
        params.put("sourceModuleId", sourceModuleId);
        params.put("targetModuleId", targetModuleId);
        return this.countQuery(sql, params).intValue();
    }

    public AutoRelationRule query(Long relationCategoryId, Long sourceModuleId, Long targetModuleId) throws Exception {
        String sql = "select * from ci_auto_relation_rule where is_deleted=0 and relation_category_id = :relationCategoryId and source_module_id =:sourceModuleId and target_module_id = :targetModuleId ";
        Map<String, Object> params = MapTools.simpleMap("relationCategoryId", relationCategoryId);
        params.put("sourceModuleId", sourceModuleId);
        params.put("targetModuleId", targetModuleId);
        List<AutoRelationRule> list = this.list(AutoRelationRule.class, sql, params);
        return list.isEmpty() ? null : list.get(0);
    }

    public boolean delete(Long id, Long userId) throws Exception {
        String sql = "update ci_auto_relation_rule set is_deleted=1,gmt_modify = :gmtModify,mender_id=:userId where id = :id";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        params.put("gmtModify", new Date());
        params.put("userId", userId);
        return this.execute(sql, params) > 0;
    }

    public AutoRelationRule queryByModuleRelationId(Long moduleRelationId) throws Exception {
        String sql = "select * from ci_auto_relation_rule where is_deleted=0 and module_relation_id = :moduleRelationId";
        List<AutoRelationRule> list = this.list(AutoRelationRule.class, sql, MapTools.simpleMap("moduleRelationId", moduleRelationId));
        return list.isEmpty() ? null : list.get(0);
    }

    public AutoRelationRule queryByModuleRelationId(Long moduleRelationId, String positionKey) throws Exception {
        String sql;
        if (StringUtils.isNotBlank(positionKey)) {
            sql = "select * from " + positionKey + ".ci_auto_relation_rule where is_deleted=0 and module_relation_id = :moduleRelationId";
        } else {
            sql = "select * from ci_auto_relation_rule where is_deleted=0 and module_relation_id = :moduleRelationId";
        }
        List<AutoRelationRule> list = this.list(AutoRelationRule.class, sql, MapTools.simpleMap("moduleRelationId", moduleRelationId));
        return list.isEmpty() ? null : list.get(0);
    }

    public List<AutoRelationRule> queryByModuleIdAndCategoryId(Long moduleId, Long relationCategoryId) throws Exception {
        String sql = "select a.* from ci_auto_relation_rule a where a.is_deleted=0 and a.relation_category_id = :relationCategoryId and (a.source_module_id= :moduleId or a.target_module_id = :moduleId) order by a.gmt_modify desc";
        Map<String, Object> params = MapTools.simpleMap("relationCategoryId", relationCategoryId);
        params.put("moduleId", moduleId);
        return this.list(AutoRelationRule.class, sql, params);
    }
}
