package com.bocloud.ams.repository.collection;

import com.bocloud.ams.entity.collection.CollectionComponent;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.model.SimpleBean;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.BasicGenericDao;
import com.megatron.framework.core.CurrentService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;


/**
 * @Author liuyuanyuan
 * @Date 2022/6/7 0007 14:28
 * @Description:
 */
@Repository
public class CollectionComponentRepository extends BasicGenericDao<CollectionComponent, Long> {

    public CollectionComponentRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate namedParameterJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, namedParameterJdbcTemplate, service);
    }

    public List<CollectionComponent> list(int page, int rows, List<Param> params, Map<String, String> sorter) throws Exception {
        String sql = "select ccc.*,cm.name as module_name, cp.name as property_name from ci_collection_component ccc left join ci_module cm on ccc.module_id = cm.id left join ci_property cp on ccc.property_id = cp.id where ccc.is_deleted = 0 ";
        sql = getQueryBuilder().buildRaw(sql, new Pager(page, rows, params, sorter), "ccc");
        return this.list(CollectionComponent.class, sql, getQueryBuilder().getParam(params));
    }

    public List<CollectionComponent> list(int page, int rows, List<Param> params, Map<String, String> sorter, String positionKey) throws Exception {
        String sql;
        if (StringUtils.isNotBlank(positionKey)) {
            sql = "select ccc.*,cm.name as module_name, cp.name as property_name from " + positionKey + "." +
                    "ci_collection_component ccc left join " + positionKey + "." +
                    "ci_module cm on ccc.module_id = cm.id left join " + positionKey + "." +
                    "ci_property cp on ccc.property_id = cp.id " +
                    "where ccc.is_deleted = 0 ";
        } else {
            sql = "select ccc.*,cm.name as module_name, cp.name as property_name from ci_collection_component ccc left join ci_module cm on ccc.module_id = cm.id left join ci_property cp on ccc.property_id = cp.id where ccc.is_deleted = 0 ";
        }
        sql = getQueryBuilder().buildRaw(sql, new Pager(page, rows, params, sorter), "ccc");
        return this.list(CollectionComponent.class, sql, getQueryBuilder().getParam(params));
    }

    public List<SimpleBean> list(List<Param> params, Map<String, String> sorter) throws Exception {
        String sql = "select a.id,a.name from ci_collection_component a where a.is_deleted=0";
        sql = getQueryBuilder().buildRaw(sql, new Pager(1, Integer.MAX_VALUE, params, sorter), "a");
        List<CollectionComponent> list = this.list(CollectionComponent.class, sql, getQueryBuilder().getParam(params));
        List<SimpleBean> beans = new ArrayList<>();
        for (CollectionComponent component : list) {
            beans.add(new SimpleBean(component.getId(), component.getName()));
        }
        return beans;
    }

    public List<SimpleBean> list(List<Param> params, Map<String, String> sorter, String positionKey) throws Exception {
        String sql;
        if (StringUtils.isEmpty(positionKey)) {
            sql = "select a.id,a.name from ci_collection_component a where a.is_deleted=0";
        } else {
            sql = "select a.id,a.name from " + positionKey + ".ci_collection_component a where a.is_deleted=0";
        }
        sql = getQueryBuilder().buildRaw(sql, new Pager(1, Integer.MAX_VALUE, params, sorter), "a");
        List<CollectionComponent> list = this.list(CollectionComponent.class, sql, getQueryBuilder().getParam(params));
        List<SimpleBean> beans = new ArrayList<>();
        for (CollectionComponent component : list) {
            beans.add(new SimpleBean(component.getId(), component.getName()));
        }
        return beans;
    }

    public int count(List<Param> params) throws Exception {
        String sql = "select count(1) from ci_collection_component a where a.is_deleted=0 ";
        sql = getQueryBuilder().buildRaw(sql, params, null, "a");
        return this.countQuery(sql, getQueryBuilder().getParam(params)).intValue();
    }

    public int count(List<Param> params, String positionKey) throws Exception {
        String sql;
        if (StringUtils.isEmpty(positionKey)) {
            sql = "select count(1) from ci_collection_component a where a.is_deleted=0 ";
        } else {
            sql = "select count(1) from " + positionKey + ".ci_collection_component a where a.is_deleted=0 ";
        }
        sql = getQueryBuilder().buildRaw(sql, params, null, "a");
        return this.countQuery(sql, getQueryBuilder().getParam(params)).intValue();
    }

    public CollectionComponent query(Long id) throws Exception {
        String sql = "select * from ci_collection_component where id = :id and is_deleted = 0";
        List<CollectionComponent> list = this.list(CollectionComponent.class, sql, MapTools.simpleMap(Common.ID, id));
        return list.isEmpty() ? null : list.get(0);
    }

    public List<CollectionComponent> queryByTemplateId(Long id) throws Exception {
        String sql = "select ccc.*,cm.name as module_name, cp.name as property_name from ci_collection_component ccc left join ci_module cm on ccc.module_id = cm.id left join ci_property cp on cp.id = ccc.property_id where ccc.template_id = :id and ccc.is_deleted = 0 ";
        return this.list(CollectionComponent.class, sql, MapTools.simpleMap(Common.ID, id));
    }

    public List<CollectionComponent> queryByTemplateId(Long id, String positionKey) throws Exception {
        String sql;
        if (StringUtils.isEmpty(positionKey)) {
            sql = "select ccc.*,cm.name as module_name, cp.name as property_name from ci_collection_component ccc left join ci_module cm on ccc.module_id = cm.id left join ci_property cp on cp.id = ccc.property_id where ccc.template_id = :id and ccc.is_deleted = 0 ";
        } else {
            sql = "select ccc.*,cm.name as module_name, cp.name as property_name from " + positionKey + "." +
                    "ci_collection_component ccc left join " + positionKey + "." +
                    "ci_module cm on ccc.module_id = cm.id left join " + positionKey + "." +
                    "ci_property cp on cp.id = ccc.property_id " +
                    "where ccc.template_id = :id and ccc.is_deleted = 0 ";
        }
        return this.list(CollectionComponent.class, sql, MapTools.simpleMap(Common.ID, id));
    }

    public List<CollectionComponent> queryByTemplateId(Long id, String types, String parameterType) throws Exception {
        String sql = "select * from ci_collection_component where template_id = :id and type in(:type) and is_deleted = 0";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        params.put("type", types);
        if (null != parameterType) {
            sql += " and parameter_type = :parameterType ";
            params.put("parameterType", parameterType);
        }
        return this.list(CollectionComponent.class, sql, params);
    }

    public boolean delete(Long id, Long userId) throws Exception {
        String sql = "update ci_collection_component set is_deleted=1,gmt_modify = :gmtModify,mender_id=:userId where id = :id";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        params.put("gmtModify", new Date());
        params.put("userId", userId);
        return this.execute(sql, params) > 0;
    }

    public boolean deleteByTemplate(Long id, Long userId) throws Exception {
        String sql = "update ci_collection_component set is_deleted=1,gmt_modify = :gmtModify,mender_id=:userId where template_id = :id";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        params.put("gmtModify", new Date());
        params.put("userId", userId);
        return this.execute(sql, params) > 0;
    }

    public List<CollectionComponent> queryByModuleId(Long moduleId) throws Exception {
        String sql = "select * from ci_collection_component where module_id = :moduleId and is_deleted = 0";
        return this.list(CollectionComponent.class, sql, MapTools.simpleMap("moduleId", moduleId));
    }
}
