package com.bocloud.ams.repository.collection;

import com.bocloud.ams.entity.collection.CollectionHistory;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.BasicGenericDao;
import com.megatron.framework.core.CurrentService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;


/**
 * @ClassName: CollectionHistoryRepository
 * @Description
 * @Author yangzhenrong
 * @Date 2022/6/15 20:37
 * @Version 1.0
 **/
@Repository
public class CollectionHistoryRepository extends BasicGenericDao<CollectionHistory, Long> {

    public CollectionHistoryRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    public List<CollectionHistory> queryByIdAndType(Long taskInstanceId, String type) throws Exception {
        String sql = "select a.* from ci_collection_history a where a.task_instance_id = :taskInstanceId and a.type = :type ";
        Map<String, Object> params = MapTools.simpleMap("taskInstanceId", taskInstanceId);
        params.put("type", type);
        return list(CollectionHistory.class, sql, params);
    }

    public List<CollectionHistory> queryByIdAndType(Long taskInstanceId, String type, String positionKey) throws Exception {
        String sql;
        if (StringUtils.isEmpty(positionKey)) {
            sql = "select a.* from ci_collection_history a where a.task_instance_id = :taskInstanceId and a.type = :type ";
        } else {
            sql = "select a.* from "+positionKey+".ci_collection_history a where a.task_instance_id = :taskInstanceId and a.type = :type ";
        }
        Map<String, Object> params = MapTools.simpleMap("taskInstanceId", taskInstanceId);
        params.put("type", type);
        return list(CollectionHistory.class, sql, params);
    }

    public List<CollectionHistory> list(Pager pager) throws Exception {
        String sql = "select a.* from ci_collection_history a where 1=1 ";
        sql = getQueryBuilder().buildRaw(sql, pager, "a");
        return this.list(CollectionHistory.class, sql, getQueryBuilder().getParam(pager.getParams()));
    }

    public List<CollectionHistory> list(Pager pager, String positionKey) throws Exception {
        String sql;
        if (StringUtils.isEmpty(positionKey)) {
            sql = "select a.* from ci_collection_history a where 1=1 ";
        } else {
            sql = "select a.* from "+positionKey+".ci_collection_history a where 1=1 ";
        }
        sql = getQueryBuilder().buildRaw(sql, pager, "a");
        return this.list(CollectionHistory.class, sql, getQueryBuilder().getParam(pager.getParams()));
    }

    public int count(List<Param> params) throws Exception {
        String sql = "select count(1) from ci_collection_history a where 1=1 ";
        sql = getQueryBuilder().buildRaw(sql, params, null, "a");
        return this.countQuery(sql, getQueryBuilder().getParam(params)).intValue();
    }

    public int count(List<Param> params, String positionKey) throws Exception {
        String sql;
        if (StringUtils.isEmpty(positionKey)) {
            sql = "select count(1) from ci_collection_history a where 1=1 ";
        } else {
            sql = "select count(1) from "+positionKey+".ci_collection_history a where 1=1 ";
        }
        sql = getQueryBuilder().buildRaw(sql, params, null, "a");
        return this.countQuery(sql, getQueryBuilder().getParam(params)).intValue();
    }

    public CollectionHistory query(Long id) throws Exception {
        String sql = "select a.* from ci_collection_history a where a.id = :id ";
        List<CollectionHistory> list = this.list(CollectionHistory.class, sql, MapTools.simpleMap(Common.ID, id));
        return list.isEmpty() ? null : list.get(0);
    }

    public List<CollectionHistory> listByTaskInstanceId(Long taskInstanceId) throws Exception {
        String sql = "select a.* from ci_collection_history a where a.task_instance_id = :taskInstanceId";
        return this.list(CollectionHistory.class, sql, MapTools.simpleMap("taskInstanceId", taskInstanceId));
    }

    public List<CollectionHistory> listByInstances(List<Long> instanceIds) throws Exception {
        String sql = "select a.* from ci_collection_history a where a.task_instance_id in (:instanceIds) ";
        return this.list(CollectionHistory.class, sql, MapTools.simpleMap("instanceIds", instanceIds));
    }

    public boolean deleteByInstanceIds(List<Long> instanceIds) {
        String sql = "delete from ci_collection_history where instance_id in (:instanceIds)";
        Map<String, Object> params = MapTools.simpleMap("instanceIds", instanceIds);
        return this.execute(sql, params) > 0;
    }

    public List<CollectionHistory> queryByIdAndType(String tableName, Long taskInstanceId, String type) throws Exception {
        String sql = "select a.* from " + tableName + " a where a.task_instance_id = :taskInstanceId and a.type = :type ";
        Map<String, Object> params = MapTools.simpleMap("taskInstanceId", taskInstanceId);
        params.put("type", type);
        return list(CollectionHistory.class, sql, params);
    }

    public List<CollectionHistory> queryByIdAndType(String tableName, Long taskInstanceId, String type, String positionKey) throws Exception {
        if (!StringUtils.isEmpty(positionKey)) {
            tableName = positionKey + "." + tableName;
        }
        String sql = "select a.* from " + tableName + " a where a.task_instance_id = :taskInstanceId and a.type = :type ";
        Map<String, Object> params = MapTools.simpleMap("taskInstanceId", taskInstanceId);
        params.put("type", type);
        return list(CollectionHistory.class, sql, params);
    }

    public int count(String tableName, List<Param> params) throws Exception {
        String sql = "select count(1) from " + tableName + " a where 1=1 ";
        sql = getQueryBuilder().buildRaw(sql, params, null, "a");
        return this.countQuery(sql, getQueryBuilder().getParam(params)).intValue();
    }

    public int count(String tableName, List<Param> params, String positionKey) throws Exception {
        if (!StringUtils.isEmpty(positionKey)) {
            tableName = positionKey + "." + tableName;
        }
        String sql = "select count(1) from " + tableName + " a where 1=1 ";
        sql = getQueryBuilder().buildRaw(sql, params, null, "a");
        return this.countQuery(sql, getQueryBuilder().getParam(params)).intValue();
    }

    public List<CollectionHistory> list(String tableName, Pager pager) throws Exception {
        String sql = "select a.* from " + tableName + " a where 1=1 ";
        sql = getQueryBuilder().buildRaw(sql, pager, "a");
        return this.list(CollectionHistory.class, sql, getQueryBuilder().getParam(pager.getParams()));
    }

    public List<CollectionHistory> list(String tableName, Pager pager, String positionKey) throws Exception {
        if (StringUtils.isNotEmpty(positionKey)) {
            tableName = positionKey + "." + tableName;
        }
        String sql = "select a.* from " + tableName + " a where 1=1 ";
        sql = getQueryBuilder().buildRaw(sql, pager, "a");
        return this.list(CollectionHistory.class, sql, getQueryBuilder().getParam(pager.getParams()));
    }

}
