package com.bocloud.ams.repository.collection;

import com.bocloud.ams.entity.collection.CollectionTemplate;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.model.SimpleBean;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.BasicGenericDao;
import com.megatron.framework.core.CurrentService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;


/**
 * @Author liuyuanyuan
 * @Date 2022/6/7 0007 14:18
 * @Description:
 */
@Repository
public class CollectionTemplateRepository extends BasicGenericDao<CollectionTemplate, Long> {

    public CollectionTemplateRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate namedParameterJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, namedParameterJdbcTemplate, service);
    }

    public boolean exists(String name) {
        String sql = "select * from ci_collection_template a where a.is_deleted = 0 and a.name = :name";
        List<CollectionTemplate> list = this.list(CollectionTemplate.class, sql, MapTools.simpleMap("name", name));
        return !list.isEmpty();
    }

    public List<CollectionTemplate> list(int page, int rows, List<Param> params, Map<String, String> sorter) throws Exception {
        String sql = "select cct.*,cm.name as module_name from ci_collection_template cct left join ci_module cm on cct.module_id = cm.id where cct.is_deleted = 0 and cm.status = 'ENABLE' ";
        sql = getQueryBuilder().buildRaw(sql, new Pager(page, rows, params, sorter), "cct");
        return this.list(CollectionTemplate.class, sql, getQueryBuilder().getParam(params));
    }

    public List<CollectionTemplate> list(int page, int rows, List<Param> params, Map<String, String> sorter, String positionKey) throws Exception {
        String sql;
        if (StringUtils.isNotBlank(positionKey)) {
            sql = "select cct.*,cm.name as module_name from " + positionKey + "." +
                    "ci_collection_template cct left join " + positionKey + "." +
                    "ci_module cm on cct.module_id = cm.id where cct.is_deleted = 0 and cm.status = 'ENABLE' ";
        } else {
            sql = "select cct.*,cm.name as module_name from ci_collection_template cct left join ci_module cm on cct.module_id = cm.id where cct.is_deleted = 0 and cm.status = 'ENABLE' ";
        }
        sql = getQueryBuilder().buildRaw(sql, new Pager(page, rows, params, sorter), "cct");
        return this.list(CollectionTemplate.class, sql, getQueryBuilder().getParam(params));
    }

    public List<SimpleBean> list(List<Param> params, Map<String, String> sorter) throws Exception {
        String sql = "select a.id,a.name,a.collection_type from ci_collection_template a left join ci_module b on a.module_id = b.id  where b.status ='ENABLE' and a.is_deleted=0 ";
        sql = getQueryBuilder().buildRaw(sql, new Pager(1, Integer.MAX_VALUE, params, sorter), "a");
        List<CollectionTemplate> list = this.list(CollectionTemplate.class, sql, getQueryBuilder().getParam(params));
        List<SimpleBean> beans = new ArrayList<>();
        for (CollectionTemplate template : list) {
            beans.add(new SimpleBean(template.getId(), template.getName(), template.getCollectionType()));
        }
        return beans;
    }

    public List<SimpleBean> list(List<Param> params, Map<String, String> sorter, String positionKey) throws Exception {
        String sql;
        if (StringUtils.isEmpty(positionKey)) {
            sql = "select a.id,a.name,a.collection_type from ci_collection_template a left join ci_module b on a.module_id = b.id  where b.status ='ENABLE' and a.is_deleted=0 ";
        } else {
            sql = "select a.id,a.name,a.collection_type from " + positionKey + "." +
                    "ci_collection_template a left join " + positionKey + "." +
                    "ci_module b on a.module_id = b.id  " +
                    "where b.status ='ENABLE' and a.is_deleted=0 ";
        }
        sql = getQueryBuilder().buildRaw(sql, new Pager(1, Integer.MAX_VALUE, params, sorter), "a");
        List<CollectionTemplate> list = this.list(CollectionTemplate.class, sql, getQueryBuilder().getParam(params));
        List<SimpleBean> beans = new ArrayList<>();
        for (CollectionTemplate template : list) {
            beans.add(new SimpleBean(template.getId(), template.getName(), template.getCollectionType()));
        }
        return beans;
    }

    public int count(List<Param> params) throws Exception {
        String sql = "select count(1) from ci_collection_template a left join ci_module b on a.module_id = b.id where b.status= 'ENABLE' and a.is_deleted=0 ";
        sql = getQueryBuilder().buildRaw(sql, params, null, "a");
        return this.countQuery(sql, getQueryBuilder().getParam(params)).intValue();
    }

    public int count(List<Param> params, String positionKey) throws Exception {
        String sql;
        if (StringUtils.isEmpty(positionKey)) {
            sql = "select count(1) from ci_collection_template a left join ci_module b on a.module_id = b.id where b.status= 'ENABLE' and a.is_deleted=0 ";
        } else {
            sql = "select count(1) from " + positionKey + "." +
                    "ci_collection_template a left join " + positionKey + "." +
                    "ci_module b on a.module_id = b.id " +
                    "where b.status= 'ENABLE' and a.is_deleted=0 ";
        }
        sql = getQueryBuilder().buildRaw(sql, params, null, "a");
        return this.countQuery(sql, getQueryBuilder().getParam(params)).intValue();
    }

    public CollectionTemplate query(Long id) throws Exception {
        String sql = "select cct.*,cm.name as module_name from ci_collection_template cct left join ci_module cm on cct.module_id = cm.id  where cct.id = :id and cct.is_deleted = 0 ";
        List<CollectionTemplate> list = this.list(CollectionTemplate.class, sql, MapTools.simpleMap(Common.ID, id));
        return list.isEmpty() ? null : list.get(0);
    }

    public CollectionTemplate query(Long id, String positionKey) throws Exception {
        String sql;
        if (StringUtils.isEmpty(positionKey)) {
            sql = "select cct.*,cm.name as module_name from ci_collection_template cct left join ci_module cm on cct.module_id = cm.id  where cct.id = :id and cct.is_deleted = 0 ";
        } else {
            sql = "select cct.*,cm.name as module_name from " + positionKey + "." +
                    "ci_collection_template cct left join " + positionKey + "." +
                    "ci_module cm on cct.module_id = cm.id  " +
                    "where cct.id = :id and cct.is_deleted = 0 ";
        }
        List<CollectionTemplate> list = this.list(CollectionTemplate.class, sql, MapTools.simpleMap(Common.ID, id));
        return list.isEmpty() ? null : list.get(0);
    }

    public boolean delete(Long id, Long userId) throws Exception {
        String sql = "update ci_collection_template set is_deleted=1,gmt_modify = :gmtModify,mender_id=:userId where id = :id";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        params.put("gmtModify", new Date());
        params.put("userId", userId);
        return this.execute(sql, params) > 0;
    }

}
