package com.bocloud.ams.repository.instance;

import com.bocloud.ams.entity.instance.InstanceAuthority;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.BasicGenericDao;
import com.megatron.framework.core.CurrentService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.Date;
import java.util.List;
import java.util.Map;


/**
 * @Author yangzhenrong
 * @Date 2022/5/23 18:16
 * @Version 1.0
 **/
@Repository
public class InstanceAuthorityRepository extends BasicGenericDao<InstanceAuthority, Long> {

    public InstanceAuthorityRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    public List<InstanceAuthority> list(Pager pager) throws Exception {
        String sql = "select a.* from ci_instance_authority a where a.is_deleted = 0 ";
        sql = getQueryBuilder().buildRaw(sql, pager, "a");
        return this.list(InstanceAuthority.class, sql, getQueryBuilder().getParam(pager.getParams()));
    }

    public List<InstanceAuthority> list(Pager pager, String positionKey) throws Exception {
        String sql;
        if (StringUtils.isNotBlank(positionKey)) {
            sql = "select a.* from " + positionKey + ".ci_instance_authority a where a.is_deleted = 0 ";
        } else {
            sql = "select a.* from ci_instance_authority a where a.is_deleted = 0 ";
        }
        sql = getQueryBuilder().buildRaw(sql, pager, "a");
        return this.list(InstanceAuthority.class, sql, getQueryBuilder().getParam(pager.getParams()));
    }

    public int count(List<Param> params) throws Exception {
        String sql = "select count(1) from ci_instance_authority a where a.is_deleted = 0 ";
        sql = getQueryBuilder().buildRaw(sql, params, null, "a");
        return this.countQuery(sql, getQueryBuilder().getParam(params)).intValue();
    }

    public int count(List<Param> params, String positionKey) throws Exception {
        String sql;
        if (StringUtils.isEmpty(positionKey)) {
            sql = "select count(1) from ci_instance_authority a where a.is_deleted = 0 ";
        } else {
            sql = "select count(1) from " + positionKey + ".ci_instance_authority a where a.is_deleted = 0 ";
        }
        sql = getQueryBuilder().buildRaw(sql, params, null, "a");
        return this.countQuery(sql, getQueryBuilder().getParam(params)).intValue();
    }

    public InstanceAuthority query(Long id) throws Exception {
        String sql = "select * from ci_instance_authority a where a.id = :id and is_deleted=0 ";
        List<InstanceAuthority> list = this.list(InstanceAuthority.class, sql, MapTools.simpleMap(Common.ID, id));
        return list.isEmpty() ? null : list.get(0);
    }

    public boolean remove(Long id, Long userId) throws Exception {
        String sql = "update ci_instance_authority set is_deleted=1,gmt_modify = :gmtModify,mender_id=:userId where id = :id";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        params.put("gmtModify", new Date());
        params.put("userId", userId);
        return this.execute(sql, params) > 0;
    }

    public boolean batchRemove(Long moduleId, List<Long> instanceIds, Long roleId, Long userId) throws Exception {
        String sql = "update ci_instance_authority set is_deleted=1,gmt_modify = :gmtModify,mender_id=:userId " + " where module_id = :moduleId and instance_id in (:instanceIds) and role_id = :roleId ";
        Map<String, Object> params = MapTools.simpleMap("moduleId", moduleId);
        params.put("instanceIds", instanceIds);
        params.put("roleId", roleId);
        params.put("gmtModify", new Date());
        params.put("userId", userId);
        return this.execute(sql, params) > 0;
    }

    public boolean remove(Long moduleId, Long instanceId, Long userId) throws Exception {
        String sql = "update ci_instance_authority set is_deleted=1,gmt_modify = :gmtModify,mender_id=:userId " + " where module_id = :moduleId and instance_id = :instanceId ";
        Map<String, Object> params = MapTools.simpleMap("moduleId", moduleId);
        params.put("instanceId", instanceId);
        params.put("gmtModify", new Date());
        params.put("userId", userId);
        return this.execute(sql, params) > 0;
    }

    public InstanceAuthority query(Long moduleId, Long instanceId, Long roleId, Long authUserId, String auth) throws Exception {
        String sql = "select * from ci_instance_authority a where is_deleted=0 and a.module_id = :moduleId and a.instance_id = :instanceId " + " and a.role_id = :roleId and a.user_id = :authUserId and a.auths like '%" + auth + "%'";
        Map<String, Object> params = MapTools.simpleMap("moduleId", moduleId);
        params.put("instanceId", instanceId);
        params.put("roleId", roleId);
        params.put("authUserId", authUserId);
        List<InstanceAuthority> list = this.list(InstanceAuthority.class, sql, params);
        return list.isEmpty() ? null : list.get(0);
    }

    public List<InstanceAuthority> listByInstanceId(Long moduleId, Long instanceId) throws Exception {
        String sql = "select a.* from ci_instance_authority a where a.is_deleted = 0 and a.module_id = :moduleId and a.instance_id = :instanceId ";
        Map<String, Object> params = MapTools.simpleMap("moduleId", moduleId);
        params.put("instanceId", instanceId);
        return this.list(InstanceAuthority.class, sql, params);
    }

    public List<InstanceAuthority> listByInstanceId(Long moduleId, Long instanceId, Long roleId) throws Exception {
        String sql = "select a.* from ci_instance_authority a where a.is_deleted = 0 and a.module_id = :moduleId and a.instance_id = :instanceId and a.role_id = :roleId";
        Map<String, Object> params = MapTools.simpleMap("moduleId", moduleId);
        params.put("instanceId", instanceId);
        params.put("roleId", roleId);
        return this.list(InstanceAuthority.class, sql, params);
    }

    public List<InstanceAuthority> listByModuleId(Long moduleId, Long authUserId) throws Exception {
        String sql = "select a.* from ci_instance_authority a where a.is_deleted = 0 and a.module_id = :moduleId and a.user_id = :authUserId ";
        Map<String, Object> params = MapTools.simpleMap("moduleId", moduleId);
        params.put("authUserId", authUserId);
        return this.list(InstanceAuthority.class, sql, params);
    }

    public List<InstanceAuthority> listByUserId(Long moduleId, Long instanceId, Long authUserId) throws Exception {
        String sql = "select * from ci_instance_authority a where is_deleted=0 and a.module_id = :moduleId and a.instance_id = :instanceId " + " and a.user_id = :authUserId ";
        Map<String, Object> params = MapTools.simpleMap("moduleId", moduleId);
        params.put("instanceId", instanceId);
        params.put("authUserId", authUserId);
        return this.list(InstanceAuthority.class, sql, params);
    }

    public List<InstanceAuthority> listByUserId(Long moduleId, Long instanceId, Long authUserId, String positioonKey) throws Exception {
        String sql;
        if (StringUtils.isNotEmpty(positioonKey)) {
            sql = "select * from " + positioonKey + ".ci_instance_authority a where is_deleted=0 and a.module_id = :moduleId and a.instance_id = :instanceId " + " and a.user_id = :authUserId ";
        } else {
            sql = "select * from ci_instance_authority a where is_deleted=0 and a.module_id = :moduleId and a.instance_id = :instanceId " + " and a.user_id = :authUserId ";
        }
        Map<String, Object> params = MapTools.simpleMap("moduleId", moduleId);
        params.put("instanceId", instanceId);
        params.put("authUserId", authUserId);
        return this.list(InstanceAuthority.class, sql, params);
    }

    public InstanceAuthority queryByAuth(Long moduleId, Long instanceId, Long authUserId, String auth) throws Exception {
        String sql = "select * from ci_instance_authority a where is_deleted=0 and a.module_id = :moduleId and a.instance_id = :instanceId " + " and a.user_id = :authUserId and a.auths like '%" + auth + "%'";
        Map<String, Object> params = MapTools.simpleMap("moduleId", moduleId);
        params.put("instanceId", instanceId);
        params.put("authUserId", authUserId);
        List<InstanceAuthority> list = this.list(InstanceAuthority.class, sql, params);
        return list.isEmpty() ? null : list.get(0);
    }

    public List<InstanceAuthority> listByModuleIdAndAuth(Long moduleId, Long authUserId, String auth) throws Exception {
        String sql = "select a.* from ci_instance_authority a where a.is_deleted = 0 and a.module_id = :moduleId " + " and a.user_id = :authUserId  and a.auths like '%" + auth + "%'";
        Map<String, Object> params = MapTools.simpleMap("moduleId", moduleId);
        params.put("authUserId", authUserId);
        return this.list(InstanceAuthority.class, sql, params);
    }

    public List<InstanceAuthority> listByInstanceId(Long moduleId, List<Long> instanceIds, List<Long> userIds) throws Exception {
        String sql = "select a.* from ci_instance_authority a where a.is_deleted = 0 and a.module_id = :moduleId and a.instance_id in (:instanceIds) and a.user_id in (:userIds)";
        Map<String, Object> params = MapTools.simpleMap("moduleId", moduleId);
        params.put("instanceIds", instanceIds);
        params.put("userIds", userIds);
        return this.list(InstanceAuthority.class, sql, params);
    }

    public List<InstanceAuthority> queryByAuth(Long moduleId, List<Long> instanceIds, Long authUserId, String auth) throws Exception {
        String sql = "select * from ci_instance_authority a where is_deleted=0 and a.module_id = :moduleId and a.instance_id in (:instanceIds) " + " and a.user_id = :authUserId and a.auths like '%" + auth + "%'";
        Map<String, Object> params = MapTools.simpleMap("moduleId", moduleId);
        params.put("instanceIds", instanceIds);
        params.put("authUserId", authUserId);
        return this.list(InstanceAuthority.class, sql, params);
    }

    public boolean batchRemove(Long moduleId, List<Long> instanceIds, Long userId) throws Exception {
        String sql = "update ci_instance_authority set is_deleted=1,gmt_modify = :gmtModify,mender_id=:userId " + " where module_id = :moduleId and instance_id in (:instanceIds) ";
        Map<String, Object> params = MapTools.simpleMap("moduleId", moduleId);
        params.put("instanceIds", instanceIds);
        params.put("gmtModify", new Date());
        params.put("userId", userId);
        return this.execute(sql, params) > 0;
    }

}
