package com.bocloud.ams.repository.instance;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.ams.entity.instance.InstanceRelation;
import com.bocloud.ams.repository.utils.CmdbUtils;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.ListTool;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.BasicGenericDao;
import com.megatron.framework.core.CurrentService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @Author liuyuanyuan
 * @Date 2022/5/16 0016 14:21
 * @Description:
 */
@Slf4j
@Repository
public class InstanceRelationRepository extends BasicGenericDao<InstanceRelation, Long> {

    public InstanceRelationRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    public int countTable(String instanceRelationTable) throws Exception {
        String sql = "select count(1) from information_schema.TABLES where table_schema = (select database()) AND TABLE_NAME = :instanceRelationTable;";
        return this.countQuery(sql, MapTools.simpleMap("instanceRelationTable", instanceRelationTable)).intValue();
    }

    public void updateOfnameBySourceId(String name, String instanceRelationTable, Long sourceInstanceId, Long userId) throws Exception {
        String sql = "update " + instanceRelationTable + " set gmt_modify = :gmtModify,mender_id=:userId,source_instance_name=:name where source_instance_id = :sourceInstanceId ";
        Map<String, Object> params = MapTools.simpleMap("sourceInstanceId", sourceInstanceId);
        params.put("name", name);
        params.put("gmtModify", new Date());
        params.put("userId", userId);
        this.execute(sql, params);
    }

    public void updateOfnameByTargetId(String name, String instanceRelationTable, Long targetInstanceId, Long userId) throws Exception {
        String sql = "update " + instanceRelationTable + " set gmt_modify = :gmtModify,mender_id=:userId,target_instance_name=:name where target_instance_id = :targetInstanceId ";
        Map<String, Object> params = MapTools.simpleMap("targetInstanceId", targetInstanceId);
        params.put("name", name);
        params.put("gmtModify", new Date());
        params.put("userId", userId);
        this.execute(sql, params);
    }

    public void createTable(String instanceRelationTable, InstanceRelation relation) throws Exception {
        // 拼接sql
        StringBuilder sqlBuilder = new StringBuilder();
        sqlBuilder.append("CREATE TABLE " + instanceRelationTable + " (\n");
        Map<String, JSONObject> fieldMap = CmdbUtils.getDeclaredFieldsInfo(relation);
        // id默认为主键
        sqlBuilder.append("id bigint NOT NULL AUTO_INCREMENT,\n");
        for (Map.Entry<String, JSONObject> entry : fieldMap.entrySet()) {
            String fieldName = entry.getKey();
            JSONObject fieldJSON = entry.getValue();
            sqlBuilder.append("" + fieldName + "");
            String fieldType = CmdbUtils.judgeDataType(fieldJSON.getString("fieldType"));
            if (!fieldJSON.getBooleanValue("nullable")) {
                if ("varchar".equals(fieldType)) {
                    sqlBuilder.append(" " + fieldType + "(" + fieldJSON.getLong("length") + ") NOT NULL,\n");
                } else if ("tinyint".equals(fieldType)) {
                    sqlBuilder.append(" " + fieldType + "(1) NOT NULL,\n");
                } else {
                    // 目前包括bigint、datetime
                    sqlBuilder.append(" " + fieldType + " NOT NULL,\n");
                }
            } else {
                if ("varchar".equals(fieldType)) {
                    sqlBuilder.append(" " + fieldType + "(" + fieldJSON.getLong("length") + ") DEFAULT NULL,\n");
                } else if ("tinyint".equals(fieldType)) {
                    sqlBuilder.append(" " + fieldType + "(1) DEFAULT NULL,\n");
                } else {
                    // 目前包括bigint、datetime
                    sqlBuilder.append(" " + fieldType + " DEFAULT NULL,\n");
                }
            }
        }
        sqlBuilder.append("PRIMARY KEY (id)\n");
        sqlBuilder.append(") ENGINE=InnoDB DEFAULT CHARSET=utf8;");
        this.execute(sqlBuilder.toString(), null);
    }

    public void dropTable(String instanceRelationTable) throws Exception {
        String sql = "DROP TABLE " + instanceRelationTable;
        this.execute(sql, null);
    }

    public List<InstanceRelation> query(String instanceTable, Long moduleId) throws Exception {
        String sql = "select a.* from " + instanceTable + " a where (a.source_id = :moduleId OR a.target_id = :moduleId) and is_deleted=0 ";
        return this.list(InstanceRelation.class, sql, MapTools.simpleMap("moduleId", moduleId));
    }

    public List<InstanceRelation> query(String instanceRelationTable, Long sourceModuleId, Long sourceInstanceId, Long targetModuleId, Long targetInstanceId) throws Exception {
        String sql = "select a.* from " + instanceRelationTable + " a where is_deleted=0 " + " and source_module_id = :sourceModuleId ";
        sql += " and target_module_id = :targetModuleId ";
        Map<String, Object> params = MapTools.simpleMap("sourceModuleId", sourceModuleId);
        params.put("targetModuleId", targetModuleId);
        if (null != sourceInstanceId) {
            sql += " and source_instance_id = :sourceInstanceId ";
            params.put("sourceInstanceId", sourceInstanceId);
        }
        if (null != targetInstanceId) {
            sql += " and target_instance_id = :targetInstanceId ";
            params.put("targetInstanceId", targetInstanceId);
        }
        return this.list(InstanceRelation.class, sql, params);
    }

    public List<InstanceRelation> query(String instanceRelationTable, Long sourceModuleId, Long sourceInstanceId, Long targetModuleId, Long targetInstanceId, String positionKey) throws Exception {
        if (StringUtils.isNotEmpty(positionKey)) {
            instanceRelationTable = positionKey + "." + instanceRelationTable;
        }
        String sql = "select a.* from " + instanceRelationTable + " a where is_deleted=0 " + " and source_module_id = :sourceModuleId ";
        sql += " and target_module_id = :targetModuleId ";
        Map<String, Object> params = MapTools.simpleMap("sourceModuleId", sourceModuleId);
        params.put("targetModuleId", targetModuleId);
        if (null != sourceInstanceId) {
            sql += " and source_instance_id = :sourceInstanceId ";
            params.put("sourceInstanceId", sourceInstanceId);
        }
        if (null != targetInstanceId) {
            sql += " and target_instance_id = :targetInstanceId ";
            params.put("targetInstanceId", targetInstanceId);
        }
        return this.list(InstanceRelation.class, sql, params);
    }

    public InstanceRelation queryById(String instanceRelationTable, Long id) throws Exception {
        String sql = "select a.* from " + instanceRelationTable + " a where id = :id and is_deleted=0 ";
        List<InstanceRelation> list = this.list(InstanceRelation.class, sql, MapTools.simpleMap(Common.ID, id));
        return list.isEmpty() ? null : list.get(0);
    }

    public boolean remove(Long id, String instanceRelationTable, Long userId) throws Exception {
        String sql = "update " + instanceRelationTable + " set is_deleted=1,gmt_modify = :gmtModify,mender_id=:userId where id = :id";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        params.put("gmtModify", new Date());
        params.put("userId", userId);
        return this.execute(sql, params) > 0;
    }

    public List<InstanceRelation> queryByModuleIds(String instanceTable, Long sourceModuleId, Long targetModuleId) throws Exception {
        String sql = "select a.* from " + instanceTable + " a where  a.source_module_id = :sourceModuleId and a.target_module_id = :targetModuleId and a.is_deleted = 0 ";
        Map<String, Object> params = MapTools.simpleMap("sourceModuleId", sourceModuleId);
        params.put("targetModuleId", targetModuleId);
        return this.list(InstanceRelation.class, sql, params);
    }

    public InstanceRelation query(String instanceRelationTable, Long sourceModuleId, Long sourceInstanceId, String sourceType, Long targetModuleId, Long targetInstanceId, String targetType) throws Exception {
        String sql = "select a.* from " + instanceRelationTable + " a where is_deleted=0 " + " and source_module_id = :sourceModuleId and source_instance_id = :sourceInstanceId and source_type = :sourceType " + " and target_module_id = :targetModuleId and target_instance_id = :targetInstanceId and target_type = :targetType ";
        Map<String, Object> params = MapTools.simpleMap("sourceModuleId", sourceModuleId);
        params.put("sourceInstanceId", sourceInstanceId);
        params.put("sourceType", sourceType);
        params.put("targetModuleId", targetModuleId);
        params.put("targetInstanceId", targetInstanceId);
        params.put("targetType", targetType);
        List<InstanceRelation> list = this.list(InstanceRelation.class, sql, params);
        return list.isEmpty() ? null : list.get(0);
    }

    public InstanceRelation query(String instanceRelationTable, Long moduleId, Long instanceId) throws Exception {
        String sql = "select a.* from " + instanceRelationTable + " a where is_deleted=0 " + " and (( source_module_id = :moduleId and source_instance_id = :instanceId ) " + " or ( target_module_id = :moduleId and target_instance_id = :instanceId ))";
        Map<String, Object> params = MapTools.simpleMap("moduleId", moduleId);
        params.put("instanceId", instanceId);
        List<InstanceRelation> list = this.list(InstanceRelation.class, sql, params);
        return list.isEmpty() ? null : list.get(0);
    }

    public InstanceRelation querySource(String instanceRelationTable, Long moduleId, Long instanceId, Long targetModuleId) throws Exception {
        String sql = "select a.* from " + instanceRelationTable + " a where is_deleted=0 " + " and source_module_id = :moduleId and source_instance_id = :instanceId and target_module_id = :targetModuleId";
        Map<String, Object> params = MapTools.simpleMap("moduleId", moduleId);
        params.put("instanceId", instanceId);
        params.put("targetModuleId", targetModuleId);
        List<InstanceRelation> list = this.list(InstanceRelation.class, sql, params);
        return list.isEmpty() ? null : list.get(0);
    }

    public InstanceRelation queryTarget(String instanceRelationTable, Long moduleId, Long instanceId, Long sourceModuleId) throws Exception {
        String sql = "select a.* ,'1' is_reversed from " + instanceRelationTable + " a where is_deleted=0 " + " and target_module_id = :moduleId and target_instance_id = :instanceId and source_module_id = :sourceModuleId";
        Map<String, Object> params = MapTools.simpleMap("moduleId", moduleId);
        params.put("instanceId", instanceId);
        params.put("sourceModuleId", sourceModuleId);
        List<InstanceRelation> list = this.list(InstanceRelation.class, sql, params);
        return list.isEmpty() ? null : list.get(0);
    }

    public List<InstanceRelation> queryByCondition(String instanceRelationTable, Long sourceModuleId, Long sourceInstanceId, Long targetModuleId, Long targetInstanceId) throws Exception {
        String sql = "select a.* from " + instanceRelationTable + " a where is_deleted=0 ";
        Map<String, Object> params = new HashMap<>();
        if (null != sourceModuleId && 0 != sourceModuleId) {
            sql += " and source_module_id = :sourceModuleId";
            params.put("sourceModuleId", sourceModuleId);
        }
        if (null != sourceInstanceId && 0 != sourceInstanceId) {
            sql += "  and source_instance_id = :sourceInstanceId ";
            params.put("sourceInstanceId", sourceInstanceId);
        }
        if (null != targetModuleId && 0 != targetModuleId) {
            sql += " and target_module_id = :targetModuleId";
            params.put("targetModuleId", targetModuleId);
        }
        if (null != targetInstanceId && 0 != targetInstanceId) {
            sql += " and target_instance_id = :targetInstanceId";
            params.put("targetInstanceId", targetInstanceId);
        }
        return this.list(InstanceRelation.class, sql, params);
    }

    public boolean removeByIds(List<Long> ids, String instanceRelationTable, Long userId) throws Exception {
        String sql = "update " + instanceRelationTable + " set is_deleted=1,gmt_modify = :gmtModify,mender_id=:userId where id in (:ids)";
        Map<String, Object> params = MapTools.simpleMap("ids", ids);
        params.put("gmtModify", new Date());
        params.put("userId", userId);
        return this.execute(sql, params) > 0;
    }

    public List<InstanceRelation> queryByParams(final String instanceRelationTableName, final List<Long> instanceIdList, final List<Long> moduleIds, final Long categoryId) throws Exception {
        String sql = "select a.* from " + instanceRelationTableName + " a where ( a.source_module_id in (:moduleIds) OR a.target_module_id in (:moduleIds) ) and (a.source_instance_id in (:instanceIdList) OR a.target_instance_id in (:instanceIdList)) and relation_category_id = :categoryId and a.is_deleted = 0 ";
        Map<String, Object> params = MapTools.simpleMap("instanceIdList", instanceIdList);
        params.put("moduleIds", moduleIds);
        params.put("categoryId", categoryId);
        return this.list(InstanceRelation.class, sql, params);
    }

    public List<InstanceRelation> querySources(String instanceRelationTable, Long moduleId, Long instanceId) throws Exception {
        String sql = "select a.*,b.source_type,b.target_type,b.relation_id,b.name relation_name from " + instanceRelationTable + " a " + " left join ci_module_relation b on a.module_relation_id = b.id " + " where a.is_deleted=0 and b.is_deleted=0 " + " and a.source_module_id = :moduleId and a.source_instance_id = :instanceId  ";
        Map<String, Object> params = MapTools.simpleMap("moduleId", moduleId);
        params.put("instanceId", instanceId);
        return this.list(InstanceRelation.class, sql, params);
    }

    public List<InstanceRelation> querySources(String instanceRelationTable, Long moduleId, Long instanceId, String positonKey) throws Exception {
        String sql;
        if (StringUtils.isNotBlank(positonKey)) {
            sql = "select a.*,b.source_type,b.target_type,b.relation_id,b.name relation_name from " + positonKey + "." + instanceRelationTable + " a " + " left join " + positonKey + ".ci_module_relation b on a.module_relation_id = b.id " + " where a.is_deleted=0 and b.is_deleted=0 " + " and a.source_module_id = :moduleId and a.source_instance_id = :instanceId  ";
        } else {
            sql = "select a.*,b.source_type,b.target_type,b.relation_id,b.name relation_name from " + instanceRelationTable + " a " + " left join ci_module_relation b on a.module_relation_id = b.id " + " where a.is_deleted=0 and b.is_deleted=0 " + " and a.source_module_id = :moduleId and a.source_instance_id = :instanceId  ";
        }
        Map<String, Object> params = MapTools.simpleMap("moduleId", moduleId);
        params.put("instanceId", instanceId);
        return this.list(InstanceRelation.class, sql, params);
    }

    public List<InstanceRelation> queryTargets(String instanceRelationTable, Long moduleId, Long instanceId) throws Exception {
        String sql = "select a.*,b.source_type,b.target_type,b.relation_id,b.name relation_name,'1' is_reversed from " + instanceRelationTable + " a " + " left join ci_module_relation b on a.module_relation_id = b.id " + " where a.is_deleted=0 and b.is_deleted=0 " + " and a.target_module_id = :moduleId and a.target_instance_id = :instanceId";
        Map<String, Object> params = MapTools.simpleMap("moduleId", moduleId);
        params.put("instanceId", instanceId);
        return this.list(InstanceRelation.class, sql, params);
    }

    public List<InstanceRelation> queryTargets(String instanceRelationTable, Long moduleId, Long instanceId, String positionKey) throws Exception {
        String sql;
        if (StringUtils.isNotBlank(positionKey)) {
            sql = "select a.*,b.source_type,b.target_type,b.relation_id,b.name relation_name,'1' is_reversed from " + positionKey + "." + instanceRelationTable + " a " + " left join " + positionKey + ".ci_module_relation b on a.module_relation_id = b.id " + " where a.is_deleted=0 and b.is_deleted=0 " + " and a.target_module_id = :moduleId and a.target_instance_id = :instanceId";
        } else {
            sql = "select a.*,b.source_type,b.target_type,b.relation_id,b.name relation_name,'1' is_reversed from " + instanceRelationTable + " a " + " left join ci_module_relation b on a.module_relation_id = b.id " + " where a.is_deleted=0 and b.is_deleted=0 " + " and a.target_module_id = :moduleId and a.target_instance_id = :instanceId";
        }
        Map<String, Object> params = MapTools.simpleMap("moduleId", moduleId);
        params.put("instanceId", instanceId);
        return this.list(InstanceRelation.class, sql, params);
    }

    public List<InstanceRelation> listReverse(final String instanceRelationTableName, Long categoryId, List<Long> sourceInstanceIds, List<Long> targetInstanceIds) throws Exception {
        String sql = " select a.*, b.relationName relation_name , '1' is_reversed from " + instanceRelationTableName + " a join ( select cmr.* , cr.reverse_name as relationName from ci_module_relation cmr join ci_relation cr on cmr.relation_id = cr.id  where cmr.relation_category_id = :categoryId and cr.category_id = :categoryId ) b on a.module_relation_id=b.id where a.is_deleted = 0  and b.is_deleted=0 and a.relation_category_id = :categoryId " + " and ( a.source_instance_id in ( :sourceInstanceIds ) or a.target_instance_id in ( :targetInstanceIds ) ) ";
        Map<String, Object> params = MapTools.simpleMap("categoryId", categoryId);
        params.put("sourceInstanceIds", sourceInstanceIds);
        params.put("targetInstanceIds", targetInstanceIds);
        return this.list(InstanceRelation.class, sql, params);
    }

    public List<InstanceRelation> listReverse(final String instanceRelationTableName, Long categoryId, List<Long> sourceInstanceIds, List<Long> targetInstanceIds, String positionKey) throws Exception {
        String sql;
        if (StringUtils.isNotBlank(positionKey)) {
            sql = " select a.*, b.relationName relation_name , '1' is_reversed from " + positionKey + "." + instanceRelationTableName + " a join ( select cmr.* , cr.reverse_name as relationName from "+positionKey+".ci_module_relation cmr join "+positionKey+".ci_relation cr on cmr.relation_id = cr.id  where cmr.relation_category_id = :categoryId and cr.category_id = :categoryId ) b on a.module_relation_id=b.id where a.is_deleted = 0  and b.is_deleted=0 and a.relation_category_id = :categoryId " + " and ( a.source_instance_id in ( :sourceInstanceIds ) or a.target_instance_id in ( :targetInstanceIds ) ) ";
        } else {
            sql = " select a.*, b.relationName relation_name , '1' is_reversed from " + instanceRelationTableName + " a join ( select cmr.* , cr.reverse_name as relationName from ci_module_relation cmr join ci_relation cr on cmr.relation_id = cr.id  where cmr.relation_category_id = :categoryId and cr.category_id = :categoryId ) b on a.module_relation_id=b.id where a.is_deleted = 0  and b.is_deleted=0 and a.relation_category_id = :categoryId " + " and ( a.source_instance_id in ( :sourceInstanceIds ) or a.target_instance_id in ( :targetInstanceIds ) ) ";
        }
        Map<String, Object> params = MapTools.simpleMap("categoryId", categoryId);
        params.put("sourceInstanceIds", sourceInstanceIds);
        params.put("targetInstanceIds", targetInstanceIds);
        return this.list(InstanceRelation.class, sql, params);
    }

    public List<InstanceRelation> listForward(final String instanceRelationTableName, Long categoryId, List<Long> sourceInstanceIds, List<Long> targetInstanceIds) throws Exception {
        String sql = "select a.*,b.relationName relation_name from " + instanceRelationTableName + " a join ( select cmr.* , cr.name as relationName from ci_module_relation cmr join ci_relation cr on cmr.relation_id = cr.id  where cmr.relation_category_id = :categoryId and cr.category_id = :categoryId ) b on a.module_relation_id=b.id where a.is_deleted = 0 and b.is_deleted=0 and a.relation_category_id = :categoryId" + " and ( a.source_instance_id in ( :sourceInstanceIds ) or a.target_instance_id in ( :targetInstanceIds ) ) ";
        Map<String, Object> params = MapTools.simpleMap("categoryId", categoryId);
        params.put("sourceInstanceIds", sourceInstanceIds);
        params.put("targetInstanceIds", targetInstanceIds);
        return this.list(InstanceRelation.class, sql, params);
    }

    public List<InstanceRelation> listForward(final String instanceRelationTableName, Long categoryId, List<Long> sourceInstanceIds, List<Long> targetInstanceIds, String positionKey) throws Exception {
        String sql;
        if (StringUtils.isNotBlank(positionKey)) {
            sql = "select a.*,b.relationName relation_name from " + positionKey + "." + instanceRelationTableName + " a join ( select cmr.* , cr.name as relationName from "+positionKey+".ci_module_relation cmr join "+positionKey+".ci_relation cr on cmr.relation_id = cr.id  where cmr.relation_category_id = :categoryId and cr.category_id = :categoryId ) b on a.module_relation_id=b.id where a.is_deleted = 0 and b.is_deleted=0 and a.relation_category_id = :categoryId" + " and ( a.source_instance_id in ( :sourceInstanceIds ) or a.target_instance_id in ( :targetInstanceIds ) ) ";
        } else {
            sql = "select a.*,b.relationName relation_name from " + instanceRelationTableName + " a join ( select cmr.* , cr.name as relationName from ci_module_relation cmr join ci_relation cr on cmr.relation_id = cr.id  where cmr.relation_category_id = :categoryId and cr.category_id = :categoryId ) b on a.module_relation_id=b.id where a.is_deleted = 0 and b.is_deleted=0 and a.relation_category_id = :categoryId" + " and ( a.source_instance_id in ( :sourceInstanceIds ) or a.target_instance_id in ( :targetInstanceIds ) ) ";
        }
        Map<String, Object> params = MapTools.simpleMap("categoryId", categoryId);
        params.put("sourceInstanceIds", sourceInstanceIds);
        params.put("targetInstanceIds", targetInstanceIds);
        return this.list(InstanceRelation.class, sql, params);
    }

    public List<InstanceRelation> listReverseByInstanceIds(final String instanceRelationTableName, Long categoryId, Long moduleId, Long instanceId) throws Exception {
        String sql = " select a.*,b.reverse_name relation_name , '1' is_reversed from " + instanceRelationTableName + " a join ( select cmr.* , cr.reverse_name from ci_module_relation cmr join ci_relation cr on cmr.relation_id = cr.id  where cmr.relation_category_id = :categoryId and cr.category_id = :categoryId ) b on a.module_relation_id=b.id where a.is_deleted = 0  and b.is_deleted=0 and a.target_module_id = :moduleId " + " and  a.target_instance_id = :instanceId ";
        Map<String, Object> params = MapTools.simpleMap("moduleId", moduleId);
        params.put("instanceId", instanceId);
        params.put("categoryId", categoryId);
        return this.list(InstanceRelation.class, sql, params);
    }

    public List<InstanceRelation> listReverseByInstanceIds(final String instanceRelationTableName, Long categoryId, Long moduleId, Long instanceId, String positionKey) throws Exception {
        String sql;
        if (StringUtils.isNotBlank(positionKey)) {
            sql = " select a.*,b.reverse_name relation_name , '1' is_reversed from " + positionKey + "." + instanceRelationTableName + " a join ( select cmr.* , cr.reverse_name from "+positionKey+".ci_module_relation cmr join "+positionKey+".ci_relation cr on cmr.relation_id = cr.id  where cmr.relation_category_id = :categoryId and cr.category_id = :categoryId ) b on a.module_relation_id=b.id where a.is_deleted = 0  and b.is_deleted=0 and a.target_module_id = :moduleId " + " and  a.target_instance_id = :instanceId ";
        } else {
            sql = " select a.*,b.reverse_name relation_name , '1' is_reversed from " + instanceRelationTableName + " a join ( select cmr.* , cr.reverse_name from ci_module_relation cmr join ci_relation cr on cmr.relation_id = cr.id  where cmr.relation_category_id = :categoryId and cr.category_id = :categoryId ) b on a.module_relation_id=b.id where a.is_deleted = 0  and b.is_deleted=0 and a.target_module_id = :moduleId " + " and  a.target_instance_id = :instanceId ";
        }
        Map<String, Object> params = MapTools.simpleMap("moduleId", moduleId);
        params.put("instanceId", instanceId);
        params.put("categoryId", categoryId);
        return this.list(InstanceRelation.class, sql, params);
    }

    public List<InstanceRelation> listForwardByInstanceIds(final String instanceRelationTableName, Long categoryId, Long moduleId, Long instanceId) throws Exception {
        String sql = "select a.*,b.name relation_name from " + instanceRelationTableName + " a join ( select cmr.* , cr.reverse_name from ci_module_relation cmr join ci_relation cr on cmr.relation_id = cr.id  where cmr.relation_category_id = :categoryId and cr.category_id = :categoryId ) b on a.module_relation_id=b.id where a.is_deleted = 0 and b.is_deleted=0 and a.source_module_id = :moduleId " + " and  a.source_instance_id = :instanceId ";
        Map<String, Object> params = MapTools.simpleMap("moduleId", moduleId);
        params.put("instanceId", instanceId);
        params.put("categoryId", categoryId);
        return this.list(InstanceRelation.class, sql, params);
    }

    public List<InstanceRelation> listForwardByInstanceIds(final String instanceRelationTableName, Long categoryId, Long moduleId, Long instanceId, String positionKey) throws Exception {
        String sql;
        if (StringUtils.isNotBlank(positionKey)) {
            sql = "select a.*,b.name relation_name from " + positionKey + "." + instanceRelationTableName + " a join ( select cmr.* , cr.reverse_name from "+positionKey+".ci_module_relation cmr join "+positionKey+".ci_relation cr on cmr.relation_id = cr.id  where cmr.relation_category_id = :categoryId and cr.category_id = :categoryId ) b on a.module_relation_id=b.id where a.is_deleted = 0 and b.is_deleted=0 and a.source_module_id = :moduleId " + " and  a.source_instance_id = :instanceId ";
        } else {
            sql = "select a.*,b.name relation_name from " + instanceRelationTableName + " a join ( select cmr.* , cr.reverse_name from ci_module_relation cmr join ci_relation cr on cmr.relation_id = cr.id  where cmr.relation_category_id = :categoryId and cr.category_id = :categoryId ) b on a.module_relation_id=b.id where a.is_deleted = 0 and b.is_deleted=0 and a.source_module_id = :moduleId " + " and  a.source_instance_id = :instanceId ";
        }
        Map<String, Object> params = MapTools.simpleMap("moduleId", moduleId);
        params.put("instanceId", instanceId);
        params.put("categoryId", categoryId);
        return this.list(InstanceRelation.class, sql, params);
    }

    public List<InstanceRelation> queryByInstanceIds(String instanceRelationTable, Long sourceModuleId, List<Long> sourceInstanceIds, Long targetModuleId, List<Long> targetInstanceIds) throws Exception {
        String sql = "select a.* from " + instanceRelationTable + " a where is_deleted=0 " + " and source_module_id = :sourceModuleId ";
        sql += " and target_module_id = :targetModuleId ";
        Map<String, Object> params = MapTools.simpleMap("sourceModuleId", sourceModuleId);
        params.put("targetModuleId", targetModuleId);
        if (!ListTool.isEmpty(sourceInstanceIds)) {
            sql += " and source_instance_id in (:sourceInstanceIds) ";
            params.put("sourceInstanceIds", sourceInstanceIds);
        }
        if (!ListTool.isEmpty(targetInstanceIds)) {
            sql += " and target_instance_id in (:targetInstanceIds) ";
            params.put("targetInstanceIds", targetInstanceIds);
        }
        return this.list(InstanceRelation.class, sql, params);
    }

    public boolean removeByInstanceId(Long instanceId, Long moduleId, String instanceRelationTable, Long userId) throws Exception {
        String sql = "update " + instanceRelationTable + " set is_deleted=1,gmt_modify = :gmtModify,mender_id=:userId " + " where (source_instance_id = :instanceId and source_module_id = :moduleId) " + " or (target_instance_id = :instanceId and target_module_id = :moduleId)";
        Map<String, Object> params = MapTools.simpleMap("instanceId", instanceId);
        params.put("moduleId", moduleId);
        params.put("gmtModify", new Date());
        params.put("userId", userId);
        return this.execute(sql, params) > 0;
    }

    public boolean batchRemoveByInstanceIds(List<Long> instanceIds, Long moduleId, String instanceRelationTable, Long userId) throws Exception {
        String sql = "update " + instanceRelationTable + " set is_deleted=1,gmt_modify = :gmtModify,mender_id=:userId " + " where (source_instance_id in (:instanceIds) and source_module_id = :moduleId) " + " or (target_instance_id in (:instanceIds) and target_module_id = :moduleId)";
        Map<String, Object> params = MapTools.simpleMap("instanceIds", instanceIds);
        params.put("moduleId", moduleId);
        params.put("gmtModify", new Date());
        params.put("userId", userId);
        return this.execute(sql, params) > 0;
    }

}
