package com.bocloud.ams.repository.instance;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.ams.entity.instance.Instance;
import com.bocloud.ams.repository.utils.CmdbUtils;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.BasicGenericDao;
import com.megatron.framework.core.CurrentService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.Date;
import java.util.List;
import java.util.Map;


/**
 * @Author yangzhenrong
 * @Date 2022/5/11 11:31
 * @Version 1.0
 **/
@Repository
public class InstanceRepository extends BasicGenericDao<Instance, Long> {

    public InstanceRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    public void createTable(String instanceTable, Instance cmdbInstance) throws Exception {
        // 拼接sql
        StringBuilder sqlBuilder = new StringBuilder();
        sqlBuilder.append("CREATE TABLE " + instanceTable + " (\n");
        Map<String, JSONObject> fieldMap = CmdbUtils.getDeclaredFieldsInfo(cmdbInstance);
        // id默认为主键
        sqlBuilder.append("id bigint NOT NULL AUTO_INCREMENT,\n");
        for (Map.Entry<String, JSONObject> entry : fieldMap.entrySet()) {
            String fieldName = entry.getKey();
            JSONObject fieldJSON = entry.getValue();
            //实例表名称字段长度和实例值表name长度保持一致
            if ("name".equalsIgnoreCase(fieldName)) {
                fieldJSON.put("length", 100);
            }
            sqlBuilder.append("" + fieldName + "");
            String fieldType = CmdbUtils.judgeDataType(fieldJSON.getString("fieldType"));
            if (!fieldJSON.getBooleanValue("nullable")) {
                if ("varchar".equals(fieldType)) {
                    sqlBuilder.append(" " + fieldType + "(" + fieldJSON.getLong("length") + ") NOT NULL,\n");
                } else if ("tinyint".equals(fieldType)) {
                    sqlBuilder.append(" " + fieldType + "(1) NOT NULL,\n");
                } else {
                    // 目前包括bigint、datetime
                    sqlBuilder.append(" " + fieldType + " NOT NULL,\n");
                }
            } else {
                if ("varchar".equals(fieldType)) {
                    sqlBuilder.append(" " + fieldType + "(" + fieldJSON.getLong("length") + ") DEFAULT NULL,\n");
                } else if ("tinyint".equals(fieldType)) {
                    sqlBuilder.append(" " + fieldType + "(1) DEFAULT NULL,\n");
                } else {
                    // 目前包括bigint、datetime
                    sqlBuilder.append(" " + fieldType + " DEFAULT NULL,\n");
                }
            }
        }
        sqlBuilder.append("PRIMARY KEY (id)\n");
        sqlBuilder.append(") ENGINE=InnoDB DEFAULT CHARSET=utf8;");
        this.execute(sqlBuilder.toString(), null);
    }

    public void dropTable(String instanceTable) throws Exception {
        String sql = "DROP TABLE " + instanceTable;
        this.execute(sql, null);
    }

    public int countTable(String instanceTable) throws Exception {
        String sql = "select count(1) from information_schema.TABLES where table_schema = (select database()) AND TABLE_NAME = :instanceTable;";
        return this.countQuery(sql, MapTools.simpleMap("instanceTable", instanceTable)).intValue();
    }

    public List<Map<String, Object>> list(Pager pager, String instanceTable, String instanceValueTable, Long userId, boolean hasAdminRole) throws Exception {
        String moduleId = CmdbUtils.getField(pager.getParams(), "moduleId");
        if (moduleId == null) {
            throw new Exception("查询失败");
        }
        String sql = buildSql(instanceTable, instanceValueTable, moduleId, userId, hasAdminRole);
        sql = "select a.* from (" + sql + ") a where 1=1 ";
        sql = getQueryBuilder().buildRaw(sql, pager, "a");
        Map<String, Object> paramMap = getQueryBuilder().getParam(pager.getParams());
        return this.list(sql, paramMap);
    }

    public List<Map<String, Object>> list(Pager pager, String instanceTable, String instanceValueTable, Long userId, boolean hasAdminRole, String positionKey) throws Exception {
        String moduleId = CmdbUtils.getField(pager.getParams(), "moduleId");
        if (moduleId == null) {
            throw new Exception("查询失败");
        }
        if (StringUtils.isNotEmpty(positionKey)) {
            instanceTable = positionKey + "." + instanceTable;
            instanceValueTable = positionKey + "." + instanceValueTable;
        }
        String sql = buildSql(instanceTable, instanceValueTable, moduleId, userId, hasAdminRole, positionKey);
        sql = "select a.* from (" + sql + ") a where 1=1 ";
        sql = getQueryBuilder().buildRaw(sql, pager, "a");
        Map<String, Object> paramMap = getQueryBuilder().getParam(pager.getParams());
        return this.list(sql, paramMap);
    }

    public int count(List<Param> params, String instanceTable, String instanceValueTable, Long userId, boolean hasAdminRole) throws Exception {
        String moduleId = CmdbUtils.getField(params, "moduleId");
        if (moduleId == null) {
            throw new Exception("查询失败");
        }
        String sql = buildSql(instanceTable, instanceValueTable, moduleId, userId, hasAdminRole);
        sql = "select count(1) from (" + sql + ") a where 1=1 ";
        sql = getQueryBuilder().buildRaw(sql, params, null, "a");
        Map<String, Object> param = getQueryBuilder().getParam(params);
        return this.countQuery(sql, param).intValue();
    }

    public int count(List<Param> params, String instanceTable, String instanceValueTable, Long userId, boolean hasAdminRole, String positionKey) throws Exception {
        String moduleId = CmdbUtils.getField(params, "moduleId");
        if (moduleId == null) {
            throw new Exception("查询失败");
        }
        if (StringUtils.isNotEmpty(positionKey)) {
            instanceTable = positionKey + "." + instanceTable;
            instanceValueTable = positionKey + "." + instanceValueTable;
        }
        String sql = buildSql(instanceTable, instanceValueTable, moduleId, userId, hasAdminRole, positionKey);
        sql = "select count(1) from (" + sql + ") a where 1=1 ";
        sql = getQueryBuilder().buildRaw(sql, params, null, "a");
        Map<String, Object> param = getQueryBuilder().getParam(params);
        return this.countQuery(sql, param).intValue();
    }

    public Instance queryByUuid(String instanceTable, String uuid, Long moduleId) throws Exception {
        String sql = "select a.* from " + instanceTable + " a where a.uuid = :uuid and module_id = :moduleId and is_deleted=0 ";
        Map<String, Object> params = MapTools.simpleMap(Common.UUID, uuid);
        params.put("moduleId", moduleId);
        List<Instance> list = this.list(Instance.class, sql, params);
        return list.isEmpty() ? null : list.get(0);
    }

    public List<Instance> queryByModuleId(String instanceTable, Long moduleId) throws Exception {
        String sql = "select a.* from " + instanceTable + " a where module_id = :moduleId and is_deleted=0 ";
        return this.list(Instance.class, sql, MapTools.simpleMap("moduleId", moduleId));
    }

    public Instance query(String instanceTable, Long id) throws Exception {
        String sql = "select a.* from " + instanceTable + " a where id = :id and is_deleted=0 ";
        List<Instance> list = this.list(Instance.class, sql, MapTools.simpleMap(Common.ID, id));
        return list.isEmpty() ? null : list.get(0);
    }

    public Instance query(String instanceTable, Long id, String positionKey) throws Exception {
        if (StringUtils.isNotEmpty(positionKey)) {
            instanceTable = positionKey + "." + instanceTable;
        }
        String sql = "select a.* from " + instanceTable + " a where id = :id and is_deleted=0 ";
        List<Instance> list = this.list(Instance.class, sql, MapTools.simpleMap(Common.ID, id));
        return list.isEmpty() ? null : list.get(0);
    }

    public boolean remove(Long id, String instanceTable, Long userId) throws Exception {
        String sql = "update " + instanceTable + " set is_deleted=1,gmt_modify = :gmtModify,mender_id=:userId where id = :id";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        params.put("gmtModify", new Date());
        params.put("userId", userId);
        return this.execute(sql, params) > 0;
    }

    private String buildSql(String instanceTable, String instanceValueTable, String moduleId, Long userId, boolean hasAdminRole) throws Exception {
        // 行转列的表头
        String sql = "SELECT GROUP_CONCAT('MAX(IF(P.ID = ',P.id,',INSV.PROPERTY_VALUE,NULL)) AS ''',P.code,'''') as sqlResult" + " from ci_property AS P where P.is_deleted = 0 and P.module_id=" + moduleId;

        List<Map<String, Object>> result = this.list(sql);
        if (result == null || result.isEmpty()) {
            throw new Exception("查询失败，模型无法使用");
        }
        String sqlResult = result.get(0).get("sqlResult").toString();

        StringBuilder sqlBuilder = new StringBuilder();
        sqlBuilder.append("SELECT INS.id,INS.module_id,INS.module_code,INS.gmt_create,INS.creator_id");
        if (sqlResult != null && !"".equals(sqlResult)) {
            sqlBuilder.append(",");
            sqlBuilder.append(sqlResult);
        }
        sqlBuilder.append(" FROM " + instanceTable + " INS");
        sqlBuilder.append(" LEFT JOIN " + instanceValueTable + " AS INSV ON INS.ID = INSV.instance_id and INSV.is_deleted=0 ");
        sqlBuilder.append(" LEFT JOIN ci_property AS P ON INS.module_id = P.module_id and INSV.property_id=P.id ");
        if (!hasAdminRole) {
            sqlBuilder.append(" LEFT JOIN ci_instance_authority AU on INS.module_id = AU.module_id and INS.id = AU.instance_id");
        }
        sqlBuilder.append(" where INS.is_deleted=0 and INS.uuid is not null and INS.module_id=" + moduleId);
        if (!hasAdminRole) {
            sqlBuilder.append(" and (( AU.auths like '%ACCESS%' and AU.user_id=" + userId + ") or INS.creator_id=" + userId + ")");
        }
        sqlBuilder.append(" GROUP BY INS.ID order by INS.id desc");
        return sqlBuilder.toString();
    }

    private String buildSql(String instanceTable, String instanceValueTable, String moduleId, Long userId, boolean hasAdminRole, String positionKey)  throws Exception {
        // 行转列的表头
        String sql;
        if (StringUtils.isNotEmpty(positionKey)) {
            sql = "SELECT GROUP_CONCAT('MAX(IF(P.ID = ',P.id,',INSV.PROPERTY_VALUE,NULL)) AS ''',P.code,'''') as sqlResult" + " from " +positionKey+"ci_property AS P where P.is_deleted = 0 and P.module_id=" + moduleId;
        } else {
            sql = "SELECT GROUP_CONCAT('MAX(IF(P.ID = ',P.id,',INSV.PROPERTY_VALUE,NULL)) AS ''',P.code,'''') as sqlResult" + " from ci_property AS P where P.is_deleted = 0 and P.module_id=" + moduleId;
        }

        List<Map<String, Object>> result = this.list(sql);
        if (result == null || result.isEmpty()) {
            throw new Exception("查询失败，模型无法使用");
        }
        String sqlResult = result.get(0).get("sqlResult").toString();

        StringBuilder sqlBuilder = new StringBuilder();
        sqlBuilder.append("SELECT INS.id,INS.module_id,INS.module_code,INS.gmt_create,INS.creator_id");
        if (sqlResult != null && !"".equals(sqlResult)) {
            sqlBuilder.append(",");
            sqlBuilder.append(sqlResult);
        }
        sqlBuilder.append(" FROM " + instanceTable + " INS");
        sqlBuilder.append(" LEFT JOIN " + instanceValueTable + " AS INSV ON INS.ID = INSV.instance_id and INSV.is_deleted=0 ");
        if (StringUtils.isNotEmpty(positionKey)) {
            sqlBuilder.append(" LEFT JOIN" +positionKey+" ci_property AS P ON INS.module_id = P.module_id and INSV.property_id=P.id ");
        } else {
            sqlBuilder.append(" LEFT JOIN ci_property AS P ON INS.module_id = P.module_id and INSV.property_id=P.id ");
        }
        if (!hasAdminRole) {
            if (StringUtils.isNotEmpty(positionKey)) {
                sqlBuilder.append(" LEFT JOIN" +positionKey+" ci_instance_authority AU on INS.module_id = AU.module_id and INS.id = AU.instance_id");
            } else {
                sqlBuilder.append(" LEFT JOIN ci_instance_authority AU on INS.module_id = AU.module_id and INS.id = AU.instance_id");
            }
        }
        sqlBuilder.append(" where INS.is_deleted=0 and INS.uuid is not null and INS.module_id=" + moduleId);
        if (!hasAdminRole) {
            sqlBuilder.append(" and (( AU.auths like '%ACCESS%' and AU.user_id=" + userId + ") or INS.creator_id=" + userId + ")");
        }
        sqlBuilder.append(" GROUP BY INS.ID order by INS.id desc");
        return sqlBuilder.toString();
    }

    public List<Map<String, Object>> listAll(String instanceTable, String instanceValueTable, Long moduleId, List<Long> ids, Long userId, boolean hasAdminRole) throws Exception {
        String sql = buildSql(instanceTable, instanceValueTable, String.valueOf(moduleId), userId, hasAdminRole);
        sql = "select a.* from (" + sql + ") a where 1=1 ";
        if (null != ids && !ids.isEmpty()) {
            sql += "and a.id in (:ids)";
        }
        return this.list(sql, MapTools.simpleMap("ids", ids));
    }

    public List<Map<String, Object>> listAll(String instanceTable, String instanceValueTable, Long moduleId, List<Long> ids, Long userId, boolean hasAdminRole, String positionKey) throws Exception {
        if (StringUtils.isNotEmpty(positionKey)) {
            instanceTable = positionKey + "." + instanceTable;
            instanceValueTable = positionKey + "." + instanceValueTable;
        }
        String sql = buildSql(instanceTable, instanceValueTable, String.valueOf(moduleId), userId, hasAdminRole, positionKey);
        sql = "select a.* from (" + sql + ") a where 1=1 ";
        if (null != ids && !ids.isEmpty()) {
            sql += "and a.id in (:ids)";
        }
        return this.list(sql, MapTools.simpleMap("ids", ids));
    }

    public List<Instance> queryByModuleId(String instanceTable, String instanceValueTable, Long moduleId, String propertyCode) throws Exception {
        String sql = "select a.id,a.uuid,a.module_id,a.module_code,b.property_value name from " + instanceTable + " a left join " + instanceValueTable + " b on a.id = b.instance_id " + " where a.module_id = :moduleId and b.property_code = :propertyCode and a.is_deleted=0 ";
        Map<String, Object> params = MapTools.simpleMap("moduleId", moduleId);
        params.put("propertyCode", propertyCode);
        return this.list(Instance.class, sql, params);
    }

    public List<Instance> queryList(String instanceTable, List<Long> ids) throws Exception {
        String sql = "select a.* from " + instanceTable + " a where id in (:ids) and is_deleted=0 ";
        return this.list(Instance.class, sql, MapTools.simpleMap("ids", ids));
    }

    public boolean batchRemove(List<Long> ids, String instanceTable, Long userId) throws Exception {
        String sql = "update " + instanceTable + " set is_deleted=1,gmt_modify = :gmtModify,mender_id=:userId where id in (:ids)";
        Map<String, Object> params = MapTools.simpleMap("ids", ids);
        params.put("gmtModify", new Date());
        params.put("userId", userId);
        return this.execute(sql, params) > 0;
    }

    public boolean deleteById(Long id, String instanceTable) throws Exception {
        String sql = "delete from " + instanceTable + " where id = :id";
        return this.execute(sql, MapTools.simpleMap(Common.ID, id)) > 0;
    }

}
