package com.bocloud.ams.repository.module;

import com.bocloud.ams.entity.module.ModuleRelation;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.BasicGenericDao;
import com.megatron.framework.core.CurrentService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @Description: 关联关系Dao
 * @Author: majiali
 * @CreateTime: 2022/5/7 14:03
 */
@Repository
public class ModuleRelationRepository extends BasicGenericDao<ModuleRelation, Long> {

    public ModuleRelationRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    public boolean delete(Long id, Long userId) throws Exception {
        String sql = "update ci_module_relation set is_deleted=1,gmt_modify = :gmtModify,mender_id=:userId where id = :id";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        params.put("gmtModify", new Date());
        params.put("userId", userId);
        return this.execute(sql, params) > 0;
    }

    public List<ModuleRelation> list(Pager pager) throws Exception {
        String sql = "select a.* from ci_module_relation a where a.is_deleted=0 ";
        sql = getQueryBuilder().buildRaw(sql, pager, "a");
        return this.list(ModuleRelation.class, sql, getQueryBuilder().getParam(pager.getParams()));
    }

    public int count(List<Param> params) throws Exception {
        String sql = "select count(1) from ci_module_relation a where a.is_deleted=0 ";
        sql = getQueryBuilder().buildRaw(sql, params, null, "a");
        return this.countQuery(sql, getQueryBuilder().getParam(params)).intValue();
    }

    public ModuleRelation query(Long id) throws Exception {
        String sql = "select a.*,sm.name source_name,tm.name target_name,c.name relation_category_name,CONCAT(a.source_type,'-',a.target_type) rule from ci_module_relation a " + " left join ci_module sm  on a.source_id =sm.id and sm.is_deleted =0 " + " left join ci_module tm  on a.target_id  =tm.id and tm.is_deleted =0 " + " left join ci_relation_category c  on a.relation_category_id  =c.id and c.is_deleted =0 " + " where a.id = :id and a.is_deleted=0 ";
        List<ModuleRelation> list = this.list(ModuleRelation.class, sql, MapTools.simpleMap(Common.ID, id));
        return list.isEmpty() ? null : list.get(0);
    }

    public ModuleRelation query(Long id, String positionKey) throws Exception {
        String sql;
        if (StringUtils.isNotBlank(positionKey)) {
            sql = "select a.*,sm.name source_name,tm.name target_name,c.name relation_category_name,CONCAT(a.source_type,'-',a.target_type) rule from " + positionKey + ".ci_module_relation a " + " left join " + positionKey + ".ci_module sm  on a.source_id =sm.id and sm.is_deleted =0 " + " left join " + positionKey + ".ci_module tm  on a.target_id  =tm.id and tm.is_deleted =0 " + " left join " + positionKey + ".ci_relation_category c  on a.relation_category_id  =c.id and c.is_deleted =0 " + " where a.id = :id and a.is_deleted=0 ";
        } else {
            sql = "select a.*,sm.name source_name,tm.name target_name,c.name relation_category_name,CONCAT(a.source_type,'-',a.target_type) rule from ci_module_relation a " + " left join ci_module sm  on a.source_id =sm.id and sm.is_deleted =0 " + " left join ci_module tm  on a.target_id  =tm.id and tm.is_deleted =0 " + " left join ci_relation_category c  on a.relation_category_id  =c.id and c.is_deleted =0 " + " where a.id = :id and a.is_deleted=0 ";
        }
        List<ModuleRelation> list = this.list(ModuleRelation.class, sql, MapTools.simpleMap(Common.ID, id));
        return list.isEmpty() ? null : list.get(0);
    }

    public ModuleRelation query(Long relationCategoryId, Long sourceModuleId, Long targetModuleId) throws Exception {
        String sql = "select a.* from ci_module_relation a where a.relation_category_id = :relationCategoryId and a.source_id = :sourceModuleId and a.target_id = :targetModuleId and is_deleted=0 ";
        Map<String, Object> params = MapTools.simpleMap("relationCategoryId", relationCategoryId);
        params.put("sourceModuleId", sourceModuleId);
        params.put("targetModuleId", targetModuleId);
        List<ModuleRelation> list = this.list(ModuleRelation.class, sql, params);
        return list.isEmpty() ? null : list.get(0);
    }

    public ModuleRelation query(Long relationCategoryId, Long sourceModuleId, Long targetModuleId, String positionKey) throws Exception {
        String sql;
        if (StringUtils.isNotEmpty(positionKey)) {
            sql = "select a.* from " + positionKey + ".ci_module_relation a where a.relation_category_id = :relationCategoryId and a.source_id = :sourceModuleId and a.target_id = :targetModuleId and is_deleted=0 ";
        } else {
            sql = "select a.* from ci_module_relation a where a.relation_category_id = :relationCategoryId and a.source_id = :sourceModuleId and a.target_id = :targetModuleId and is_deleted=0 ";
        }
        Map<String, Object> params = MapTools.simpleMap("relationCategoryId", relationCategoryId);
        params.put("sourceModuleId", sourceModuleId);
        params.put("targetModuleId", targetModuleId);
        List<ModuleRelation> list = this.list(ModuleRelation.class, sql, params);
        return list.isEmpty() ? null : list.get(0);
    }

    public List<ModuleRelation> list(List<Param> params, Map<String, String> sorter) throws Exception {
        String sql = "select a.* from ci_module_relation a where is_deleted=0";
        sql = getQueryBuilder().buildRaw(sql, new Pager(1, Integer.MAX_VALUE, params, sorter), "a");
        return this.list(ModuleRelation.class, sql, getQueryBuilder().getParam(params));
    }

    public List<ModuleRelation> list(Long moduleId, Long categoryId) throws Exception {
        String sql = "select a.*,b.name relation_name,b.code relation_code from ci_module_relation a join ci_relation b on a.relation_id=b.id where a.is_deleted = 0 and b.is_deleted=0 and a.relation_category_id = :categoryId" + " and a.source_id=:moduleId ";
        Map<String, Object> params = MapTools.simpleMap("categoryId", categoryId);
        params.put("moduleId", moduleId);
        return this.list(ModuleRelation.class, sql, params);
    }

    public List<ModuleRelation> list(Long moduleId, Long categoryId, String positionKey) throws Exception {
        String sql;
        if (StringUtils.isNotEmpty(positionKey)) {
            sql = "select a.*,b.name relation_name,b.code relation_code from " + positionKey + ".ci_module_relation a join " + positionKey + ".ci_relation b on a.relation_id=b.id where a.is_deleted = 0 and b.is_deleted=0 and a.relation_category_id = :categoryId" + " and a.source_id=:moduleId ";
        } else {
            sql = "select a.*,b.name relation_name,b.code relation_code from ci_module_relation a join ci_relation b on a.relation_id=b.id where a.is_deleted = 0 and b.is_deleted=0 and a.relation_category_id = :categoryId" + " and a.source_id=:moduleId ";
        }
        Map<String, Object> params = MapTools.simpleMap("categoryId", categoryId);
        params.put("moduleId", moduleId);
        return this.list(ModuleRelation.class, sql, params);
    }

    public List<ModuleRelation> listReverse(Long moduleId, Long categoryId) throws Exception {
        String sql = " select a.*,b.reverse_name relation_name,b.code relation_code,'1' is_reversed from ci_module_relation a join ci_relation b on a.relation_id=b.id where a.is_deleted = 0  and b.is_deleted=0 and a.relation_category_id = :categoryId " + " and a.target_id=:moduleId ";
        Map<String, Object> params = MapTools.simpleMap("categoryId", categoryId);
        params.put("moduleId", moduleId);
        return this.list(ModuleRelation.class, sql, params);
    }

    public List<ModuleRelation> listReverse(Long moduleId, Long categoryId, String positionKey) throws Exception {
        String sql;
        if (StringUtils.isNotEmpty(positionKey)) {
            sql = " select a.*,b.reverse_name relation_name,b.code relation_code,'1' is_reversed from " + positionKey + ".ci_module_relation a join " + positionKey + ".ci_relation b on a.relation_id=b.id where a.is_deleted = 0  and b.is_deleted=0 and a.relation_category_id = :categoryId " + " and a.target_id=:moduleId ";
        } else {
            sql = " select a.*,b.reverse_name relation_name,b.code relation_code,'1' is_reversed from ci_module_relation a join ci_relation b on a.relation_id=b.id where a.is_deleted = 0  and b.is_deleted=0 and a.relation_category_id = :categoryId " + " and a.target_id=:moduleId ";
        }
        Map<String, Object> params = MapTools.simpleMap("categoryId", categoryId);
        params.put("moduleId", moduleId);
        return this.list(ModuleRelation.class, sql, params);
    }

    public List<ModuleRelation> listByCategoryId(Long categoryId) throws Exception {
        String sql = "select a.*,b.name relation_name,b.code relation_code from ci_module_relation a join ci_relation b on a.relation_id=b.id where a.is_deleted = 0 and b.is_deleted=0 and a.relation_category_id = :categoryId";
        return this.list(ModuleRelation.class, sql, MapTools.simpleMap("categoryId", categoryId));
    }

    public List<ModuleRelation> listByCategoryId(Long categoryId, String positionKey) throws Exception {
        String sql;
        if (StringUtils.isNotEmpty(positionKey)) {
            sql = "select a.*,b.name relation_name,b.code relation_code from " + positionKey + ".ci_module_relation a join " + positionKey + ".ci_relation b on a.relation_id=b.id where a.is_deleted = 0 and b.is_deleted=0 and a.relation_category_id = :categoryId";
        } else {
            sql = "select a.*,b.name relation_name,b.code relation_code from ci_module_relation a join ci_relation b on a.relation_id=b.id where a.is_deleted = 0 and b.is_deleted=0 and a.relation_category_id = :categoryId";
        }
        return this.list(ModuleRelation.class, sql, MapTools.simpleMap("categoryId", categoryId));
    }

    public List<ModuleRelation> queryModuleRealtionAndModuleImage(Long categoryId) throws Exception {
        String sql = "SELECT cm.id as module_id,cm.icon_image, cmr.* FROM ci_module_relation cmr ,ci_module cm WHERE (cmr.source_id =cm.id or cmr .target_id =cm.id) and cmr.is_deleted =0 and cmr.relation_category_id = :categoryId ";
        return this.list(ModuleRelation.class, sql, MapTools.simpleMap("categoryId", categoryId));
    }

    public List<ModuleRelation> queryModuleRealtionAndModuleImage(Long categoryId, String positionKey) throws Exception {
        String sql;
        if (StringUtils.isNotEmpty(positionKey)) {
            sql = "SELECT cm.id as module_id,cm.icon_image, cmr.* FROM "+positionKey+".ci_module_relation cmr ,ci_module cm WHERE (cmr.source_id =cm.id or cmr .target_id =cm.id) and cmr.is_deleted =0 and cmr.relation_category_id = :categoryId ";
        } else {
            sql = "SELECT cm.id as module_id,cm.icon_image, cmr.* FROM ci_module_relation cmr ,ci_module cm WHERE (cmr.source_id =cm.id or cmr .target_id =cm.id) and cmr.is_deleted =0 and cmr.relation_category_id = :categoryId ";
        }
        return this.list(ModuleRelation.class, sql, MapTools.simpleMap("categoryId", categoryId));
    }

    public List<ModuleRelation> queryModuleList(Long categoryId) throws Exception {
        String sql = "select a.* from ci_module_relation a where a.is_deleted=0 and a.relation_category_id = :categoryId";
        return this.list(ModuleRelation.class, sql, MapTools.simpleMap("categoryId", categoryId));
    }

    public List<ModuleRelation> queryModuleList(Long categoryId, String positionKey) throws Exception {
        String sql;
        if (StringUtils.isNotEmpty(positionKey)) {
            sql = "select a.* from " + positionKey + ".ci_module_relation a where a.is_deleted=0 and a.relation_category_id = :categoryId";
        } else {
            sql = "select a.* from ci_module_relation a where a.is_deleted=0 and a.relation_category_id = :categoryId";
        }
        return this.list(ModuleRelation.class, sql, MapTools.simpleMap("categoryId", categoryId));
    }

    public List<ModuleRelation> queryByModuleId(Long moduleId) throws Exception {
        String sql = "select a.*, m1.name source_name,m2.name target_name from ci_module_relation a" + " left join ci_module m1 on m1.id=a.source_id " + " left join ci_module m2 on m2.id=a.target_id  " + " where a.is_deleted=0 and (a.source_id = :moduleId OR a.target_id = :moduleId) ";
        return this.list(ModuleRelation.class, sql, MapTools.simpleMap("moduleId", moduleId));
    }

    public List<ModuleRelation> query(Long relationCategoryId, Long moduleId) throws Exception {
        String sql = "select a.* ,r.name relation_name,r.reverse_name relation_reverse_name from ci_module_relation a left join ci_relation r on a.relation_id =r.id  where a.is_deleted=0 and a.relation_category_id = :relationCategoryId and (a.source_id = :moduleId OR a.target_id = :moduleId) ";
        Map<String, Object> params = MapTools.simpleMap("relationCategoryId", relationCategoryId);
        params.put("moduleId", moduleId);
        return this.list(ModuleRelation.class, sql, params);
    }

    public List<ModuleRelation> query(Long relationCategoryId, Long moduleId, String positionKey) throws Exception {
        String sql;
        if (StringUtils.isNotEmpty(positionKey)) {
            sql = "select a.* ,r.name relation_name,r.reverse_name relation_reverse_name from " + positionKey + ".ci_module_relation a left join " + positionKey + ".ci_relation r on a.relation_id =r.id  where a.is_deleted=0 and a.relation_category_id = :relationCategoryId and (a.source_id = :moduleId OR a.target_id = :moduleId) ";
        } else {
            sql = "select a.* ,r.name relation_name,r.reverse_name relation_reverse_name from ci_module_relation a left join ci_relation r on a.relation_id =r.id  where a.is_deleted=0 and a.relation_category_id = :relationCategoryId and (a.source_id = :moduleId OR a.target_id = :moduleId) ";
        }
        Map<String, Object> params = MapTools.simpleMap("relationCategoryId", relationCategoryId);
        params.put("moduleId", moduleId);
        return this.list(ModuleRelation.class, sql, params);
    }

    public boolean deleteByModuleId(Long moduleId, Long userId) throws Exception {
        String sql = " update ci_module_relation set is_deleted = 1 , gmt_modify = :gmtModify , mender_id = :userId where source_id = :moduleId or target_id = :moduleId ";
        Map<String, Object> params = MapTools.simpleMap("moduleId", moduleId);
        params.put("gmtModify", new Date());
        params.put("userId", userId);
        return this.execute(sql, params) > 0;
    }

    public List<ModuleRelation> listAll() throws Exception {
        String sql = "select a.*, m1.name source_name,m2.name target_name from ci_module_relation a" + " left join ci_module m1 on m1.id=a.source_id " + " left join ci_module m2 on m2.id=a.target_id where a.is_deleted=0 ";
        return this.list(ModuleRelation.class, sql);
    }

    public List<ModuleRelation> queryByRelationId(Long relationId) throws Exception {
        String sql = "select a.* from ci_module_relation a where a.is_deleted=0 and a.relation_id = :relationId ";
        return this.list(ModuleRelation.class, sql, MapTools.simpleMap("relationId", relationId));
    }

    public List<ModuleRelation> queryAll(Long relationCategoryId, Long moduleId) throws Exception {
        String sql = "select a.*,sb.name source_name ,rb.name target_name,r.name relation_name ,r.reverse_name relation_reverse_name  from ci_module_relation a,ci_module sb,ci_module rb,ci_relation r " + " where a.is_deleted=0  and a.source_id=sb.id and a.target_id =rb.id and a.relation_id =r.id and a.relation_category_id = :relationCategoryId and (a.source_id = :moduleId OR a.target_id = :moduleId) ";
        Map<String, Object> params = MapTools.simpleMap("relationCategoryId", relationCategoryId);
        params.put("moduleId", moduleId);
        return this.list(ModuleRelation.class, sql, params);
    }

    public List<ModuleRelation> queryAll(Long relationCategoryId, Long moduleId, String positionKey) throws Exception {
        String sql;
        if (StringUtils.isNotEmpty(positionKey)) {
            sql = "select a.*,sb.name source_name ,rb.name target_name,r.name relation_name ,r.reverse_name relation_reverse_name  from " + positionKey + ".ci_module_relation a," + positionKey + ".ci_module sb," + positionKey + ".ci_module rb," + positionKey + ".ci_relation r " + " where a.is_deleted=0  and a.source_id=sb.id and a.target_id =rb.id and a.relation_id =r.id and a.relation_category_id = :relationCategoryId and (a.source_id = :moduleId OR a.target_id = :moduleId) ";
        } else {
            sql = "select a.*,sb.name source_name ,rb.name target_name,r.name relation_name ,r.reverse_name relation_reverse_name  from ci_module_relation a,ci_module sb,ci_module rb,ci_relation r " + " where a.is_deleted=0  and a.source_id=sb.id and a.target_id =rb.id and a.relation_id =r.id and a.relation_category_id = :relationCategoryId and (a.source_id = :moduleId OR a.target_id = :moduleId) ";
        }
        Map<String, Object> params = MapTools.simpleMap("relationCategoryId", relationCategoryId);
        params.put("moduleId", moduleId);
        return this.list(ModuleRelation.class, sql, params);
    }

}
