package com.bocloud.ams.repository.module;

import com.bocloud.ams.entity.module.CiModule;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.BasicGenericDao;
import com.megatron.framework.core.CurrentService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @Author liuyuanyuan
 * @Date 2022/5/5 0005 14:47
 * @Description:
 */
@Repository
public class ModuleRepository extends BasicGenericDao<CiModule, Long> {

    public ModuleRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    public List<CiModule> list(Pager pager) throws Exception {
        String sql = "select a.* from ci_module a where a.is_deleted=0 ";
        sql = getQueryBuilder().buildRaw(sql, pager, "a");
        return this.list(CiModule.class, sql, getQueryBuilder().getParam(pager.getParams()));
    }

    public List<CiModule> list(Pager pager, String positionKey) throws Exception {
        String sql;
        if (StringUtils.isNotBlank(positionKey)) {
            sql = "select a.* from "+positionKey+".ci_module a where a.is_deleted=0 ";
        } else {
            sql = "select a.* from ci_module a where a.is_deleted=0 ";
        }
        sql = getQueryBuilder().buildRaw(sql, pager, "a");
        return this.list(CiModule.class, sql, getQueryBuilder().getParam(pager.getParams()));
    }

    public int count(List<Param> params) throws Exception {
        String sql = "select count(1) from ci_module a where a.is_deleted=0 ";
        sql = getQueryBuilder().buildRaw(sql, params, null, "a");
        return this.countQuery(sql, getQueryBuilder().getParam(params)).intValue();
    }

    public int count(List<Param> params, String positionKey) throws Exception {
        String sql;
        if (StringUtils.isNotEmpty(positionKey)) {
            sql = "select count(1) from " + positionKey + ".ci_module a where a.is_deleted=0 ";
        } else {
            sql = "select count(1) from ci_module a where a.is_deleted=0 ";
        }
        sql = getQueryBuilder().buildRaw(sql, params, null, "a");
        return this.countQuery(sql, getQueryBuilder().getParam(params)).intValue();
    }

    public boolean delete(Long id, Long userId) throws Exception {
        String sql = "update ci_module set is_deleted = 1, gmt_modify = :gmtModify, mender_id = :menderId where id = :id ";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        params.put("gmtModify", new Date());
        params.put("menderId", userId);
        return this.execute(sql, params) > 0;
    }

    public List<CiModule> existCollectionTemplate(String moduleName) throws Exception {
        String sql = "SELECT cm.* FROM ci_module cm WHERE cm.status = 'ENABLE' and cm.is_deleted = 0 AND EXISTS (SELECT * FROM ci_collection_template cct WHERE cct.is_deleted = 0 AND cm.id = cct.module_id) ";
        if (null != moduleName) {
            sql += " AND cm.name LIKE '%" + moduleName + "%'";
        }
        return this.list(CiModule.class, sql, null);
    }

    public List<CiModule> existCollectionTemplate(String moduleName, String positionKey) throws Exception {
        String sql;
        if (StringUtils.isNotEmpty(positionKey)) {
            sql = "SELECT cm.* FROM " + positionKey + ".ci_module cm WHERE cm.status = 'ENABLE' and cm.is_deleted = 0 AND EXISTS (SELECT * FROM " + positionKey + ".ci_collection_template cct WHERE cct.is_deleted = 0 AND cm.id = cct.module_id) ";
        } else {
            sql = "SELECT cm.* FROM ci_module cm WHERE cm.status = 'ENABLE' and cm.is_deleted = 0 AND EXISTS (SELECT * FROM ci_collection_template cct WHERE cct.is_deleted = 0 AND cm.id = cct.module_id) ";
        }
        if (null != moduleName) {
            sql += " AND cm.name LIKE '%" + moduleName + "%'";
        }
        return this.list(CiModule.class, sql, null);
    }

    public List<CiModule> existCollectionTask(String moduleName) throws Exception {
        String sql = "SELECT cm.* FROM ci_module cm WHERE cm.status = 'ENABLE' and cm.is_deleted = 0 AND EXISTS (SELECT * FROM ci_collection_task cct WHERE cct.is_deleted = 0 AND cm.id = cct.module_id) ";
        if (null != moduleName) {
            sql += " AND cm.name LIKE '%" + moduleName + "%'";
        }
        return this.list(CiModule.class, sql, null);
    }

    public List<CiModule> existCollectionTask(String moduleName, String positionKey) throws Exception {
        String sql;
        if (StringUtils.isNotEmpty(positionKey)) {
            sql = "SELECT cm.* FROM " + positionKey + ".ci_module cm WHERE cm.status = 'ENABLE' and cm.is_deleted = 0 AND EXISTS (SELECT * FROM " + positionKey + ".ci_collection_task cct WHERE cct.is_deleted = 0 AND cm.id = cct.module_id) ";
        } else {
            sql = "SELECT cm.* FROM ci_module cm WHERE cm.status = 'ENABLE' and cm.is_deleted = 0 AND EXISTS (SELECT * FROM ci_collection_task cct WHERE cct.is_deleted = 0 AND cm.id = cct.module_id) ";
        }
        if (null != moduleName) {
            sql += " AND cm.name LIKE '%" + moduleName + "%'";
        }
        return this.list(CiModule.class, sql, null);
    }

    public List<CiModule> all() throws Exception {
        String sql = "select * from ci_module where is_deleted=0 ";
        return this.list(CiModule.class, sql, null);
    }

    public List<CiModule> all(String positionKey) throws Exception {
        String sql;
        if (StringUtils.isNotBlank(positionKey)) {
            sql = "select * from " + positionKey + ".ci_module where is_deleted=0 ";
        } else {
            sql = "select * from ci_module where is_deleted=0 ";
        }
        return this.list(CiModule.class, sql, null);
    }

    public List<CiModule> queryByGroupId(Long groupId) throws Exception {
        String sql = "select * from ci_module where is_deleted=0 and module_group_id = :groupId ORDER BY gmt_create DESC";
        return this.list(CiModule.class, sql, MapTools.simpleMap("groupId", groupId));
    }

    public CiModule query(Long id) throws Exception {
        String sql = " select a.* from ci_module a where a.id = :id and a.is_deleted=0 ";
        List<CiModule> list = this.list(CiModule.class, sql, MapTools.simpleMap(Common.ID, id));
        return list.isEmpty() ? null : list.get(0);
    }

    public CiModule query(Long id, String positionKey) throws Exception {
        String sql;
        if (StringUtils.isEmpty(positionKey)) {
            sql = " select a.* from ci_module a where a.id = :id and a.is_deleted=0 ";
        } else {
            sql = " select a.* from " + positionKey + ".ci_module a where a.id = :id and a.is_deleted=0 ";
        }
        List<CiModule> list = this.list(CiModule.class, sql, MapTools.simpleMap(Common.ID, id));
        return list.isEmpty() ? null : list.get(0);
    }

    public int countSameNameOrCode(String name, String code) throws Exception {
        String sql = "select count(1) from ci_module where is_deleted=0 ";
        if (null != code) {
            sql += " and code = :code ";
        } else if (null != name) {
            sql += " and name = :name ";
        }
        Map<String, Object> params = MapTools.simpleMap("code", code);
        params.put("name", name);
        return this.countQuery(sql, params).intValue();
    }

    public List<CiModule> listByIds(List<Long> ids) throws Exception {
        String sql = "select a.* from ci_module a where a.is_deleted = 0  and a.id in  (:ids) ";
        return this.list(CiModule.class, sql, MapTools.simpleMap("ids", ids));
    }

    public List<CiModule> listByIds(List<Long> ids, String positionKey) throws Exception {
        String sql;
        if (StringUtils.isNotEmpty(positionKey)) {
            sql = "select a.* from " + positionKey + ".ci_module a where a.is_deleted = 0  and a.id in  (:ids) ";
        } else {
            sql = "select a.* from ci_module a where a.is_deleted = 0  and a.id in  (:ids) ";
        }
        return this.list(CiModule.class, sql, MapTools.simpleMap("ids", ids));
    }

    /**
     * order by field (id, :ids)按照查询的ids顺序返回数据
     *
     * @param ids
     * @return
     * @throws Exception
     */
    public List<CiModule> queryModuleListByIds(List<Long> ids) throws Exception {
        String sql = "select * from ci_module where is_deleted = 0 and id in  (:ids) order by field (id, :ids)";
        return this.list(CiModule.class, sql, MapTools.simpleMap("ids", ids));
    }

    public List<CiModule> queryModuleListByIds(List<Long> ids, String positionKey) throws Exception {
        String sql;
        if (StringUtils.isNotEmpty(positionKey)) {
            sql = "select * from " + positionKey + ".ci_module where is_deleted = 0 and id in  (:ids) order by field (id, :ids)";
        } else {
            sql = "select * from ci_module where is_deleted = 0 and id in  (:ids) order by field (id, :ids)";
        }
        return this.list(CiModule.class, sql, MapTools.simpleMap("ids", ids));
    }


    public CiModule queryByCode(String code) throws Exception {
        String sql = "select * from ci_module a where a.code = :code and is_deleted=0 ";
        List<CiModule> list = this.list(CiModule.class, sql, MapTools.simpleMap("code", code));
        return list.isEmpty() ? null : list.get(0);
    }

}
