package com.bocloud.ams.service.collection.impl;

import com.bocloud.ams.entity.collection.CollectionHistoryItem;
import com.bocloud.ams.repository.collection.CollectionHistoryItemRepository;
import com.bocloud.ams.service.collection.CollectionHistoryItemService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.GridHelper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Optional;

@Slf4j
@Service("collectionHistoryItemService")
public class CollectionHistoryItemServiceImpl implements CollectionHistoryItemService {
    @Autowired
    private CollectionHistoryItemRepository historyItemRepository;

    @Override
    public GeneralResult<GridBean<CollectionHistoryItem>> list(Pager pager, String positionKey) {
        try {
            pager.setParams(Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>()));
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>()));
            pager.getSorter().put(Common.ID, Common.ONE);
            int total = this.historyItemRepository.count(pager.getParams(), positionKey);
            List<CollectionHistoryItem> list = this.historyItemRepository.list(pager, positionKey);
            GridBean<CollectionHistoryItem> gridBean = GridHelper.getBean(pager.getPage(), pager.getRows(), total, list);
            return new GeneralResult<>(true, gridBean, "查询成功");
        } catch (Exception e) {
            log.error("Collection history item list fail:", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

    @Override
    public GeneralResult<GridBean<CollectionHistoryItem>> backupHistoryItems(Pager pager, String positionKey) {
        try {
            pager.setParams(Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>()));
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>()));
            pager.getSorter().put(Common.ID, Common.ONE);
            List<Param> params = pager.getParams();
            String tableName = null;
            for (Param param : params) {
                Map<String, Object> map = param.getParam();
                if(null != map && map.containsKey("tableName")){
                    tableName = map.get("tableName").toString();
                    map.remove("tableName");
                }
            }
            int total = this.historyItemRepository.count(tableName,pager.getParams(), positionKey);
            List<CollectionHistoryItem> list = this.historyItemRepository.list(tableName,pager);
            GridBean<CollectionHistoryItem> gridBean = GridHelper.getBean(pager.getPage(), pager.getRows(), total, list);
            return new GeneralResult<>(true, gridBean, "查询成功");
        } catch (Exception e) {
            log.error("Collection history item list fail:", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }
}
