package com.bocloud.ams.service.instance.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.TypeReference;
import com.bocloud.ams.entity.association.AutoRelationRule;
import com.bocloud.ams.entity.association.AutoRelationRuleItem;
import com.bocloud.ams.entity.association.AutoRelationRuleItemValue;
import com.bocloud.ams.entity.audit.OperationalAudit;
import com.bocloud.ams.entity.enums.InstanceEnum;
import com.bocloud.ams.entity.enums.OperationalAuditType;
import com.bocloud.ams.entity.instance.Instance;
import com.bocloud.ams.entity.instance.InstanceRelation;
import com.bocloud.ams.entity.instance.InstanceValue;
import com.bocloud.ams.entity.module.CiModule;
import com.bocloud.ams.entity.module.ModuleRelation;
import com.bocloud.ams.entity.module.RelationCategory;
import com.bocloud.ams.repository.association.AutoRelationRuleItemRepository;
import com.bocloud.ams.repository.association.AutoRelationRuleItemValueRepository;
import com.bocloud.ams.repository.association.AutoRelationRuleRepository;
import com.bocloud.ams.repository.audit.OperationalAuditRepository;
import com.bocloud.ams.repository.instance.InstanceRelationRepository;
import com.bocloud.ams.repository.instance.InstanceRepository;
import com.bocloud.ams.repository.instance.InstanceValueRepository;
import com.bocloud.ams.repository.module.*;
import com.bocloud.ams.repository.utils.CmdbUtils;
import com.bocloud.ams.service.audit.OperationalAuditService;
import com.bocloud.ams.service.instance.InstanceRelationHistoryRecordService;
import com.bocloud.ams.service.instance.InstanceRelationService;
import com.bocloud.ams.service.utils.*;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;

/**
 * @Author liuyuanyuan
 * @Date 2022/5/16 0016 14:25
 * @Description:
 */
@Slf4j
@Service("instanceRelationService")
public class InstanceRelationServiceImpl implements InstanceRelationService {

    @Autowired
    private LockFactory lockFactory;
    @Autowired
    private InstanceRelationRepository instanceRelationRepository;
    @Autowired
    private ModuleRelationRepository moduleRelationRepository;
    @Autowired
    private AutoRelationRuleRepository autoRelationRuleRepository;
    @Autowired
    private InstanceRepository instanceRepository;
    @Autowired
    private InstanceValueRepository instanceValueRepository;
    @Autowired
    private AutoRelationRuleItemRepository autoRelationRuleItemRepository;
    @Autowired
    private AutoRelationRuleItemValueRepository autoRelationRuleItemValueRepository;
    @Autowired
    private PropertyRepository propertyRepository;
    @Autowired
    private PropertyItemRepository propertyItemRepository;
    @Autowired
    private RepositoryHandler handler;
    @Autowired
    private InstanceHistoryHelper instanceHistoryHelper;
    @Autowired
    private OperationalAuditRepository operationalAuditRepository;
    @Autowired
    private OperationalAuditService operationalAuditService;
    @Autowired
    private RelationCategoryRepository relationCategoryRepository;
    @Autowired
    private InstanceRelationHistoryRecordService instanceRelationHistoryRecordService;
    @Autowired
    private ModuleRepository moduleRepository;

    @Override
    public GeneralResult<Void> createTable(String relationCategoryCode, Long userId) throws Exception {
        try {
            String relationTable = CmdbUtils.CMDB_INSTANCE_RELATION + relationCategoryCode.toLowerCase();
            if (instanceRelationRepository.countTable(relationTable) > 0) {
                log.error("创建CI实例关系表{}失败", relationTable);
                throw new Exception("创建CI实例关系表失败");
                // return new GeneralResult<>(false, "创建失败");
            }
            instanceRelationRepository.createTable(relationTable, new InstanceRelation());
            return new GeneralResult<>(true, "创建成功");
        } catch (Exception e) {
            log.error("create module instance table error:", e);
            throw new Exception("创建CI实例关系表失败");
            // return new GeneralResult<>(false, "创建失败");
        }
    }

    @Override
    public GeneralResult<Void> dropTable(String relationCategoryCode, Long userId) throws Exception {
        try {
            String relationTable = CmdbUtils.CMDB_INSTANCE_RELATION + relationCategoryCode.toLowerCase();
            if (instanceRelationRepository.countTable(relationTable) == 0) {
                log.warn("删除表空间{}失败", relationTable);
                // return new GeneralResult<>(false, "删除失败");
                throw new Exception("删除CI实例关系表失败");
            }
            instanceRelationRepository.dropTable(relationTable);
            return new GeneralResult<>(true, "删除成功");
        } catch (Exception e) {
            log.error("drop module instance table error:", e);
            throw new Exception("删除CI实例关系表失败");
            // return new GeneralResult<>(false, "删除失败");
        }
    }

    @Override
    public GeneralResult<Void> create(InstanceRelation instanceRelation, RequestContext context) {
        Long relationCategoryId = instanceRelation.getRelationCategoryId();
        Long moduleRelationId = instanceRelation.getModuleRelationId();

        Long sourceModuleId = instanceRelation.getSourceModuleId();
        String sourceModuleCode = instanceRelation.getSourceModuleCode();
        Long sourceInstanceId = instanceRelation.getSourceInstanceId();

        Long targetModuleId = instanceRelation.getTargetModuleId();
        Long targetInstanceId = instanceRelation.getTargetInstanceId();
        String targetModuleCode = instanceRelation.getTargetModuleCode();

        Long currentModuleId = instanceRelation.getCurrentModuleId();
        Long currentInstanceId = instanceRelation.getCurrentInstanceId();
        //判断是否为源
        boolean isSource = false;
        if ((currentModuleId + "-" + currentInstanceId).equals(sourceModuleId + "-" + sourceInstanceId)) {
            isSource = true;
        }
        instanceRelation.setWay(RelationHistoryConstant.Way.MANUAL.getValue());
        try {
            RelationCategory relationCategory = relationCategoryRepository.query(relationCategoryId);
            if (null == relationCategory) {
                instanceRelationHistoryRecordService.save(instanceRelation, isSource,
                        RelationHistoryConstant.Status.FAILED.name(), RelationHistoryConstant.Type.RELATION.name(),
                        "关系分组不存在", context);
                return new GeneralResult<>(false, "关系分组不存在");
            }
            instanceRelation.setRelationCategoryCode(relationCategory.getCode());

            ModuleRelation moduleRelation =
                    moduleRelationRepository.query(relationCategoryId, sourceModuleId, targetModuleId);
            if (null == moduleRelation) {
                instanceRelationHistoryRecordService.save(instanceRelation, isSource,
                        RelationHistoryConstant.Status.FAILED.name(), RelationHistoryConstant.Type.RELATION.name(),
                        "模型关系不存在", context);
                return new GeneralResult<>(false, "模型关系不存在");
            }
            instanceRelation.setModuleRelationName(moduleRelation.getName());
            //封装校验是否符合实例关系数据
            //源实例
            String sourceIntanceTable = CmdbUtils.CMDB_INSTANCE + sourceModuleCode.toLowerCase();
            String sourceIntanceValueTable = CmdbUtils.CMDB_INSTANCE_VALUE + sourceModuleCode.toLowerCase();
            Instance sourceInstance = instanceRepository.query(sourceIntanceTable, sourceInstanceId);
            List<InstanceValue> sourceInstanceValues = instanceValueRepository.listByInstanceId(sourceInstanceId, sourceIntanceValueTable);
            if (null == sourceInstance || CollectionUtils.isEmpty(sourceInstanceValues)) {
                return new GeneralResult<>(false, "模型关系中源实例不存在");
            }
            sourceInstance.setInstanceValues(sourceInstanceValues);
            for (InstanceValue instanceValue : sourceInstanceValues) {
                if (Common.NAME.equals(instanceValue.getPropertyCode())) {
                    instanceRelation.setSourceInstanceName(instanceValue.getPropertyValue());
                    break;
                }
            }
            //目标实例
            String targetIntanceTable = CmdbUtils.CMDB_INSTANCE + targetModuleCode.toLowerCase();
            String targetIntanceValueTable = CmdbUtils.CMDB_INSTANCE_VALUE + targetModuleCode.toLowerCase();
            Instance targetInstance = instanceRepository.query(targetIntanceTable, targetInstanceId);
            List<InstanceValue> targetnstanceValues = instanceValueRepository.listByInstanceId(targetInstanceId, targetIntanceValueTable);
            if (null == targetInstance || CollectionUtils.isEmpty(targetnstanceValues)) {
                return new GeneralResult<>(false, "模型关系中目标实例不存在");
            }
            targetInstance.setInstanceValues(targetnstanceValues);
            for (InstanceValue instanceValue : targetnstanceValues) {
                if (Common.NAME.equals(instanceValue.getPropertyCode())) {
                    instanceRelation.setTargetInstanceName(instanceValue.getPropertyValue());
                    break;
                }
            }
            // 检验是否满足模型关系
            String sourceType = moduleRelation.getSourceType();
            String targetType = moduleRelation.getTargetType();
            String relationTable = CmdbUtils.CMDB_INSTANCE_RELATION + relationCategory.getCode().toLowerCase();
            String instanceRelationType = sourceType + "-" + targetType;
            if (ModuleConstant.RELATION_TYPE_1.equals(sourceType)
                    && ModuleConstant.RELATION_TYPE_1.equals(targetType)) {
                InstanceRelation sourceInstanceRelation = instanceRelationRepository.querySource(relationTable, sourceModuleId, sourceInstanceId, targetModuleId);
                InstanceRelation targetInstanceRelation = instanceRelationRepository.queryTarget(relationTable, targetModuleId, targetInstanceId, sourceModuleId);
                if (null != sourceInstanceRelation || null != targetInstanceRelation) {
                    instanceRelationHistoryRecordService.save(instanceRelation, isSource,
                            RelationHistoryConstant.Status.FAILED.name(), RelationHistoryConstant.Type.RELATION.name(),
                            "模型实例已关联，模型关系" + instanceRelationType, context);
                    return new GeneralResult<>(false, "模型实例已关联，模型关系" + instanceRelationType);
                }

            } else if (ModuleConstant.RELATION_TYPE_N.equals(sourceType)
                    && ModuleConstant.RELATION_TYPE_1.equals(targetType)) {
                InstanceRelation sourceInstanceRelation =
                        instanceRelationRepository.querySource(relationTable, sourceModuleId, sourceInstanceId, targetModuleId);
                if (null != sourceInstanceRelation) {
                    instanceRelationHistoryRecordService.save(instanceRelation, isSource,
                            RelationHistoryConstant.Status.FAILED.name(), RelationHistoryConstant.Type.RELATION.name(),
                            "模型实例已关联，模型关系" + instanceRelationType, context);
                    return new GeneralResult<>(false, "模型实例已关联，模型关系" + instanceRelationType);
                }
            } else if (ModuleConstant.RELATION_TYPE_1.equals(sourceType)
                    && ModuleConstant.RELATION_TYPE_N.equals(targetType)) {
                InstanceRelation targetInstanceRelation =
                        instanceRelationRepository.queryTarget(relationTable, targetModuleId, targetInstanceId, sourceModuleId);
                if (null != targetInstanceRelation) {
                    instanceRelationHistoryRecordService.save(instanceRelation, isSource,
                            RelationHistoryConstant.Status.FAILED.name(), RelationHistoryConstant.Type.RELATION.name(),
                            "模型实例已关联，模型关系" + instanceRelationType, context);
                    return new GeneralResult<>(false, "模型实例已关联，模型关系" + instanceRelationType);
                }
            }
            // 判断模型关联规则
            AutoRelationRule rule = autoRelationRuleRepository.queryByModuleRelationId(moduleRelationId);
            // 当存在关联规则则需要验证规则对应项
            if (null != rule) {
                List<AutoRelationRuleItem> items = autoRelationRuleItemRepository.queryByRuleId(rule.getId());
                for (AutoRelationRuleItem item : items) {
                    // 需要查询对应的字段code及类型相关值
                    List<AutoRelationRuleItemValue> values =
                            autoRelationRuleItemValueRepository.queryAllByItemId(item.getId());
                    item.setValues(values);
                }
                rule.setItems(items);
                // 是否满足
                boolean isSatisfied = AutoModuleInstanceRelationUtils.isInstanceRelation(sourceInstance, targetInstance, rule);
                if (!isSatisfied) {
                    instanceRelationHistoryRecordService.save(instanceRelation, isSource,
                            RelationHistoryConstant.Status.FAILED.name(), RelationHistoryConstant.Type.RELATION.name(),
                            "模型实例关系不符合对应关联规则", context);
                    return new GeneralResult<>(false, "模型实例关系不符合对应关联规则");
                }
            }
            instanceRelation.setDeleted(false);
            instanceRelation.setCreatorId(context.getTarget());
            instanceRelation.setGmtCreate(new Date());
            handler.saveInstanceRelation(instanceRelation, relationTable);
            // 关联记录
            instanceRelationHistoryRecordService.save(instanceRelation, isSource,
                    RelationHistoryConstant.Status.SUCCESS.name(), RelationHistoryConstant.Type.RELATION.name(), "关联成功",
                    context);
            // 操作记录
            instanceHistoryHelper.history(instanceRelation, InstanceEnum.catalog.RELATION.name(),
                    InstanceEnum.category.CREATE.name(), context.getTarget());

            addOperationalAudit(isSource,instanceRelation,RelationHistoryConstant.Type.RELATION.name(),context.getTarget());
            return new GeneralResult<>(true, "关联成功");
        } catch (Exception e) {
            log.error("instance relation create fail:", e);
            // 关联记录
            instanceRelationHistoryRecordService.save(instanceRelation, isSource,
                    RelationHistoryConstant.Status.FAILED.name(), RelationHistoryConstant.Type.RELATION.name(),
                    e.getMessage(), context);
            return new GeneralResult<>(false, "关联异常");
        }
    }

    @Override
    public GeneralResult<Void> remove(Long id, String categoryCode, Long currentModuleId, Long currentInstanceId, RequestContext context) {
        String path = InstanceRelation.class.getSimpleName() + "_" + id;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            RelationCategory category = relationCategoryRepository.queryByCode(categoryCode);
            if (null == category) {
                return new GeneralResult<>(false, "关系分组不存在");
            }
            String relationTable = CmdbUtils.CMDB_INSTANCE_RELATION + categoryCode.toLowerCase();
            InstanceRelation relation = instanceRelationRepository.queryById(relationTable, id);
            if (null == relation) {
                return new GeneralResult<>(false, "模型实例关系不存在");
            }
            //手工取消
            relation.setWay(RelationHistoryConstant.Way.MANUAL.getValue());
            boolean isSource = (currentModuleId + "-" + currentInstanceId).equals(relation.getSourceModuleId() + "-" + relation.getSourceInstanceId());
            boolean result = instanceRelationRepository.remove(id, relationTable, context.getTarget());
            // 关联记录
            if (result) {
                instanceRelationHistoryRecordService.save(relation, isSource,
                        RelationHistoryConstant.Status.SUCCESS.name(), RelationHistoryConstant.Type.CANCEL.name(), "取消关联成功",
                        context);
            } else {
                instanceRelationHistoryRecordService.save(relation, isSource,
                        RelationHistoryConstant.Status.FAILED.name(), RelationHistoryConstant.Type.CANCEL.name(), "取消关联失败",
                        context);
            }
            instanceHistoryHelper.history(relation, InstanceEnum.catalog.RELATION.name(),
                    InstanceEnum.category.DEL.name(), context.getTarget());
            addOperationalAudit(isSource,relation,RelationHistoryConstant.Type.CANCEL.name(),context.getTarget());
            return new GeneralResult<>(true, "取消关联成功");
        } catch (Exception e) {
            log.error("instance relation remove fail:", e);
            return new GeneralResult<>(false, "取消关联异常");
        }
    }
    //增加审计日志
    private void addOperationalAudit(boolean isSource,InstanceRelation relation,String type,Long userId) {

        try {
            // 操作审计
            Map<String, Object> afterMap = null;
            Map<String, Object> beforeMap = null;
            OperationalAuditType auditType;
            OperationalAudit audit;
            CiModule module;
            //取消关联
            if (RelationHistoryConstant.Type.CANCEL.name().equals(type)) {
                auditType = OperationalAuditType.ASSOCIATION;
                beforeMap = JSON.parseObject(JSON.toJSONString(relation), new TypeReference<Map<String, Object>>() {
                });
            } else {
                auditType = OperationalAuditType.RELATION;
                afterMap = JSON.parseObject(JSON.toJSONString(relation), new TypeReference<Map<String, Object>>() {
                });

            }

            String moduleName = "";
            if (isSource) {
                module = moduleRepository.query(relation.getSourceModuleId());
                if (null != module) {
                    moduleName = module.getName();
                }
                audit = operationalAuditService.buildLog(auditType, moduleName,
                                relation.getSourceInstanceName(), relation.getSourceModuleCode(),
                                relation.getSourceInstanceId(), userId, beforeMap, afterMap);
            } else {
                module = moduleRepository.query(relation.getTargetModuleId());
                if (null != module) {
                    moduleName = module.getName();
                }
                audit = operationalAuditService.buildLog(auditType, moduleName,
                                relation.getTargetInstanceName(), relation.getTargetModuleCode(),
                                relation.getTargetInstanceId(), userId, beforeMap, afterMap);
            }

            operationalAuditRepository.save(audit);
        } catch (Exception e) {
            log.error("记录操作审计信息异常", e);
        }
    }

}
