package com.bocloud.ams.service.instance.impl;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.bocloud.ams.entity.instance.InstanceValue;
import com.bocloud.ams.repository.instance.InstanceValueRepository;
import com.bocloud.ams.repository.utils.CmdbUtils;
import com.bocloud.ams.service.instance.InstanceValueService;
import com.megatron.common.model.GeneralResult;

import lombok.extern.slf4j.Slf4j;

/**
 * @ClassName: CmdbInstanceValueServiceImpl
 * @Description
 * @Author yangzhenrong
 * @Date 2022/5/11 11:25
 * @Version 1.0
 **/
@Slf4j
@Service("instanceValueService")
public class InstanceValueServiceImpl implements InstanceValueService {

    @Autowired
    private InstanceValueRepository instanceValueRepository;

    @Override
    public GeneralResult<Void> createTable(String moduleCode, Long userId) {
        try {
            String valueTable = CmdbUtils.CMDB_INSTANCE_VALUE + moduleCode.toLowerCase();
            if (instanceValueRepository.countTable(valueTable) > 0) {
                log.warn("创建表空间{}失败", valueTable);
                return new GeneralResult<>(false, "创建失败");
            }
            instanceValueRepository.createTable(valueTable, new InstanceValue());
            return new GeneralResult<>(true, "创建成功");
        } catch (Exception e) {
            log.error("create module instance value table error:", e);
            return new GeneralResult<>(false, "创建失败");
        }
    }

    @Override
    public GeneralResult<Void> dropTable(String moduleCode, Long userId) {
        try {
            String valueTable = CmdbUtils.CMDB_INSTANCE_VALUE + moduleCode.toLowerCase();
            if (instanceValueRepository.countTable(valueTable) == 0) {
                log.warn("删除表空间{}失败", valueTable);
                return new GeneralResult<>(false, "删除失败");
            }
            instanceValueRepository.dropTable(valueTable);
            return new GeneralResult<>(true, "删除成功");
        } catch (Exception e) {
            log.error("drop module instance value table error:", e);
            return new GeneralResult<>(false, "删除失败");
        }
    }
}
