package com.bocloud.ams.service.internal;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.ams.entity.Role;
import com.bocloud.cmp.boot.model.BoCloudService;
import com.megatron.framework.core.Service;
import com.bocloud.cmp.boot.model.BocloudToken;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import com.megatron.framework.core.CurrentService;
import com.megatron.framework.core.Service;
import com.megatron.framework.http.core.HttpMethod;
import com.megatron.framework.http.core.ServiceFactory;
import com.megatron.framework.http.model.RemoteService;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Component
public class SmsInternalService implements InitializingBean {

    private final ServiceFactory serviceFactory;
    private final CurrentService currentService;

    @Autowired
    public SmsInternalService(ServiceFactory serviceFactory, CurrentService currentService) {
        this.serviceFactory = serviceFactory;
        this.currentService = currentService;
    }

    private static final Service SERVICE = Service.create(BoCloudService.SMS.getName());
    private final Map<String, Object> headers = new HashMap<>(4);

    public GeneralResult<JSONObject> detail(Long userId) {
        String url = "/v1/users/" + userId;
        RemoteService remoteService = serviceFactory.build(SERVICE, url, HttpMethod.GET, headers, null);
        GeneralResult<?> result = remoteService.invoke();
        if (result.isFailed()) {
            return new GeneralResult<>(false, result.getMessage());
        }
        String data = JSONObject.toJSONString(result.getData());
        return new GeneralResult<>(true, JSONObject.parseObject(data), "success");
    }

    public GeneralResult<List<Role>> getRolesByUserId(Long userId) {
        String url = "/v1/users/" + userId + "/roles";
        RemoteService remoteService = serviceFactory.build(SERVICE, url, HttpMethod.GET, headers, MapTools.simpleMap(Common.CONDITION, "cmdbQueryRoles"));
        return (GeneralResult<List<Role>>) remoteService.invoke();
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        this.headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
        this.headers.put("source", currentService.getService().getName());
    }
}
