package com.bocloud.ams.service.module;

import com.bocloud.ams.entity.model.ColumnAndSearchItemModel;
import com.bocloud.ams.entity.model.InstanceTopo;
import com.bocloud.ams.entity.model.PathModel;
import com.bocloud.ams.entity.model.ResponseInstanceTopologyModel;
import com.bocloud.ams.entity.module.TopologyConfig;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import jakarta.servlet.http.HttpServletResponse;

import java.util.List;
import java.util.Map;

/**
 * @author 胡立伟
 * @program: bocloud.ams
 * @description:
 * @date 2022/5/12 17:40
 */
public interface TopologyConfigService {

    /**
     * 列表分页查询
     *
     * @param pager
     * @return
     */
    GeneralResult<GridBean<TopologyConfig>> list(Pager pager, String positionKey);

    /**
     * 新增模型
     *
     * @param config
     * @param target
     * @return
     */
    GeneralResult<Void> save(TopologyConfig config, Long target);

    /**
     * 更新模型
     *
     * @param config
     * @param target
     * @return
     */
    GeneralResult<Void> modify(TopologyConfig config, Long target);

    /**
     * 删除拓扑实例
     *
     * @param id
     * @param target
     * @return
     */
    GeneralResult<Void> remove(Long id, Long target);

    /**
     * 查询实例拓扑详情
     *
     * @param id
     * @return
     */
    GeneralResult<TopologyConfig> detail(Long id, String positionKey);

    /**
     * 根据起始模型查询最大拓扑层级
     *
     * @param categoryId
     * @param startModuleId
     * @return
     */
    GeneralResult<Integer> maxLevel(Long categoryId, Long startModuleId, String positionKey);

    /**
     * 获取拓扑路径
     *
     * @param categoryId
     * @param startModuleId
     * @param topologyLevel
     * @param ignoreModuleId
     * @param wayModuleId
     * @return
     */
    GeneralResult<List<PathModel>> topologyPath(Long categoryId, Long startModuleId, Integer topologyLevel,
                                                String ignoreModuleId, String wayModuleId, String positionKey);

    /**
     * 获取查询项以及列表项
     *
     * @param id
     * @return
     */
    GeneralResult<ColumnAndSearchItemModel> searchItemList(Long id, String positionKey);

    /**
     * 查询资源列表
     *
     * @param pager
     * @return
     */
    GeneralResult<GridBean<Map<String, Object>>> resourcelist(Pager pager, String positionKey);

    /**
     * 根据实例拓扑id查询对应起始模型的实例集合
     *
     * @param id
     * @return
     */
    GeneralResult<List<ResponseInstanceTopologyModel>> startModuleInstanceList(Long id, String positionKey);

    /**
     * R
     * 下载
     *
     * @param response
     */
    GeneralResult<Void> download(Long id, HttpServletResponse response, String positionKey);

    /**
     * 实例拓扑-全局拓扑
     *
     * @param id
     * @param instanceId
     * @return
     */
    GeneralResult<InstanceTopo> topoList(Long id, Long instanceId, String positionKey);

    /**
     * 实例资源拓扑
     *
     * @param id
     * @param instanceId
     * @return
     */
    GeneralResult<InstanceTopo> topology(Long id, Long instanceId, String positionKey);

    /**
     * 实例拓扑-根据实例id查找下一层级实例拓扑
     *
     * @param id
     * @param instanceId
     * @return
     */
    GeneralResult<InstanceTopo> nextTopoList(Long id, Long moduleId, Long instanceId, String positionKey);

}
