package com.bocloud.ams.service.module.impl;

import com.bocloud.ams.entity.module.CiModule;
import com.bocloud.ams.entity.module.ModuleGroup;
import com.bocloud.ams.repository.instance.InstanceRepository;
import com.bocloud.ams.repository.module.ModuleGroupRepository;
import com.bocloud.ams.repository.module.ModuleRepository;
import com.bocloud.ams.repository.utils.CmdbUtils;
import com.bocloud.ams.service.instance.InstanceService;
import com.bocloud.ams.service.module.ModuleGroupService;
import com.bocloud.ams.service.utils.ModuleConstant;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.Param;
import com.megatron.common.model.RequestContext;
import com.megatron.common.model.Sign;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.concurrent.Callable;
import java.util.concurrent.FutureTask;
import java.util.concurrent.TimeUnit;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * @Author 胡立伟
 * @Date 2022/4/29 0029 16:13
 * @Description:
 */
@Slf4j
@Service("moduleGroupService")
public class ModuleGroupServiceImpl implements ModuleGroupService {
    @Autowired
    private ModuleGroupRepository moduleGroupRepository;
    @Autowired
    private ModuleRepository moduleRepository;
    @Autowired
    private LockFactory lockFactory;
    @Autowired
    private InstanceService instanceService;
    @Autowired
    private InstanceRepository instanceRepository;
    @Autowired
    private ThreadPoolTaskExecutor threadPoolTaskExecutor;

    @Override
    public GeneralResult<List<ModuleGroup>> collections(String category, String moduleName, String positionKey) {
        try {
            // 获取模型分组所有数据
            List<ModuleGroup> groups = moduleGroupRepository.all(positionKey);
            if (CollectionUtils.isEmpty(groups)) {
                return new GeneralResult<>(true, groups, "查询成功");
            }
            List<ModuleGroup> parents =
                groups.stream().filter(item -> item.getParentId().equals(0L)).collect(Collectors.toList());
            // 对模型分组根据parentId进行分组
            Map<Long, List<ModuleGroup>> groupMap =
                groups.stream().collect(Collectors.groupingBy(ModuleGroup::getParentId));
            if ("template".equalsIgnoreCase(category)) {
                // 查询模型下存在采集模板的模型
                List<CiModule> template = moduleRepository.existCollectionTemplate(moduleName, positionKey);
                Map<Long, List<CiModule>> moduleMap =
                    template.stream().collect(Collectors.groupingBy(CiModule::getGroupId));
                this.getTree(parents, groupMap, moduleMap);
            } else if ("task".equalsIgnoreCase(category)) {
                // 查询模型下存在采集任务的模型
                List<CiModule> task = moduleRepository.existCollectionTask(moduleName, positionKey);
                Map<Long, List<CiModule>> moduleMap = task.stream().collect(Collectors.groupingBy(CiModule::getGroupId));
                this.getTree(parents, groupMap, moduleMap);
            }
            return new GeneralResult<>(true, filterData(parents), "查询成功");
        } catch (Exception e) {
            log.error("moduleGroupService collections fail:", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

    private List<ModuleGroup> filterData(List<ModuleGroup> parents) {
        List<ModuleGroup> groups = parents.stream().filter(group -> {
            return !(null == group.getChildrenGroups() && null == group.getModules());
        }).collect(Collectors.toList());
        return groups;
    }

    private Boolean getTree(List<ModuleGroup> parents, Map<Long, List<ModuleGroup>> groupMap,
        Map<Long, List<CiModule>> moduleMap) {
        Boolean exist = false;
        if (null == parents) {
            return false;
        }
        Iterator<ModuleGroup> iterator = parents.iterator();
        while (iterator.hasNext()) {
            ModuleGroup group = iterator.next();
            List<ModuleGroup> children = groupMap.get(group.getId());
            // 当前节点不存在下级节点及模型数据直接剔除
            if (moduleMap.containsKey(group.getId())) {
                group.setModules(moduleMap.get(group.getId()));
                exist = true;
            } else {
                if (null == children) {
                    iterator.remove();
                    continue;
                }
            }
            if (this.getTree(children, groupMap, moduleMap)) {
                group.setChildrenGroups(children);
                exist = true;
            }
        }
        return exist;
    }

    @Override
    public GeneralResult<List<ModuleGroup>> queryRoot(String positionKey) {
        try {
            List<ModuleGroup> rootGroups = moduleGroupRepository.queryRoot(positionKey);
            if (CollectionUtils.isEmpty(rootGroups)) {
                return new GeneralResult<>(true, rootGroups, "查询成功");
            }
            List<Long> rootGroupIds = rootGroups.stream().map(ModuleGroup::getId).collect(Collectors.toList());
            // 获取模型分组所有数据
            List<ModuleGroup> moduleGroups = moduleGroupRepository.all(positionKey);
            // 获取根节点的子节点集合
            List<ModuleGroup> children = moduleGroups.stream().filter(item -> rootGroupIds.contains(item.getParentId()))
                .collect(Collectors.toList());
            // 获取模型所有数据
            List<CiModule> modules = moduleRepository.all(positionKey);
            // 对模型根据groupId进行分组
            Map<Long, List<CiModule>> moduleMap = modules.stream().filter(item -> null != item.getGroupId())
                .collect(Collectors.groupingBy(CiModule::getGroupId));
            // 判断根节点是否包含子节点以及模型
            for (ModuleGroup moduleGroup : rootGroups) {
                List<ModuleGroup> childGroups = children.stream()
                    .filter(item -> item.getParentId().equals(moduleGroup.getId())).collect(Collectors.toList());
                List<CiModule> childMudules = moduleMap.get(moduleGroup.getId());
                if (CollectionUtils.isEmpty(childGroups) && CollectionUtils.isEmpty(childMudules)) {
                    moduleGroup.setExistNextData(false);
                } else {
                    moduleGroup.setExistNextData(true);
                }
            }
            return new GeneralResult<>(true, rootGroups, "查询成功");
        } catch (Exception e) {
            log.error("moduleGroupService queryRoot fail:", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

    @Override
    public GeneralResult<List<ModuleGroup>> queryRootChildren(String positionKey) {
        try {
            // 获取模型分组所有数据
            List<ModuleGroup> groups = moduleGroupRepository.all(positionKey);
            List<ModuleGroup> parent =
                groups.stream().filter(item -> item.getParentId().equals(0L)).collect(Collectors.toList());
            // 对模型分组根据parentId进行分组
            Map<Long, List<ModuleGroup>> groupMap =
                groups.stream().collect(Collectors.groupingBy(ModuleGroup::getParentId));
            if (CollectionUtils.isNotEmpty(parent)) {
                this.getRootTree(parent, groupMap);
            }
            return new GeneralResult<>(true, parent, "查询成功");
        } catch (Exception e) {
            log.error("moduleGroupService queryRootChildren fail:", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

    @Override
    @Transactional(rollbackFor = {Exception.class})
    public GeneralResult<Void> save(ModuleGroup group, Long target) {
        try {
            if ("其他".equals(group.getName())) {
                return new GeneralResult<>(false, "添加失败，当前分组名称为非法名称，请尝试其他名称");
            }
            int sameName = moduleGroupRepository.countSameNameOrCode(group.getName(), null);
            if (sameName > 0) {
                return new GeneralResult<>(false, "添加失败，该名称已存在，请尝试其他名称");
            }
            int sameCode = moduleGroupRepository.countSameNameOrCode(null, group.getCode());
            if (sameCode > 0) {
                return new GeneralResult<>(false, "添加失败，该编码已存在，请尝试其他编码");
            }
            if (null == group.getParentId()) {
                group.setParentId(0L);
            }
            group.setGmtCreate(new Date());
            group.setCreatorId(target);
            group.setMenderId(target);
            moduleGroupRepository.save(group);
            return new GeneralResult<>(true, "创建成功");
        } catch (Exception e) {
            log.error("moduleGroupService save fail:", e);
            return new GeneralResult<>(false, "创建失败");
        }
    }

    @Override
    public GeneralResult<ModuleGroup> detail(Long id, String positionKey) {
        try {
            ModuleGroup group = moduleGroupRepository.query(id, positionKey);
            return new GeneralResult<>(true, group, "查询成功");
        } catch (Exception e) {
            log.error("moduleGroupService detail fail:", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

    @Override
    @Transactional(rollbackFor = {Exception.class})
    public GeneralResult<Void> modify(ModuleGroup group, Long target) {
        String path = ModuleGroup.class.getSimpleName() + "_" + group.getId();
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            ModuleGroup moduleGroup = moduleGroupRepository.query(group.getId());
            if (null == moduleGroup) {
                return new GeneralResult<>(false, "模型分组数据不存在");
            }
            if (!moduleGroup.getName().equals(group.getName())) {
                int sameName = moduleGroupRepository.countSameNameOrCode(group.getName(), null);
                if (sameName > 0) {
                    return new GeneralResult<>(false, "更新失败，该名称已存在，请尝试其他名称");
                }
            }
            if (!moduleGroup.getCode().equals(group.getCode())) {
                int sameCode = moduleGroupRepository.countSameNameOrCode(null, group.getCode());
                if (sameCode > 0) {
                    return new GeneralResult<>(false, "更新失败，该编码已存在，请尝试其他编码");
                }
            }
            if (!group.getParentId().equals(moduleGroup.getParentId())) {
                List<ModuleGroup> groups = moduleGroupRepository.queryChildren(group.getId());
                List<CiModule> modules = moduleRepository.queryByGroupId(group.getId());
                if (CollectionUtils.isNotEmpty(groups) || CollectionUtils.isNotEmpty(modules)) {
                    return new GeneralResult<>(false, "当前分组存在子数据，不可移动！");
                }
            }
            if (null == group.getParentId()) {
                group.setParentId(0L);
            }
            BeanUtils.copyProperties(group, moduleGroup);
            moduleGroup.setGmtModify(new Date());
            moduleGroup.setMenderId(target);
            moduleGroupRepository.update(moduleGroup);
            return new GeneralResult<>(true, "更新成功");
        } catch (Exception e) {
            log.error("moduleGroupService modify fail:", e);
            return new GeneralResult<>(false, "更新失败");
        }
    }

    @Override
    public GeneralResult<ModuleGroup> queryChildren(Long groupId, String menu, RequestContext context, String positionKey) {
        try {
            // 获取模型分组所有数据
            List<ModuleGroup> groups = moduleGroupRepository.all(positionKey);
            ModuleGroup group = groups.stream().filter(item -> item.getId().equals(groupId)).findFirst().orElse(null);
            if (null == group) {
                return new GeneralResult<>(false, "不存在此分组！");
            }
            // 对模型分组根据parentId进行分组
            Map<Long, List<ModuleGroup>> groupMap =
                groups.stream().collect(Collectors.groupingBy(ModuleGroup::getParentId));
            // 获取模型所有数据
            List<CiModule> modules = moduleRepository.all(positionKey);
            if ("resource".equalsIgnoreCase(menu)) {
                modules = modules.stream().filter(item -> ModuleConstant.STATUS_ENABLE.equals(item.getStatus()))
                    .collect(Collectors.toList());
            }
            // 对模型根据groupId进行分组
            Map<Long, List<CiModule>> moduleMap = modules.stream().collect(Collectors.groupingBy(CiModule::getGroupId));
            // 获取当前节点的子节点集合并判断子节点是否还包含数据
            List<ModuleGroup> moduleGroups = groupMap.get(groupId);
            if (CollectionUtils.isNotEmpty(moduleGroups)) {
                for (ModuleGroup moduleGroup : moduleGroups) {
                    List<ModuleGroup> childGroups = groupMap.get(moduleGroup.getId());
                    List<CiModule> childModules = moduleMap.get(moduleGroup.getId());
                    if (CollectionUtils.isEmpty(childGroups) && CollectionUtils.isEmpty(childModules)) {
                        moduleGroup.setExistNextData(false);
                    } else {
                        moduleGroup.setExistNextData(true);
                    }
                }
            }
            // 当前节点的模型集合
            List<CiModule> childModules = moduleMap.get(groupId);
            // 资源目录中查询模型对应的实例数量
            if ("resource".equalsIgnoreCase(menu)) {
                if (CollectionUtils.isNotEmpty(childModules)) {
                    countModuleInstanceNum(childModules, context, positionKey);
                }
            }
            group.setChildrenGroups(moduleGroups);
            group.setModules(childModules);
            if (CollectionUtils.isEmpty(moduleGroups) && CollectionUtils.isEmpty(childModules)) {
                group.setExistNextData(false);
            } else {
                group.setExistNextData(true);
            }
            return new GeneralResult<>(true, group, "查询成功");
        } catch (Exception e) {
            log.error("moduleGroupService queryChildren fail:", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

    private void countModuleInstanceNum(List<CiModule> childModules, RequestContext context, String positionKey) {

        log.info("--------------start count instance num---------");
        try {
            boolean hasAdminRole = instanceService.checkAdminRole(context.getTarget());
            List<FutureTask<Map<Long, Integer>>> futureTasks = new ArrayList<>();
            for (CiModule module : childModules) {
                FutureTask<Map<Long, Integer>> task =
                    new FutureTask<>(new CountInstanceNumTask(module, context.getTarget(), hasAdminRole, positionKey));
                futureTasks.add(task);
                threadPoolTaskExecutor.submit(task);
            }
            Map<Long, Integer> resultMap = new HashMap<>();
            for (FutureTask<Map<Long, Integer>> task : futureTasks) {
                resultMap.putAll(task.get());
            }
            for (CiModule module : childModules) {
                module.setInstanceNum(resultMap.get(module.getId()));
            }
        } catch (Exception e) {
            log.error("count instance num error", e);
        }
        log.info("--------------end  count instance num---------");
    }

    private class CountInstanceNumTask implements Callable<Map<Long, Integer>> {

        private CiModule module;
        private Long userId;
        private boolean hasAdminRole;
        private String positionKey;

        public CountInstanceNumTask(CiModule module, Long userId, boolean hasAdminRole, String positionKey) {
            this.module = module;
            this.userId = userId;
            this.hasAdminRole = hasAdminRole;
            this.positionKey = positionKey;
        }

        @Override
        public Map<Long, Integer> call() throws Exception {
            Map<Long, Integer> map = new HashMap<>();
            map.put(module.getId(), countInstanceNum(module, userId, hasAdminRole, positionKey));
            return map;
        }
    }

    private int countInstanceNum(CiModule module, Long userId, boolean hasAdminRole, String positionKey) {
        int total = 0;
        try {
            String table = CmdbUtils.CMDB_INSTANCE + module.getCode().toLowerCase();
            String valueTable = CmdbUtils.CMDB_INSTANCE_VALUE + module.getCode().toLowerCase();
            List<Param> params = new ArrayList<>();
            Map<String, Object> newMap = new HashMap<>();
            newMap.put("moduleId", module.getId());
            Param param = new Param();
            param.setParam(newMap);
            param.setSign(Sign.EQ);
            params.add(param);
            total = instanceRepository.count(params, table, valueTable, userId, hasAdminRole, positionKey);
        } catch (Exception e) {
            log.error("查询模型实例数量出错", e);
        }
        return total;
    }

    @Override
    @Transactional(rollbackFor = {Exception.class})
    public GeneralResult<Void> remove(Long id, Long userId) {
        String path = ModuleGroup.class.getSimpleName() + "_" + id;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            ModuleGroup moduleGroup = moduleGroupRepository.query(id);
            if (null == moduleGroup) {
                return new GeneralResult<>(false, "模型分组数据不存在");
            }
            List<ModuleGroup> groups = moduleGroupRepository.queryChildren(id);
            if (!groups.isEmpty()) {
                return new GeneralResult<>(false, "当前模型分组下存在子节点，拒绝删除");
            }
            List<CiModule> modules = moduleRepository.queryByGroupId(id);
            if (!modules.isEmpty()) {
                return new GeneralResult<>(false, "当前模型分组下存在模型数据，拒绝删除");
            }
            moduleGroupRepository.delete(id, userId);
            return new GeneralResult<>(true, "删除成功");
        } catch (Exception e) {
            log.error("moduleGroupService remove fail:", e);
            return new GeneralResult<>(false, "删除失败");
        }
    }

    /**
     * 新查询模型分组
     *
     * @return 返回各分组中模型数量
     */
    @Override
    public GeneralResult<List<ModuleGroup>> querylist(String positionKey) {
        try {
            // 获取根数据
            List<ModuleGroup> rootGroups = moduleGroupRepository.queryRoot(positionKey);
            if (CollectionUtils.isEmpty(rootGroups)) {
                return new GeneralResult<>(true, rootGroups, "查询成功");
            }
            // 获取模型分组所有数据
            List<ModuleGroup> moduleGroups = moduleGroupRepository.all(positionKey);
            // 对模型分组根据parentId进行分组
            Map<Long, List<ModuleGroup>> groupMap =
                moduleGroups.stream().collect(Collectors.groupingBy(ModuleGroup::getParentId));
            List<CiModule> modules = moduleRepository.all(positionKey);
            Map<Long, List<CiModule>> moduleMap = modules.stream().collect(Collectors.groupingBy(CiModule::getGroupId));
            for (ModuleGroup group : rootGroups) {
                this.getGroupTree(group, groupMap, moduleMap);
            }
            return new GeneralResult<>(true, rootGroups, "查询成功");
        } catch (Exception e) {
            log.error("moduleGroupService queryRoot fail:", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

    private void getRootTree(List<ModuleGroup> groups, Map<Long, List<ModuleGroup>> groupMap) {
        for (ModuleGroup group : groups) {
            List<ModuleGroup> children = groupMap.get(group.getId());
            if (CollectionUtils.isEmpty(children)) {
                continue;
            }
            group.setChildrenGroups(children);
            this.getRootTree(children, groupMap);
        }
    }

    private void getGroupTree(ModuleGroup group, Map<Long, List<ModuleGroup>> groupMap,
        Map<Long, List<CiModule>> moduleMap) {
        List<ModuleGroup> children = groupMap.get(group.getId());
        if (CollectionUtils.isNotEmpty(children)) {
            group.setChildrenGroups(children);
            for (ModuleGroup child : children) {
                this.getGroupTree(child, groupMap, moduleMap);
            }
        }
        List<CiModule> modules = moduleMap.get(group.getId());
        int moduleCounts = 0;
        if (CollectionUtils.isNotEmpty(modules)) {
            moduleCounts = modules.size();
        }
        int sum = 0;
        if (CollectionUtils.isNotEmpty(group.getChildrenGroups())) {
            for (ModuleGroup moduleGroup : group.getChildrenGroups()) {
                sum = sum + moduleGroup.getCount();
            }
        }
        group.setCount(moduleCounts + sum);
        ModuleGroup other = new ModuleGroup();
        other.setParentId(group.getId());
        other.setCode("!" + group.getCode() + group.getId());
        other.setName("其他");
        other.setCount(moduleCounts);
        if (CollectionUtils.isNotEmpty(group.getChildrenGroups())) {
            group.getChildrenGroups().add(other);
        } else {
            List<ModuleGroup> list = new ArrayList<>();
            list.add(other);
            group.setChildrenGroups(list);
        }
    }


    @Override
    public GeneralResult<List<ModuleGroup>> queryRootChildren(Long groupId , String menu, RequestContext context, String positionKey) {
        try {
            List<ModuleGroup> rootGroups = moduleGroupRepository.queryRoot(positionKey);
            if (CollectionUtils.isEmpty(rootGroups)) {
                return new GeneralResult<>(true, rootGroups, "查询成功");
            }
            // 获取模型分组所有数据
            List<ModuleGroup> groups = moduleGroupRepository.all(positionKey);
            // 获取模型所有数据
            List<CiModule> modules = moduleRepository.all(positionKey);
            // 对模型根据groupId进行分组
            Map<Long, List<CiModule>> moduleMap = modules.stream().filter(item -> null != item.getGroupId())
                    .collect(Collectors.groupingBy(CiModule::getGroupId));
            Map<Long, ModuleGroup> moduleGroupMap = groups.stream().collect(Collectors.toMap(ModuleGroup::getId, Function.identity()));
            //获取groupId对应的所有父节点id
            List<Long> parentIds = new ArrayList<>();
            getParentIds(parentIds, moduleGroupMap, groupId);
            // 对模型分发根据parentId进行分组
            Map<Long, List<ModuleGroup>> moduleGroupListMap = groups.stream().collect(Collectors.groupingBy(ModuleGroup::getParentId));
            // 获取模型所有数据
            if ("resource".equalsIgnoreCase(menu)) {
                modules = modules.stream().filter(item -> ModuleConstant.STATUS_ENABLE.equals(item.getStatus())).collect(Collectors.toList());
            }
            //当前模型分组及父分组下所有模型数据
            List<CiModule> newModules = modules.stream().filter(item -> parentIds.contains(item.getGroupId())).collect(Collectors.toList());
            //获取模型对应的实例数
            if ("resource".equalsIgnoreCase(menu)) {
                if (CollectionUtils.isNotEmpty(newModules)) {
                    countModuleInstanceNum(newModules, context, positionKey);
                }
            }
            // 对模型根据groupId进行分组
            Map<Long, List<CiModule>> newModuleMap = newModules.stream().collect(Collectors.groupingBy(CiModule::getGroupId));
            // 判断根节点是否包含子节点以及模型
            for (ModuleGroup moduleGroup : rootGroups) {
                List<ModuleGroup> childGroups = moduleGroupListMap.get(moduleGroup.getId());
                List<CiModule> childMudules = moduleMap.get(moduleGroup.getId());
                if (CollectionUtils.isEmpty(childGroups) && CollectionUtils.isEmpty(childMudules)) {
                    moduleGroup.setExistNextData(false);
                } else {
                    moduleGroup.setExistNextData(true);
                }
                if (parentIds.contains(moduleGroup.getId())) {
                    //moduleGroup 初始化模型及子分组
                    dealModuleGroup(parentIds, moduleGroup, moduleMap, newModuleMap, moduleGroupListMap);
                }
            }
            return new GeneralResult<>(true, rootGroups, "查询成功");
        } catch (Exception e) {
            log.error("moduleGroupService queryChildren fail:", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

    private void dealModuleGroup(List<Long> parentIds, ModuleGroup moduleGroup, Map<Long, List<CiModule>> moduleMap, Map<Long, List<CiModule>> newModuleMap, Map<Long, List<ModuleGroup>> moduleGroupListMap) {
        List<CiModule> listModule = moduleMap.get(moduleGroup.getId());
        List<ModuleGroup> listGroup = moduleGroupListMap.get(moduleGroup.getId());
        if (CollectionUtils.isEmpty(listModule) && CollectionUtils.isEmpty(listGroup)) {
            moduleGroup.setExistNextData(false);
        } else {
            moduleGroup.setExistNextData(true);
        }
        if (parentIds.contains(moduleGroup.getId())) {
            moduleGroup.setModules(newModuleMap.get(moduleGroup.getId()));
            moduleGroup.setChildrenGroups(listGroup);
            if (CollectionUtils.isEmpty(listGroup)) {
                return;
            }
            for (ModuleGroup group : listGroup) {
                dealModuleGroup(parentIds, group, moduleMap, newModuleMap, moduleGroupListMap);
            }
        }
    }

    //获取当前节点及其所有父节点的id
    private void getParentIds(List<Long> ids, Map<Long, ModuleGroup> moduleGroupMap, Long groupId) {
        ids.add(0, groupId);
        ModuleGroup group = moduleGroupMap.get(groupId);
        if (group.getParentId() == 0) {
            return;
        }
        getParentIds(ids, moduleGroupMap, group.getParentId());
    }

}
