package com.bocloud.ams.service.module.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.ams.entity.module.Property;
import com.bocloud.ams.entity.module.PropertyItem;
import com.bocloud.ams.entity.module.PropertyPool;
import com.bocloud.ams.entity.module.PropertyPoolItem;
import com.bocloud.ams.repository.module.PropertyGroupRepository;
import com.bocloud.ams.repository.module.PropertyItemRepository;
import com.bocloud.ams.repository.module.PropertyPoolItemRepository;
import com.bocloud.ams.repository.module.PropertyPoolRepository;
import com.bocloud.ams.repository.module.PropertyRepository;
import com.bocloud.ams.repository.module.RegularExpressionRepository;
import com.bocloud.ams.service.module.PropertyService;
import com.bocloud.ams.service.utils.PropertyConstant;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.utils.Common;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.beanutils.BeanUtilsBean;
import org.apache.commons.beanutils.locale.LocaleBeanUtils;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @Author 胡立伟
 * @Date 2022/5/9 0009 18:29
 * @Description:
 */
@Slf4j
@Service("propertyService")
public class PropertyServiceImpl implements PropertyService {

    @Autowired
    PropertyRepository propertyRepository;
    @Autowired
    PropertyItemRepository propertyItemRepository;
    @Autowired
    PropertyGroupRepository propertyGroupRepository;
    @Autowired
    private PropertyPoolRepository propertyPoolRepository;
    @Autowired
    private PropertyPoolItemRepository propertyPoolItemRepository;
    @Autowired
    RegularExpressionRepository regularExpressionRepository;

    @Override
    public GeneralResult<Property> detail(Long id, String positionKey) {
        try {
            Property property = propertyRepository.query(id, positionKey);
            if (null == property) {
                return new GeneralResult<>(false, "没有此模型");
            }
            List<PropertyItem> items = propertyItemRepository.queryByPropertyId(id, positionKey);
            this.propertyHandle(property, items, positionKey);
            return new GeneralResult<>(true, property, "查询成功");
        } catch (Exception e) {
            log.error("propertyService detail fail:", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

    @Override
    public void propertyHandle(Property property, List<PropertyItem> items, String positionKey) throws Exception {
        // 判断当前属性是否是引用池中的属性
        if (null != property.getPropertyPoolId()) {
            PropertyPool pool = propertyPoolRepository.query(property.getPropertyPoolId(), positionKey);
            this.copyProperties(property, pool);
            List<PropertyPoolItem> poolItems = propertyPoolItemRepository.listByPropertyPoolId(pool.getId(), positionKey);
            List<PropertyItem> propertyItems = new ArrayList<>();
            if (CollectionUtils.isNotEmpty(poolItems)) {
                for (PropertyPoolItem poolItem : poolItems) {
                    PropertyItem propertyItem = new PropertyItem();
                    LocaleBeanUtils.copyProperties(propertyItem, poolItem);
                    // 需要把当前属性的id赋值给copy过后的属性propertyItem
                    propertyItem.setPropertyId(property.getId());
                    if (CollectionUtils.isNotEmpty(poolItem.getItemList())) {
                        propertyItem.setItemList(JSONObject.parseArray(poolItem.getItemValue(), PropertyItem.class));
                    }
                    propertyItems.add(propertyItem);
                }
            }
            // 把从池表中引用的属性 转化成 自定义的属性处理
            this.handleProperty(property, propertyItems);
        } else {
            this.handleProperty(property, items);
        }
        if (property.getRegexpId() != null) {
            property.setRegularExpression(regularExpressionRepository.query(property.getRegexpId(), positionKey));
        }
    }

    private void handleProperty(Property property, List<PropertyItem> items) {
        // 如果是表格需要单独处理
        if (PropertyConstant.PropertyType.SINGLELINETABLE.equals(property.getType())
            || PropertyConstant.PropertyType.MULTILINETABLE.equals(property.getType())) {
            // 在是表格的前提下 如果是 引用已有模型的属性 还需要单独处理
            if (Common.ONE.equals(property.getTableType()) && null != property.getLinkModuleId()
                && StringUtils.isNotEmpty(property.getPropertyIds())) {
                List<Long> propertyIds = Arrays.asList(property.getPropertyIds().split(",")).stream()
                    .map(s -> Long.parseLong(s)).collect(Collectors.toList());
                List<Property> appointProperties = propertyRepository.appointPropertylist(propertyIds);
                List<PropertyItem> appointItems = new ArrayList<>();
                for (Property appointProperty : appointProperties) {
                    PropertyItem appointItem = new PropertyItem();
                    // todo
                    appointItem.setId(appointProperty.getId());
                    appointItem.setPropertyId(appointProperty.getId());
                    appointItem.setModuleId(appointProperty.getModuleId());
                    appointItem.setName(appointProperty.getName());
                    appointItem.setCode(appointProperty.getCode());
                    appointItem.setType(appointProperty.getType());
                    List<PropertyItem> propertyItems =
                        propertyItemRepository.queryByPropertyId(appointProperty.getId());
                    if (CollectionUtils.isNotEmpty(propertyItems)) {
                        appointItem.setItemList(propertyItems);
                    }
                    appointItems.add(appointItem);
                }
                property.setItemList(appointItems);
            } else {
                List<PropertyItem> propertyItems = items.stream()
                    .filter(item -> item.getPropertyId().equals(property.getId())).collect(Collectors.toList());
                for (PropertyItem propertyItem : propertyItems) {
                    if (null != propertyItem.getItemValue()) {
                        propertyItem.setItemList(JSON.parseArray(propertyItem.getItemValue(), PropertyItem.class));
                    }
                }
                property.setItemList(propertyItems);
            }
        } else {
            List<PropertyItem> propertyItems = items.stream()
                .filter(item -> item.getPropertyId().equals(property.getId())).collect(Collectors.toList());
            property.setItemList(propertyItems);
        }
    }

    private void copyProperties(Property property, PropertyPool pool)
        throws IllegalAccessException, InvocationTargetException {
        Long id = property.getId();
        BeanUtilsBean.getInstance().copyProperties(property, pool);
        property.setId(id);
    }

}
