package com.bocloud.ams.service.utils;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.ams.entity.collection.*;
import com.bocloud.ams.entity.enums.CollectionEnum;
import com.bocloud.ams.entity.instance.Instance;
import com.bocloud.ams.entity.instance.InstanceValue;
import com.bocloud.ams.entity.module.CiModule;
import com.bocloud.ams.entity.module.Property;
import com.bocloud.ams.repository.collection.CollectionComponentRepository;
import com.bocloud.ams.repository.collection.CollectionTaskInstanceRepository;
import com.bocloud.ams.repository.collection.CollectionTaskRepository;
import com.bocloud.ams.repository.collection.CollectionTemplateRepository;
import com.bocloud.ams.repository.instance.InstanceValueRepository;
import com.bocloud.ams.repository.module.ModuleRepository;
import com.bocloud.ams.repository.module.PropertyRepository;
import com.bocloud.ams.repository.utils.CmdbUtils;
import com.bocloud.ams.service.collection.CollectionHistoryService;
import com.bocloud.ams.service.instance.InstanceAutoRelationService;
import com.bocloud.ams.service.instance.InstanceService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.JSONTools;
import com.megatron.common.utils.ResultTools;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Component;

import java.util.*;

@Slf4j
@Component("collectionHistoryHelper")
public class CollectionHistoryHelper {
    @Autowired
    private CollectionTaskInstanceRepository taskInstanceRepository;
    @Autowired
    private CollectionTaskRepository taskRepository;
    @Autowired
    private CollectionTemplateRepository templateRepository;
    @Autowired
    private CollectionComponentRepository componentRepository;
    @Autowired
    private ModuleRepository moduleRepository;
    @Autowired
    private PropertyRepository propertyRepository;
    @Autowired
    private InstanceValueRepository instanceValueRepository;
    @Autowired
    private CollectionHistoryService historyService;
    @Autowired
    private InstanceService instanceService;
    @Autowired
    private RedisTemplate<String, String> redisTemplate;
    @Autowired
    private InstanceAutoRelationService instanceAutoRelationService;

    public GeneralResult history(Long taskInstanceId, String code, String output, Long operator) {
        String userName = redisTemplate.opsForValue().get("User_Name_" + operator);
        JSONObject object = JSONTools.isJSONObj(output);
        if (null != object) {
            try {
                CollectionTaskInstance taskInstance = taskInstanceRepository.query(taskInstanceId);
                if (null == taskInstance) {
                    return new GeneralResult(false, "采集任务实例不存在");
                }
                CollectionTask task = taskRepository.query(taskInstance.getTaskId());
                if (null == task) {
                    return new GeneralResult(false, "采集任务不存在");
                }
                CollectionTemplate template = templateRepository.query(task.getTemplateId());
                if (null == template) {
                    return new GeneralResult(false, "采集任务模板不存在");
                }
                CiModule module = moduleRepository.query(task.getModuleId());
                if (null == module) {
                    return new GeneralResult<>(false, "模型不存在");
                }
                //是否人工调和
                Boolean reconcile = task.getReconcile();
                //是否自动关联
                Boolean autoRelation = task.getAutoRelation();
                //查询模型属性
                List<Property> properties = propertyRepository.listNotHiddenByCmdbModuleId(task.getModuleId());
                Map<String, Property> propertyMap = new HashMap<>();
                for (Property property : properties) {
                    propertyMap.put(property.getCode(), property);
                }
                String instanceTable = CmdbUtils.CMDB_INSTANCE + module.getCode().toLowerCase();
                String instanceValueTable = CmdbUtils.CMDB_INSTANCE_VALUE + module.getCode().toLowerCase();
                //查询所有实例
                List<InstanceValue> instanceValues = instanceValueRepository.listAll(instanceValueTable);

                Map<Long, List<InstanceValue>> instanceMap = new HashMap();
                for (InstanceValue instanceValue : instanceValues) {
                    List<InstanceValue> instanceValueList = new ArrayList<>();
                    if (instanceMap.containsKey(instanceValue.getInstanceId())) {
                        instanceValueList = instanceMap.get(instanceValue.getInstanceId());
                    }
                    instanceValueList.add(instanceValue);
                    instanceMap.put(instanceValue.getInstanceId(), instanceValueList);
                }

                //属性对照
                List<CollectionComponent> components = componentRepository.queryByTemplateId(template.getId());
                Map<String, CollectionComponent> componentMap = new HashMap();
                Map<String, CollectionComponent> outMap = new HashMap();
                for (CollectionComponent component : components) {
                    if (CollectionEnum.parameter.CONTRAST.getValue().equals(component.getType())) {
                        componentMap.put(component.getName(), component);
                    }
                    if (CollectionEnum.parameter.OUTPUT.getValue().equals(component.getType())) {
                        componentMap.put(component.getName(), component);
                        outMap.put(component.getName(), component);
                    }
                }
                //输出参数为空，直接跳出
                if (outMap.isEmpty()) {
                    taskInstanceRepository.updateStatus(taskInstanceId, CollectionEnum.Status.SUCCESS.name());
                    return new GeneralResult(true, "保存成功");
                }
                if (CollectionEnum.workerCode.DATABASE.getValue().equals(code)) {
                    dealDatabaseData(taskInstanceId, object, reconcile, instanceMap, componentMap, propertyMap, module, userName, operator, autoRelation);
                } else if (CollectionEnum.workerCode.SCRIPT.getValue().equals(code)) {
                    dealScriptData(taskInstanceId, object, reconcile, instanceMap, componentMap, propertyMap, module, userName, operator, autoRelation);
                } else if (CollectionEnum.workerCode.METHOD.getValue().equals(code)) {
                    dealMethodData(taskInstanceId, object, reconcile, instanceMap, componentMap, propertyMap, module, userName, operator, autoRelation);
                }
                //剩下的实例属于未采集的
                for (Map.Entry<Long, List<InstanceValue>> instance : instanceMap.entrySet()) {
                    CollectionHistory history = new CollectionHistory();
                    history.setModuleId(module.getId());
                    history.setInstanceId(instance.getKey());
                    List<InstanceValue> instanceValuesList = instance.getValue();
                    for (InstanceValue value : instanceValuesList) {
                        if (Common.UUID.equals(value.getPropertyCode())) {
                            history.setInstanceId(instance.getKey());
                        }
                    }
                    history.setOperator(userName);
                    history.setGmtCreate(new Date());
                    history.setTaskInstanceId(taskInstanceId);
                    history.setType(CollectionEnum.historyType.NOTCOLLECTED.name());
                    history.setStatus(CollectionEnum.historyStatus.SUCCESS.name());
                    historyService.create(history);
                }
            } catch (Exception e) {
                log.error("采集记录保存失败：", e);
                return new GeneralResult(true, "保存失败");
            }
            return new GeneralResult(true, "保存成功");
        } else {
            return ResultTools.formatErrResult();
        }
    }

    private void dealDatabaseData(Long taskInstanceId, JSONObject object, Boolean reconcile, Map<Long, List<InstanceValue>> instanceMap, Map<String, CollectionComponent> componentMap, Map<String, Property> propertyMap, CiModule module, String userName, Long operator, Boolean autoRelation) throws Exception {
        boolean updateRequiredInstance = true;
        try {
            JSONArray dataArray = object.getJSONArray("data");
            for (int i = 0; i < dataArray.size(); i++) {
                List<JSONObject> dataList = JSONArray.parseArray(String.valueOf(dataArray.get(i)), JSONObject.class);
                dealData(taskInstanceId, dataList, reconcile, instanceMap, componentMap, propertyMap, module, userName, operator, autoRelation);
                updateRequiredInstance = false;
            }
        } catch (Exception e) {
            log.error("处理数据库数据异常:", e);
        } finally {
            if (updateRequiredInstance) {
                taskInstanceRepository.updateStatus(taskInstanceId, CollectionEnum.Status.ERROR.name());
            }
        }

    }

    /**
     * 检测字符出现的数量
     *
     * @param value
     * @param sub
     * @return
     */
    public static int times(String value, String sub) {
        int index;
        int len = sub.length();
        int times = 0;

        while (value.indexOf(sub) != -1) {
            index = value.indexOf(sub) + len;
            value = value.substring(index);
            times++;
        }
        return times;
    }

    private void dealScriptData(Long taskInstanceId, JSONObject object, Boolean reconcile, Map<Long, List<InstanceValue>> instanceMap, Map<String, CollectionComponent> componentMap, Map<String, Property> propertyMap, CiModule module, String userName, Long operator, Boolean autoRelation) throws Exception {
        boolean updateRequiredInstance = true;
        try {
            JSONArray dataArray = object.getJSONArray("data");
            List<JSONObject> dataList = new ArrayList<>();
            for (int i = 0; i < dataArray.size(); i++) {
                JSONObject dataObject = JSONObject.parseObject(JSON.toJSONString(dataArray.get(i)));
                if (dataObject.getBooleanValue("success")) {
                    String stdout = JSONObject.parseObject(dataObject.getString("message")).getString("stdout");
                    GeneralResult<JSONObject> stdoutObj = JSONTools.isJSON(stdout);
                    if (stdoutObj.isSuccess()) {
                        dataList.add(stdoutObj.getData());
                    } else {
                        String message = stdout.replaceAll("\\s*|\r|\n|\t", "");
                        boolean existJson = message.contains("[") || message.contains("{");
                        if (existJson) {
                            if(!message.endsWith(",")){
                                message +=",";
                            }
                            JSONObject result = new JSONObject();
                            //处理json的情况
                            String jsonMessage = message;
                            String oneMessage = message;
                            int times = times(message, "[");
                            for (int j = 1; j <= times; j++) {
                                String key = StringUtils.substringAfterLast(StringUtils.substringBefore(jsonMessage, ":["), ",");
                                if (StringUtils.isEmpty(key)) {
                                    continue;
                                }
                                String valueDate = StringUtils.substringBefore(StringUtils.substringAfter(jsonMessage, key + ":"), "],") + "]";
                                String otherReplaceData = key + ":" + valueDate;
                                GeneralResult jsonValue = JSONTools.isJSONArray(valueDate);
                                if (jsonValue.isSuccess()) {
                                    JSONArray value = JSONArray.parseArray(valueDate);
                                    result.put(key, value);
                                    jsonMessage = StringUtils.substringAfter(jsonMessage, otherReplaceData);
                                    oneMessage = oneMessage.replace(otherReplaceData, "").replaceAll(",,", ",");
                                }else {
                                    if (valueDate.startsWith("[") && valueDate.endsWith("]")) {
                                        try {
                                            result.put(key, JSONArray.parseArray(valueDate));
                                            jsonMessage = StringUtils.substringAfter(jsonMessage, otherReplaceData);
                                            oneMessage = oneMessage.replace(otherReplaceData, "").replaceAll(",,", ",");
                                        } catch (Exception e) {
                                            //字符串转数组出错，考虑是字符串中包含]、}]等问题引起的，需要特殊处理下
                                            //兼容字符串中包含]、}]字符
                                            // 包含"}]的还没有兼容，解析的时候还是会有问题
                                            valueDate = StringUtils.substringBefore(StringUtils.substringAfter(jsonMessage, key + ":"), "}],") + "}]";
                                            otherReplaceData = key + ":" + valueDate;
                                            jsonValue = JSONTools.isJSONArray(valueDate);
                                            if (jsonValue.isSuccess()) {
                                                result.put(key, jsonValue.getData());
                                                jsonMessage = StringUtils.substringAfter(jsonMessage, otherReplaceData);
                                                oneMessage = oneMessage.replace(otherReplaceData, "").replaceAll(",,", ",");
                                            } else {
                                                valueDate = StringUtils.substringBefore(StringUtils.substringAfter(jsonMessage, key + ":"), "\"}],") + "\"}]";
                                                otherReplaceData = key + ":" + valueDate;
                                                jsonValue = JSONTools.isJSONArray(valueDate);
                                                if (jsonValue.isSuccess()) {
                                                    result.put(key, jsonValue.getData());
                                                    jsonMessage = StringUtils.substringAfter(jsonMessage, otherReplaceData);
                                                    oneMessage = oneMessage.replace(otherReplaceData, "").replaceAll(",,", ",");
                                                }
                                            }
                                        }

                                    }
                                }

                            }

                            jsonMessage = oneMessage;
                            int timesObj = times(oneMessage, "{");
                            for (int j = 1; j <= timesObj; j++) {
                                String key = StringUtils.substringAfterLast(StringUtils.substringBefore(jsonMessage, ":{"), ",");
                                if (StringUtils.isEmpty(key)) {
                                    continue;
                                }
                                String valueDate = StringUtils.substringBefore(StringUtils.substringAfter(jsonMessage, key + ":"), "}") + "}";
                                String otherReplaceData = key + ":" + valueDate;
                                GeneralResult jsonValue = JSONTools.isJSON(valueDate);
                                if (jsonValue.isSuccess()) {
                                    JSONObject value = JSONObject.parseObject(valueDate);
                                    result.put(key, value);
                                    jsonMessage = StringUtils.substringAfter(jsonMessage, otherReplaceData);
                                    oneMessage = oneMessage.replace(otherReplaceData, "").replaceAll(",,", ",");
                                }

                            }

                            //处理1对1的情况
                            String[] oneDataSplit = oneMessage.split(",");
                            for (String msg : oneDataSplit) {
                                if (msg.contains(":")) {
                                    String[] msgSplit = msg.split(":");
                                    result.put(msgSplit[0], msg.substring(msgSplit[0].length() + 1));
                                }
                            }
                            dataList.add(result);
                        } else {
                            String[] messageSplit = message.split(",");
                            JSONObject data = new JSONObject();
                            for (String msg : messageSplit) {
                                if (msg.contains(":")) {
                                    String[] msgSplit = msg.split(":");
                                    data.put(msgSplit[0], msg.substring(msgSplit[0].length() + 1));
                                }
                            }
                            dataList.add(data);
                        }
                    }
                    updateRequiredInstance = false;
                }
            }
            dealData(taskInstanceId, dataList, reconcile, instanceMap, componentMap, propertyMap, module, userName, operator, autoRelation);
        } catch (Exception e) {
            log.error("处理脚本数据异常:", e);
        } finally {
            if (updateRequiredInstance) {
                taskInstanceRepository.updateStatus(taskInstanceId, CollectionEnum.Status.ERROR.name());
            }
        }
    }

    private void dealMethodData(Long taskInstanceId, JSONObject object, Boolean reconcile, Map<Long, List<InstanceValue>> instanceMap, Map<String, CollectionComponent> componentMap, Map<String, Property> propertyMap, CiModule module, String userName, Long operator, Boolean autoRelation) throws Exception {
        boolean updateRequiredInstance = true;
        try {
            JSONObject dataObject = object.getJSONObject("data");
            if (dataObject.getBooleanValue("success")) {
                List<JSONObject> dataList = new ArrayList<>();
                GeneralResult result = JSONTools.isJSON(dataObject.getString("data"));
                if (result.isSuccess()) {
                    JSONObject dataObj = dataObject.getJSONObject("data");
                    dataList.add(dataObj);
                } else {
                    dataList = JSONArray.parseArray(dataObject.getString("data"), JSONObject.class);
                }
                dealData(taskInstanceId, dataList, reconcile, instanceMap, componentMap, propertyMap, module, userName, operator, autoRelation);
                updateRequiredInstance = false;
            }
        } catch (Exception e) {
            log.error("处理接口数据异常:", e);
        } finally {
            if (updateRequiredInstance) {
                taskInstanceRepository.updateStatus(taskInstanceId, CollectionEnum.Status.ERROR.name());
            }
        }
    }

    private void dealData(Long taskInstanceId, List<JSONObject> dataList, Boolean reconcile, Map<Long, List<InstanceValue>> instanceMap, Map<String, CollectionComponent> componentMap, Map<String, Property> propertyMap, CiModule module, String userName, Long operator, Boolean autoRelation) {
        try {
            int successCount = 0;
            int failCount = 0;
            for (JSONObject data : dataList) {
                //转换成：属性:数据
                Map<String, Object> dataMap = new HashMap<>();
                Map<String, Object> contrastDataMap = new HashMap<>();
                Map<String, Object> outputDataMap = new HashMap<>();
                for (Map.Entry<String, Object> entry : data.entrySet()) {
                    if (componentMap.containsKey(entry.getKey())) {
                        CollectionComponent component = componentMap.get(entry.getKey());
                        if (CollectionEnum.parameter.CONTRAST.getValue().equals(component.getType())) {
                            contrastDataMap.put(component.getPropertyCode(), entry.getValue());
                        } else if (CollectionEnum.parameter.OUTPUT.getValue().equals(component.getType())) {
                            outputDataMap.put(component.getPropertyCode(), entry.getValue());
                        }
                        dataMap.put(component.getPropertyCode(), entry.getValue());
                    }
                }
                Map<String, Object> oldDataMap = new HashMap<>();
                //匹配实例数据
                Long instanceId = null;
                String instanceUuId = null;
                //存放当前实例属性值
                Map<String, Object> valueMap = new HashMap<>();
                for (Map.Entry<Long, List<InstanceValue>> instance : instanceMap.entrySet()) {
                    boolean isCurrent = true;
                    List<InstanceValue> instanceValues = instance.getValue();
                    valueMap = new HashMap<>();
                    for (InstanceValue value : instanceValues) {
                        valueMap.put(value.getPropertyCode(), value.getPropertyValue());
                    }
                    for (Map.Entry<String, Object> entry : contrastDataMap.entrySet()) {
                        if (!valueMap.containsKey(entry.getKey()) || (null != valueMap.get(entry.getKey()) && !valueMap.get(entry.getKey()).equals(contrastDataMap.get(entry.getKey())))) {
                            isCurrent = false;
                            break;
                        }
                    }
                    if (isCurrent) {
                        instanceId = instance.getKey();
                        //取出原属性值
                        oldDataMap = new HashMap<>();
                        List<InstanceValue> oldValues = instanceMap.get(instanceId);
                        for (InstanceValue value : oldValues) {
                            if (Common.UUID.equals(value.getPropertyCode())) {
                                instanceUuId = value.getPropertyValue();
                            }
                            if (dataMap.containsKey(value.getPropertyCode())) {
                                oldDataMap.put(value.getPropertyCode(), value.getPropertyValue());
                                //剔除替换的
                                valueMap.remove(value.getPropertyCode());
                            }
                        }
                        instanceMap.remove(instanceId);
                        break;
                    }
                }
                //封装实例数据
                JSONObject instanceObject = new JSONObject();
                instanceObject.put("moduleId", module.getId());
                instanceObject.put("moduleCode", module.getCode());

                CollectionHistory history = new CollectionHistory();
                history.setModuleId(module.getId());
                history.setOperator(userName);
                history.setGmtCreate(new Date());
                history.setTaskInstanceId(taskInstanceId);
                history.setOutput(JSON.toJSONString(data));
                if (null != instanceId && !oldDataMap.isEmpty()) {
                    history.setInstanceId(instanceId);
                    history.setInstanceUuid(instanceUuId);
                    history.setType(CollectionEnum.historyType.EDIT.name());
                    instanceObject.put(Common.ID, instanceId);
                    instanceObject.put(Common.UUID, instanceUuId);
                } else {
                    history.setType(CollectionEnum.historyType.ADD.name());
                }
                boolean isNeedUpdate = false;
                List<CollectionHistoryItem> items = new ArrayList<>();
                for (Map.Entry<String, Object> entry : dataMap.entrySet()) {
                    if (propertyMap.containsKey(entry.getKey())) {
                        Property property = propertyMap.get(entry.getKey());
                        CollectionHistoryItem item = new CollectionHistoryItem();
                        item.setPropertyId(property.getId());
                        item.setPropertyCode(property.getCode());
                        if (CollectionEnum.historyType.EDIT.name().equals(history.getType()) && null != oldDataMap.get(property.getCode())) {
                            item.setBeforeValue(String.valueOf(oldDataMap.get(property.getCode())));
                        } else if (CollectionEnum.historyType.ADD.name().equals(history.getType())) {
                            isNeedUpdate = true;
                        }
                        if (null != entry.getValue()) {
                            item.setAfterValue(String.valueOf(entry.getValue()));
                        }
                        if ((null != oldDataMap.get(property.getCode()) && null != entry.getValue() && !oldDataMap.get(property.getCode()).equals(entry.getValue())) || (null == oldDataMap.get(property.getCode()) && null != entry.getValue()) || (null != oldDataMap.get(property.getCode()) && null == entry.getValue())) {
                            isNeedUpdate = true;
                        }
                        items.add(item);

                        // instanceObject.put(property.getCode(), entry.getValue());
                        if ((PropertyConstant.PropertyType.CHECKBOX.equals(property.getType()) || PropertyConstant.PropertyType.MULTISELECT.equals(property.getType()) || PropertyConstant.PropertyType.JSON.equals(property.getType()) || PropertyConstant.PropertyType.SINGLELINETABLE.equals(property.getType()) || PropertyConstant.PropertyType.MULTILINETABLE.equals(property.getType())) && null != entry.getValue() && StringUtils.isNotEmpty(String.valueOf(entry.getValue()))) {
                            GeneralResult result = JSONTools.isJSON(String.valueOf(entry.getValue()));
                            if (result.isSuccess()) {
                                instanceObject.put(property.getCode(), JSONObject.parseObject(String.valueOf(entry.getValue())));
                            } else {
                               instanceObject.put(property.getCode(), JSONArray.parseArray(String.valueOf(entry.getValue())));
                            }
                        } else {
                            instanceObject.put(property.getCode(), entry.getValue());
                        }
                    }
                }
                if (!isNeedUpdate) {
                    history.setStatus(CollectionEnum.historyStatus.SUCCESS.name());
                    history.setType(CollectionEnum.historyType.NOUPDATE.name());
                    successCount += 1;
                }
                history.setHistoryItems(items);

                if (null != reconcile && !reconcile && isNeedUpdate) {
                    if (null != instanceId && !oldDataMap.isEmpty()) {
                        //未采集的属性值保留
                        for (Map.Entry<String, Object> entry : valueMap.entrySet()) {
                            if (propertyMap.containsKey(entry.getKey())) {
                                instanceObject.put(entry.getKey(), entry.getValue());
                            }
                        }
                        GeneralResult result = instanceService.modify(instanceObject, operator);
                        if (result.isSuccess()) {
                            history.setStatus(CollectionEnum.historyStatus.SUCCESS.name());
                            successCount += 1;
                            //自动关联
                            if (autoRelation) {
                                log.info("开启实例自动关联关系;");
                                try {
                                    Instance instance = (Instance) result.getData();
                                    instanceAutoRelationService.instanceAutoRelation(instance, operator);
                                } catch (Exception e) {
                                    log.error("采集任务中存在实例关系自动关联失败情况", e);
                                }
                            }
                        } else {
                            history.setStatus(CollectionEnum.historyStatus.FAILED.name());
                            history.setReason(result.getMessage());
                            failCount += 1;
                        }
                    } else {
                        //如果名称为空则自动生产
                        if (null == instanceObject.getString("name")) {
                            instanceObject.put("name", "AutomaticGeneration" + System.currentTimeMillis());
                        }
                        GeneralResult result = instanceService.create(instanceObject, operator, false);
                        if (result.isSuccess()) {
                            Instance instance = (Instance) result.getData();
                            history.setInstanceId(instance.getId());
                            history.setInstanceUuid(instance.getUuid());
                            history.setStatus(CollectionEnum.historyStatus.SUCCESS.name());
                            successCount += 1;
                            //自动关联
                            if (autoRelation) {
                                log.info("开启实例自动关联关系;");
                                try {
                                    instanceAutoRelationService.instanceAutoRelation(instance, operator);
                                } catch (Exception e) {
                                    log.error("采集任务中存在实例关系自动关联失败情况", e);
                                }
                            }
                        } else {
                            history.setStatus(CollectionEnum.historyStatus.FAILED.name());
                            history.setReason(result.getMessage());
                            failCount += 1;
                        }
                    }
                } else if ((null == reconcile || reconcile) && isNeedUpdate) {
                    history.setStatus(CollectionEnum.historyStatus.RECONCILED.name());
                }
                historyService.create(history);
            }
            //最后更新任务状态
            if (dataList.size() == successCount) {
                taskInstanceRepository.updateStatus(taskInstanceId, CollectionEnum.Status.SUCCESS.name());
            } else if (0 != failCount) {
                taskInstanceRepository.updateStatus(taskInstanceId, CollectionEnum.Status.ERROR.name());
            }
        } catch (Exception e) {
            log.error("处理数据异常:", e);
            try {
                taskInstanceRepository.updateStatus(taskInstanceId, CollectionEnum.Status.ERROR.name());
            }catch (Exception e1){
                log.error("更新实例状态异常:", e1);
            }

        }
    }
}
