package com.bocloud.ams.service.utils;

import com.bocloud.ams.entity.enums.InstanceEnum;
import com.bocloud.ams.entity.instance.Instance;
import com.bocloud.ams.entity.instance.InstanceHistory;
import com.bocloud.ams.entity.instance.InstanceRelation;
import com.bocloud.ams.entity.instance.InstanceValue;
import com.bocloud.ams.entity.module.CiModule;
import com.bocloud.ams.repository.instance.InstanceHistoryRepository;
import com.bocloud.ams.repository.instance.InstanceRelationRepository;
import com.bocloud.ams.repository.instance.InstanceValueRepository;
import com.bocloud.ams.repository.module.ModuleRepository;
import com.bocloud.ams.repository.utils.CmdbUtils;
import com.megatron.common.utils.Common;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Component;

import java.util.Date;

/**
 * @ClassName: CmdbInstanceHistoryHelper
 * @Description模型实例变更记录工具类
 * @Author yangzhenrong
 * @Date 2022/5/17 11:07
 * @Version 1.0
 **/
@Slf4j
@Component("instanceHistoryHelper")
public class InstanceHistoryHelper {
    @Autowired
    private InstanceValueRepository instanceValueRepository;
    @Autowired
    private InstanceHistoryRepository instanceHistoryRepository;
    @Autowired
    private InstanceRelationRepository instanceRelationRepository;
    @Autowired
    private ModuleRepository moduleRepository;
    @Autowired
    private RedisTemplate<String, String> redisTemplate;

    public void history(Object object, String catalog, String category, Long operator) {
        try {
            String userName = redisTemplate.opsForValue().get("User_Name_" + operator);
            StringBuffer content = new StringBuffer();
            InstanceEnum.category historyCategory = InstanceEnum.category.valueOf(category);
            Long instanceId = null;
            Long moduleId = null;
            if (InstanceEnum.catalog.INSTANCE.name().equals(catalog)) {
                Instance instance = (Instance) object;
                instanceId = instance.getId();
                moduleId = instance.getModuleId();
                CiModule module = moduleRepository.query(moduleId);
                if (null == module) {
                    log.error("模型不存在");
                    return;
                }
                String moduleName = module.getName();
                String instanceValueTable = CmdbUtils.CMDB_INSTANCE_VALUE + instance.getModuleCode().toLowerCase();
                InstanceValue instanceValue = instanceValueRepository.query(instanceValueTable, Common.NAME, instanceId);
                if (null == instanceValue) {
                    log.error("模型实例缺少关键属性[name]");
                    return;
                }
                String instanceName = instanceValue.getPropertyValue();
                switch (historyCategory) {
                    case CREATE:
                        content.append("新增实例:" + moduleName + "-" + instanceName);
                        break;
                    case MODIFY:
                        content.append("修改实例:" + moduleName + "-" + instanceName);
                        break;
                    case DEL:
                        content.append("删除实例:" + moduleName + "-" + instanceName);
                        break;
                    default:
                        break;
                }
            }
            if (InstanceEnum.catalog.RELATION.name().equals(catalog)) {
                InstanceRelation instanceRelation = (InstanceRelation) object;
                Long sourceInstanceId = instanceRelation.getSourceInstanceId();
                Long sourceModuleId = instanceRelation.getSourceModuleId();

                //源
                instanceId = sourceInstanceId;
                moduleId = sourceModuleId;

                Long targetInstanceId = instanceRelation.getTargetInstanceId();
                Long targetModuleId = instanceRelation.getTargetModuleId();
                CiModule sourceModule = moduleRepository.query(sourceModuleId);
                CiModule targetModule = moduleRepository.query(targetModuleId);
                if (null == sourceModule || null == targetModule) {
                    log.error("关联关系中的模型不存在");
                    return;
                }
                String sourceModuleName = sourceModule.getName();
                String targetModuleName = targetModule.getName();

                String sourceInstanceValueTable = CmdbUtils.CMDB_INSTANCE_VALUE + instanceRelation.getSourceModuleCode().toLowerCase();
                String targetInstanceValueTable = CmdbUtils.CMDB_INSTANCE_VALUE + instanceRelation.getTargetModuleCode().toLowerCase();
                InstanceValue sourceInstanceValue = instanceValueRepository.query(sourceInstanceValueTable, Common.NAME, sourceInstanceId);
                InstanceValue targetInstanceValue = instanceValueRepository.query(targetInstanceValueTable, Common.NAME, targetInstanceId);
                if (null == sourceInstanceValue || null == targetInstanceValue) {
                    log.error("模型实例关系的源实例或目标实例缺少关键属性[name]");
                    return;
                }
                String instanceRelationName = instanceRelation.getModuleRelationName();
                String sourceInstanceName = sourceInstanceValue.getPropertyValue();
                String targetInstanceName = targetInstanceValue.getPropertyValue();
                switch (historyCategory) {
                    case CREATE:
                        content.append("新增实例关系:[" + sourceModuleName + "-" + sourceInstanceName + "]与[" + targetModuleName + "-" + targetInstanceName + "]关联:[" + instanceRelationName + "]");
                        break;
                    case MODIFY:
                        content.append("修改实例关系:[" + sourceModuleName + "-" + sourceInstanceName + "]与[" + targetModuleName + "-" + targetInstanceName + "]关联:[" + instanceRelationName + "]");
                        break;
                    case DEL:
                        content.append("取消实例关系:[" + sourceModuleName + "-" + sourceInstanceName + "]与[" + targetModuleName + "-" + targetInstanceName + "]关联:[" + instanceRelationName + "]");
                        break;
                    default:
                        break;
                }
            }
            InstanceHistory instanceHistory = new InstanceHistory(moduleId, instanceId, catalog, category, content.toString(), userName, new Date());
            instanceHistoryRepository.save(instanceHistory);
        } catch (Exception e) {
            log.error("变更记录保存失败:", e);
        }
    }

    public InstanceHistory getInstanceHistory(Instance instance, CiModule module, InstanceValue instanceNameValue, String catalog, String category, Long operator) {

        InstanceHistory instanceHistory = null;
        try {
            String userName = redisTemplate.opsForValue().get("User_Name_" + operator);
            StringBuffer content = new StringBuffer();
            InstanceEnum.category historyCategory = InstanceEnum.category.valueOf(category);
            Long instanceId = instance.getId();
            Long moduleId = instance.getModuleId();
            String moduleName = module.getName();
            String instanceName = instanceNameValue.getPropertyValue();
            switch (historyCategory) {
                case CREATE:
                    content.append("新增实例:" + moduleName + "-" + instanceName);
                    break;
                case MODIFY:
                    content.append("修改实例:" + moduleName + "-" + instanceName);
                    break;
                case DEL:
                    content.append("删除实例:" + moduleName + "-" + instanceName);
                    break;
                case BATCH_MODIFY:
                    content.append("批量编辑:" + moduleName + "-" + instanceName);
                    break;
                case BATCH_DELETE:
                    content.append("批量删除:" + moduleName + "-" + instanceName);
                    break;
                default:
                    break;
            }
            instanceHistory = new InstanceHistory(moduleId, instanceId, catalog, category, content.toString(), userName, new Date());
        } catch (Exception e) {
            log.error("获取实例操作历史失败",e);
        }
        return instanceHistory;
    }

}
