package com.bocloud.ams.task.worker;

import com.bocloud.ams.task.annotation.Module;
import com.bocloud.ams.task.input.Input;
import com.megatron.common.locator.ClassLocator;
import lombok.extern.slf4j.Slf4j;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

/**
 * @ Description：前端输入类定位器，用来对Input的具体实现类基于code进行定位， @ Author ：丁明威 @ Date
 * ：Created in 16:49 2018/12/27 @ Modified By：
 */
@Slf4j
public class InputClassLocator {
    private static Map<String, Class<?>> inputClassMap = new ConcurrentHashMap<>();

    private List<Class<?>> getInputClasses(String code) {
        @SuppressWarnings("rawtypes")
        List<Class<?>> allClass = ClassLocator.getAllClass(Input.class.getPackage().getName(), "input");
        List<Class<?>> fixClass = new ArrayList<>();
        for (Class<?> clazz : allClass) {
            if (clazz.isAnnotationPresent(Module.class)) {
                Module module = (Module) clazz.getAnnotation(Module.class);
                if (module.code().equalsIgnoreCase(code)) {
                    fixClass.add(clazz);
                }
            }
        }
        return fixClass;
    }

    public Class<?> getInputClass(String code) {
        if (inputClassMap.containsKey(code)) {
            return inputClassMap.get(code);
        }
        List<Class<?>> inputClassList = getInputClasses(code);
        if (inputClassList.isEmpty()) {
            return null;
        }
        if (inputClassList.size() > 1) {
            log.error("more than one class found!");
            return null;
        }
        inputClassMap.put(code, inputClassList.get(0));
        return inputClassList.get(0);
    }
}