package com.bocloud.ams.task.worker;

import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;

import org.apache.commons.lang3.StringUtils;
import org.springframework.amqp.rabbit.core.RabbitAdmin;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.ams.task.annotation.Worker;
import com.bocloud.ams.task.exception.NodeFormatException;
import com.bocloud.ams.task.input.ScriptInput;
import com.bocloud.ams.task.input.Target;
import com.bocloud.ams.task.model.Option;
import com.bocloud.ams.task.model.ScriptFront;
import com.bocloud.ams.task.model.WorkerHost;
import com.bocloud.ams.task.model.WorkerScript;
import com.megatron.common.utils.ListTool;
import com.megatron.common.utils.MapTools;
import com.megatron.framework.http.core.ServiceFactory;

import lombok.extern.slf4j.Slf4j;

/**
 * @ Description： @ Author ：丁明威 @ Date ：Created in 17:46 2019/1/4 @ Modified By：
 */
@Slf4j
@Worker(name = "task.maintain.script")
public class ScriptWorker extends MaintainWorker {
    private ScriptInput input;

    /**
     * 构造方法
     *
     * @param code
     * @param rabbitAdmin
     * @param serviceFactory
     * @param timeout
     */
    public ScriptWorker(String code, RabbitAdmin rabbitAdmin, ServiceFactory serviceFactory, Integer timeout) {
        super(code, rabbitAdmin, serviceFactory, timeout);
    }

    @Override
    protected void init(Object param) throws NodeFormatException {
        if (param instanceof ScriptInput) {
            this.input = (ScriptInput)param;
            Target target = input.getTarget();
            // 目标主机切片
            burst(target);
            // 构建请求参数
            buildParam();
        } else {
            log.error("Script worker param illegal!");
            throw new NodeFormatException("Worker Param illegal!");
        }
    }

    /**
     * 参数构建
     */
    @Override
    void buildParam() {
        if (null == input) {
            return;
        }
        ScriptFront front = input.getFronts();
        String content = front.getContent();
        if (!StringUtils.isEmpty(front.getParams())) {
            List<JSONObject> params = JSONObject.parseArray(front.getParams(), JSONObject.class);
            if (!ListTool.isEmpty(params)) {
                for (JSONObject param : params) {
                    content = content.replaceAll("\\$\\{\\{" + param.getString("key") + "\\}\\}",
                        Matcher.quoteReplacement(param.getString("value")));
                }
            }
        }
        Target target = input.getTarget();
        if (null == target.getSudo()) {
            target.setSudo(false);
        }
        Option options = new Option(null, target.getSudo(), target.getBecomeUser(), target.getBecomePass(),
            target.getBecomeMethod());
        WorkerScript script = new WorkerScript(front.getCategory(), null, content, options);
        for (List<WorkerHost> hosts : this.targets) {
            Map<String, Object> param = MapTools.simpleMap("targets", hosts);
            param.put("script", script);
            param.put("response", "async");
            param.put("timeout", this.getTimeout());
            paramList.add(param);
        }
    }

}