package com.bocloud.ams.task.worker;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;

import org.springframework.amqp.rabbit.core.RabbitAdmin;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Repository;

import com.megatron.framework.http.core.ServiceFactory;
import com.megatron.framework.lock.LockFactory;

import lombok.extern.slf4j.Slf4j;

@Slf4j
@Repository
public class WorkerFactory {
    private final ServiceFactory serviceFactory;
    private final RabbitAdmin rabbitAdmin;
    private WorkerClassLocator classLocator;

    @Autowired
    public WorkerFactory(RabbitAdmin rabbitAdmin, StringRedisTemplate redisTemplate, ServiceFactory serviceFactory,
        LockFactory lockFactory) {
        this.rabbitAdmin = rabbitAdmin;
        this.serviceFactory = serviceFactory;
        classLocator = new WorkerClassLocator();
    }

    public AbstractWorker buildWorker(String code, Integer timeout) {
        Class<?> clazz = classLocator.getWorkerClass(code);
        if (null != clazz) {
            try {
                Constructor<?> constructor =
                    clazz.getDeclaredConstructor(String.class, RabbitAdmin.class, ServiceFactory.class, Integer.class);
                try {
                    return (AbstractWorker)constructor.newInstance(code, rabbitAdmin, serviceFactory, timeout);
                } catch (IllegalAccessException e) {
                    log.error("init worker {} of {} IllegalAccessException!", clazz, code, e);
                }
            } catch (NoSuchMethodException e) {
                log.error("init worker {} of {} NoSuchMethodException!", clazz, code, e);
            } catch (InstantiationException e) {
                log.error("init worker {} of {} InstantiationException!", clazz, code, e);
            } catch (InvocationTargetException e) {
                log.error("init worker {} of {} InvocationTargetException!", clazz, code, e);
            }
        }
        return null;
    }
}